//-------------------------------------------------------------------
//                                                                  -
// Module      : AccessBean.java                                    -
//                                                                  -
// Description : CASP for WebSphere Acess Outbound. EJB business    -
//               method call to CASP Business Service request       -
//               access sample mapping class                        -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Copyright Candle Corporation 2001,2003                           -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Details:                                                         -
//   The delivery of this toolkit includes a worked sample which    -
//   provides a example User EJB which invokes a CASP Business      -
//   Service via business methods called "getProductDetails()"      -
//   or "buyProduct()"                                              -
//   This sample Access Bean class maps these business methods into -
//   appropriate CASP SOAP request or datagram to be passed         -
//   to the KxoOutbound Bean                                        -
//                                                                  -
//   This code can boilerplated using the TODO tags for each        -
//   business method call from the customers middle tier EJB logic  -
//   to the relevant CASP business service requests or posts.       -
//                                                                  -
// Invocation Syntax:                                               -
//                                                                  -
//  String cost  getProductDetails(       Return "cost"             -
//                     int    productId,    Input                   -
//                     String description,  In/Out                  -
//                     String information)  Output                  -
//   Mapped to KxoOutbound "requestResponse" which is               -
//   TX_NOT_SUPPORTED                                               -
//   User EJB must not be in a JTA container transaction context    -
//                                                                  -
//  String cost  buyProduct(       Return "transTally"              -
//                     int    productId,    Input                   -
//                     String description,  In                      -
//                     String information)  on error output         -
//   Mapped to KxoOutbound "sendDatagram" which is                  -
//   TX_REQUIRED so User EJB                                        -
//   must pass a JTA container transaction context                  -
//                                                                  -
// Diagnostics:                                                     -
//   WAS RAS trace can be enabled setting Server trace properties   -
//   com.candle.roma.kxo.outbound.sample.*=all=enabled              -
//                                                                  -
//-------------------------------------------------------------------

package com.candle.roma.kxo.outbound.sample;

// Imports
import java.io.*;
import java.util.*;

// these are so we can narrow the context
import javax.rmi.PortableRemoteObject;

// these are so we can use the context assoicated with the calling EJB
import javax.naming.InitialContext;
import javax.naming.Context;

// CASP message handling API
import com.candle.roma.kxm.*;

// CASP SOAP handling utilities ...
import com.candle.roma.kxo.soap.*;

// CASP KXO Logger
import com.candle.roma.kxo.LogHelper;

// KxoOutbound Bean
import com.candle.roma.kxo.outbound.*; // KxoOutbound Bean

//-------------------------------------------------------------------
/**
  * 
  * AccessBean provides access to the KxoOutbound Bean. It also maps
  * a business-oriented API call to the SOAP oriented eBPprocess call.
  * TODO: this code should be tailored by the end user for each BS
  * @author 	Candle
  * @version 1.0  $Revision: 1.12 $
  */
public class AccessBean {

  // =======================================================================
  // kxo-websphere integrated log helper wrapper
  private static final LogHelper LOG = new LogHelper(AccessBean.class);

 /**
   * Construction.
   *
   * @since   1.0
   */
  public AccessBean()
  {
  }

 /**
   * Provides middle teir User EJB with a business method that 
   * maps to a syncronous inquiry to a eBP Business Service.
   * The business service is invoked sending the passed parameters via a
   * eBP SOAP request using eBPprocess method  on the KxoOutbound Bean, and the 
   * return param is compiled from its eBP SOAP response message.
   * This call can not operate under a JTA transaction.
   *
   * @param   productId    Integer identifying product to enquire on
   * @param   description  String Buffer containig product description which
   *                       maybe updated as a result of calling business service
   * @param   information  String Buffer to return information obtained as
   *                       a result of calling the eBP business service
   * @return  String containing the product cost obtained from calling the eBP
   *          business service or details of error encountered
   * @since   1.01
   */
  public String getProductDetailsByXml(int          productId,
                                       StringBuffer description,
                                       StringBuffer information) 
  {
    // Create an XML message Access Interface for our Target Business Service
    XmlAccessBean xmlAccess = new XmlAccessBean("KxoOutboundOrderSampleBS");

    LOG.debug("getProductDetaulsByXml");

    java.util.Vector msgs = new java.util.Vector();
    try {
      // Build the XML doc to send
      KxmMessage xmlDoc = XmlAccessUtils.createKxmMessage( "ProductDetailRequest" );
      msgs.add( xmlDoc );
      xmlDoc.getField("ProductId").set(productId);
      xmlDoc.getField("ProductDescription").set(description.toString());

      // Send it
      String xmlResponse = xmlAccess.requestXmlResponse(XmlAccessUtils.writeString(xmlDoc));

      // Extract fields from the response
      KxmMessage xmlResponseDoc = XmlAccessUtils.parseKxmMessage(xmlResponse.getBytes("UTF-8"));
      msgs.add( xmlResponseDoc );

      description.append(xmlResponseDoc.getField("ProductDescription").getString());
      information.append(xmlResponseDoc.getField("ProductInformation").getString());
      return xmlResponseDoc.getField("ProductCost").getString();
    }
    catch (Exception ex)
    {
      // Return the exception as information
      information.append("Exception : " + ex.toString());
      return("");
    }
    finally
    {
         XmlAccessUtils.destroyKxmMessages( msgs );
    }
  }

 /**
   * Provides middle teir User EJB with a business method that 
   * maps to a transactional datagram post to a eBP Business Service.
   * The business service is invoked sending the passed parameters via a
   * eBP SOAP datagram using eBPpost method on the KxoOutbound Bean, and the 
   * return param is compiled from its return.
   * This call must be invoked within a JTA transaction.
   *
   * @param   productId    Integer identifying product to enquire on
   * @param   description  String Buffer containing product description 
   * @param   information  String Buffer to return error detail information 
   * @return  String containing the message transaction audit / tally code
   *          from the post to the eBP business service or details of any 
   *          error encountered
   * @since   1.01
   */
   public String buyProductByXml(int          productId,
                                 StringBuffer description,
                                 StringBuffer information)
  {
    // Create an XML message Access Interface for our Target Business Service
    XmlAccessBean xmlAccess = new XmlAccessBean("KxoOutboundOrderSampleBS");

    LOG.debug("buyProductByXml");

    java.util.Vector msgs = new java.util.Vector();
    try {
      // Build the XML doc to send
      KxmMessage xmlDoc = XmlAccessUtils.createKxmMessage( "ProductDetailRequest" );
      msgs.add( xmlDoc );
      xmlDoc.getField("ProductId").set(productId);
      xmlDoc.getField("ProductDescription").set(description.toString());

      // Send it
      String messageId = xmlAccess.sendXmlDatagram(XmlAccessUtils.writeString(xmlDoc));
      return(messageId);
    }
    catch (Exception ex) {
      // Return the exception as information
      information.append("Exception : " + ex.toString());
      return("");
    }
    finally
    {
      XmlAccessUtils.destroyKxmMessages( msgs );
    }
  }
}
