//-------------------------------------------------------------------
//                                                                  -
// Module      : SOAPAccessBean                                     -
//                                                                  -
// Description : CASP for WebSphere Acess Outbound. EJB business    -
//               method call via SOAP to a CASP Business Service    -
//               access mapping bean class                          -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Copyright Candle Corporation 2001,2003                           -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Details:                                                         -
//   This sample SOAP Access Bean class shows how to take a         -
//   preformed XML SOAP doc and pass the request or  datagram       -
//   messages to be passed to the KxoOutbound Bean         -
//                                                                  -
//   This code can boilerplated using the TODO tags for each        -
//   business method call from the customers middle tier EJB logic  -
//   to the relevant CASP business service requests or posts.       -
//                                                                  -
// Invocation Syntax:                                               -
//                                                                  -
// Diagnostics:                                                     -
//                                                                  -
//   WAS RAS trace can be enabled setting Server trace properties   -
//   com.candle.roma.kxo.outbound.sample.*=all=enabled              -
//                                                                  -
//-------------------------------------------------------------------

package com.candle.roma.kxo.outbound.sample;

// Imports
import java.io.*;
import java.util.*;

// these are so we can narrow the context
import javax.rmi.PortableRemoteObject;

// these are so we can use the context assoicated with the calling EJB
import javax.naming.InitialContext;
import javax.naming.Context;

// eBP message handling API
import com.candle.roma.kxm.*;

// eBP SOAP handling utilities ...
import com.candle.roma.kxo.soap.*;

// eBP KXO Logger
import com.candle.roma.kxo.LogHelper;

// KxoOutbound Bean
import com.candle.roma.kxo.outbound.*; // KxoOutbound Bean

//-------------------------------------------------------------------


/**
  * SOAPAccessBean provides access to the KxoOutbound Bean It passes
  * SOAP XML envolope to the CASP Business Service
  * TODO: this code should be tailored by the end user for each BS
  * @author 	Candle
  * @version 2.0  $Revision: 1.8 $
  */
public class SOAPAccessBean {

  // =======================================================================
  // kxo-websphere integrated log helper wrapper
  private static final LogHelper LOG = new LogHelper(SOAPAccessBean.class);

  private static final String SOAP_METHOD_RESPONSE_TAG = "Response";

  // =======================================================================
  // The Kxo Outbound EJB Object - held as a static so that we only look
  // it up once
  private static KxoOutbound     kxoOutbound = null;

 /**
    * Construction.
    *
   * @since   1.0
   */
  public SOAPAccessBean() {}

 /**
   * Provides middle tier User EJB with a business method that 
   * dispatches a CASP SOAP envelope in a request response 
   * to a CASP Business Service  The Service identified in the SOAP
   * body will be sent the contents of the body as an XML doc The
   * XML doc returned in the Service response will be placed in a SOAP
   * response to return from this method If an error is encountered
   * a SOAP fault response will be returned This call can not operate
   * under a JTA transaction.
   *
   * @param   soapEnvelope String CASP SOAP envelope
   * @return  String containig the soapEnvelope response
   *          from the business service or a SOAP fault
   * @since   2.0
   */
  public String requestSOAPResponse(String soapEnvelope)  throws Exception
  {
    KxoSOAPEnvelope soapRequestEnvelope;
    String          responseMethodName;
    String          response;

    LOG.debug("requestSOAPResponse()");

    lookupKxoBean();

    soapRequestEnvelope  = buildSoapEnvelope(soapEnvelope);
    response             = kxoOutbound.requestResponse(soapRequestEnvelope);

    KxoSOAPMsg soapMsg = null;
    try {
      // Prepare the SOAP envelope for the response which will just be an
      // XML doc that we need to plant in a SOAP response message
      soapMsg = new KxoSOAPMsg();
      responseMethodName = soapRequestEnvelope.getBusinessService();
      responseMethodName += SOAP_METHOD_RESPONSE_TAG;

      soapMsg.setResponse(responseMethodName);

      // Response is either the returned XML document or a 
      // fault. Either way, it forms the Body of our SOAP 
      // envelope.
      soapMsg.setBody(response);
      String soapEnv = soapMsg.getEnvelope().getEnvelope();
      return(soapEnv);
    }
    catch (KxoSOAPException ex) {
      LOG.debug("Error creating SOAP envelope", ex);
      throw new Exception(ex.toString());
    }
    finally
    {
      if( soapMsg != null )
      {
        soapMsg.destroy();
      }
         
    }
  }

 /**
   * Provides middle teir User EJB with a business method that 
   * dispatches a CASP SOAP envelope in a transactional datagram
   * to a CASP Business Service  The Service identified in the SOAP
   * body will be sent the contents of the body as an XML doc 
   * The message identifier used in the send will be placed in a SOAP
   * response to return from this method If an error is encountered
   * a SOAP fault response will be returned This call must operate
   * under a JTA transaction.
   *
   * @param   soapEnvelope String CASP SOAP envelope
   * @return  String soapEnvelope either the message ID or a fault.
   * @since   2.0
   */
  public String sendXmlDatagram(String soapEnvelope) throws Exception
  {
    KxoSOAPEnvelope soapDatagramEnvelope;
    String          responseMethodName;
    String          response;

    LOG.debug("sendXmlDatagram()");

    lookupKxoBean();

    soapDatagramEnvelope= buildSoapEnvelope(soapEnvelope);
    response            = kxoOutbound.sendDatagram(soapDatagramEnvelope);

    KxoSOAPMsg soapMsg = null;
    try {
      // Prepare the SOAP envelope for the response
      soapMsg = new KxoSOAPMsg();
      responseMethodName = soapDatagramEnvelope.getBusinessService();
      responseMethodName += SOAP_METHOD_RESPONSE_TAG;
      soapMsg.setResponse(responseMethodName);

      // Response is either the message ID or a fault.
      // Either way, it forms the body of our SOAP envelope.
      soapMsg.setBody(response);
      String soapEnv = soapMsg.getEnvelope().getEnvelope();
      return(soapEnv);
    }
    catch (KxoSOAPException ex) {
      LOG.debug("Error creating SOAP envelope", ex);
      throw new Exception(ex.toString());
    }
    finally
    {
      if( soapMsg != null )
      {
        soapMsg.destroy();
      }
         
    }
  }


  // -----------------------------------------------------------------------
  // Implementation
  private synchronized void lookupKxoBean() throws Exception
  {
    LOG.debug("lookupKxoBean(\"ejb/KxoOutboundHome\")"); 
    
    if( kxoOutbound == null )
    {
      InitialContext ctx = new InitialContext();
      // Lookup based on our local resource reference which
      // was mapped to the EJB at deployment time
      Object         obj = ctx.lookup("java:comp/env/ejb/KxoOutboundHome");
      KxoOutboundHome li = (KxoOutboundHome)PortableRemoteObject.narrow(obj,
                                                       KxoOutboundHome.class);
      kxoOutbound = li.create();
      LOG.info("KxoSOAP Outbound Resolved EJB (\"ejb/KxoOutboundHome\")"); 
    }
    return;
  }

  private KxoSOAPEnvelope buildSoapEnvelope(String soapEnvelope) throws Exception
  {
    KxoSOAPMsg soapMsg = null;
    try {
      // Validates the envelope
      soapMsg = new KxoSOAPMsg(soapEnvelope.getBytes("UTF-8"));
      KxoSOAPEnvelope soapEnv = soapMsg.getEnvelope();
      return(soapEnv);
    }
    catch (KxoSOAPException ex) {
      throw new Exception("Unable to build envelope" + ex.toString());
    }
    finally
    {
      if( soapMsg != null )
      {
        soapMsg.destroy();
      }
         
    }
  }
}

