//-------------------------------------------------------------------
//                                                                  -
// Module      : XmlAccessBean                                      -
//                                                                  -
// Description : CASP for WebSphere Acess Outbound. EJB business    -
//               method call to CASP Business Service SOAP request  -
//               access mapping bean class                          -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Copyright Candle Corporation 2001,2003                           -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Details:                                                         -
//   This sample XML Access Bean class shows how to take a XML      -
//   stream and pass it to the appropriate CASP SOAP request or     -
//   datagram messages to be passed to the KxoOutbound Bean         -
//                                                                  -
//   This code can boilerplated using the TODO tags for each        -
//   business method call from the customers middle tier EJB logic  -
//   to the relevant CASP business service requests or posts.       -
//                                                                  -
// Invocation Syntax:                                               -
//                                                                  -
// Diagnostics:                                                     -
//                                                                  -
//   WAS RAS trace can be enabled setting Server trace properties   -
//   com.candle.roma.kxo.outbound.sample.*=all=enabled              -
//                                                                  -
//-------------------------------------------------------------------

package com.candle.roma.kxo.outbound.sample;

// Imports
import java.io.*;
import java.util.*;

// these are so we can narrow the context
import javax.rmi.PortableRemoteObject;

// these are so we can use the context assoicated with the calling EJB
import javax.naming.InitialContext;
import javax.naming.Context;

// eBP message handling API
import com.candle.roma.kxm.*;

// eBP SOAP handling utilities ...
import com.candle.roma.kxo.soap.*;

// eBP KXO Logger
import com.candle.roma.kxo.LogHelper;

// KxoOutbound Bean
import com.candle.roma.kxo.outbound.*; 

//-------------------------------------------------------------------
/**
  * XmlAccessBean provides access to the KxoOutbound Bean It maps
  * a business-oriented XML stream method call to the SOAP oriented 
  * CASP business service call.
  * TODO: this code should be tailored by the end user for each BS
  * @author 	Candle
  * @version 2.0  $Revision: 1.8 $
  */
public class XmlAccessBean {

  // =======================================================================
  // kxo-websphere integrated log helper wrapper
  private static final LogHelper LOG = new LogHelper(XmlAccessBean.class);


  // =======================================================================
  // The Kxo Outbound EJB Object - held as a static so that we only look
  // it up once
  private static KxoOutbound     kxoOutbound = null;

  // =======================================================================
  // The Business Service this Access Bean is interfaceing
  private String businessService = null;
  private String businessServiceResponse = null;
  private static final String SOAP_METHOD_RESPONSE_TAG = "Response";

 /**
    * Construction.
    *
   * @since   1.0
   */
  public XmlAccessBean(String targetBusinessService)
  {
    if (LOG.isDebugEnabled())
      LOG.debug("XmlAccessBean(" + targetBusinessService + ")");

    businessService = targetBusinessService;
    businessServiceResponse = targetBusinessService + SOAP_METHOD_RESPONSE_TAG;
  }

 /**
   * Provides middle tier User EJB with a business method that 
   * maps to a syncronous inquiry with an XML stream to a CASP Business 
   * Service.
   * 
   * If wrong (non-XML) response is received then this method throws Exception.
   *
   * @param   xmlDoc  String containing the XML doc to pass to the
   *                  CASP business service
   * @return  String containing the XML doc returned from the
   *          business service.
   * @exception Exception some error
   * @since   2.0
   */
  public String requestXmlResponse(String xmlDoc) throws Exception
  {
    LOG.debug("requestXmlResponse()");

    lookupKxoBean();
                
    KxoSOAPEnvelope soapRequestEnvelope = buildSoapRequest(xmlDoc);
    String response = kxoOutbound.requestResponse(soapRequestEnvelope);
    
    // Response string is either the response XML document
    // from our BusinessService, or it's a SOAP-Fault
    // so parse it to get the XML document name

    KxmMessage responseXmlDoc = null;

    try {
      
      try {
        responseXmlDoc  = new KxmMessage(response.getBytes("UTF-8"), KxmMessage.STANDALONE);
      
        // bind SOAP-ENV prefix to SOAP namespace URI for easy logging
        // and root name check
        responseXmlDoc.declareNs( 
          "", 
          KxoSOAPConstants.SOAP_NS, 
          KxoSOAPConstants.SOAP_URI );

        String responseDocName = responseXmlDoc.getXmlDocName();

LOG.debug("Response document name: " + responseDocName);
        if (LOG.isDebugEnabled())
          LOG.debug("Response document name: " + responseDocName);

        // if not SOAP-ENV:Fault then success, just return response
        if( !responseDocName.equals(KxoSOAPConstants.TAG_FAULT) )
          return response;

      }
      catch (KxmException ex) {
        throw new Exception("Response is not well-formed XML: " + response);
      }

      throwSoapFault( responseXmlDoc );

    }
    finally {
      // release KxmMessage immediately for better performance,
      // this object uses resources outside JVM
      if( responseXmlDoc != null )
        responseXmlDoc.destroy();
    }

    return "";
  }

  /**
    *  Throws SOAP fault error message.
    */ 
  private void throwSoapFault( KxmMessage faultDoc ) throws Exception
  {
      StringBuffer error = new StringBuffer();
      error.append( "SOAP Fault: " );
      appendFaultElement( faultDoc, "faultcode", error );
      appendFaultElement( faultDoc, "faultstring", error );
      appendFaultElement( faultDoc, "faultactor", error );
      appendFaultElement( faultDoc, "detail", error );
      throw new Exception( new String(error) );
  }  

  /** 
    * Append "fieldName=fieldValue" string to error message.
    *  
    * @param fault      SOAP-ENV:Fault XML message
    * @param faultElem  XML element to append to error message
    * @param error      error StringBuffer to append info to 
    */ 
  private void appendFaultElement( 
    KxmMessage fault, 
    String faultElem, 
    StringBuffer error ) throws KxmException 
  {
    KxmField field = fault.getField( faultElem );
    if( !field.isEmpty() ) {
      error.append( faultElem );
      error.append( "=\"" );
      error.append( field.getString() );
      error.append( "\" " );
    } 
  }

 /**
   * Provides middle tier User EJB with a business method that 
   * maps to a transactional datagram post of an XML doc to a CASP
   * Business Service. The business service is invoked sending the
   * passed XML Doc via a CASP SOAP datagram using post method on the
   * KxoOutbound Bean, and the return param is the message flow
   * identifier. This call can be invoked within a JTA transaction.
   *
   * @param   xmlDoc  String containing the XML doc to pass to the
   *                  CASP business service
   * @return  String containing the message transaction audit / tally code
   *          from the post to the CASP business service.
   * @since   2.0
   * @exception Exception some error
   */
  public String sendXmlDatagram(String xmlDoc) throws Exception
  {
    LOG.debug("sendXmlDatagram()");

    lookupKxoBean();

    KxoSOAPEnvelope soapDatagramEnvelope = buildSoapDatagram(xmlDoc);
    String response = kxoOutbound.sendDatagram(soapDatagramEnvelope);

    KxmMessage responseXmlDoc = null;

    try {
      
      String responseDocName = null;

      try {
        responseXmlDoc = XmlAccessUtils.parseKxmMessage(response.getBytes("UTF-8"));

        // bind SOAP-ENV prefix to SOAP namespace URI for easy logging
        // and root name check
        responseXmlDoc.declareNs( 
          "", 
          KxoSOAPConstants.SOAP_NS, 
          KxoSOAPConstants.SOAP_URI );
      
        responseDocName = responseXmlDoc.getXmlDocName();

        if (LOG.isDebugEnabled())
          LOG.debug("Response document name: " + responseDocName);

        // Expected response, return ID of the sent message
        if( responseDocName.equals(businessServiceResponse) )
        {
          return responseXmlDoc.getField(KxoSOAPConstants.TAG_MESSAGE_ID).getString();
        }
      }  
      catch (KxmException ex) {
        throw new Exception("Unexpected response: " + response);
      }

      // SOAP-ENV:Fault
      if( responseDocName.equals(KxoSOAPConstants.TAG_FAULT) )
        throwSoapFault( responseXmlDoc );

      // some unexpected response
      throw new Exception("Unexpected response: " + response);

    }
    finally{
      // release KxmMessage immediately for better performance,
      // this object uses resources outside JVM
      if( responseXmlDoc != null )
        responseXmlDoc.destroy();
    }
  }

  // -----------------------------------------------------------------------
  // Implementation
  private synchronized void lookupKxoBean() throws Exception
  {
    LOG.debug("lookupKxoBean(\"ejb/KxoOutboundHome\")"); 
    
    if ( kxoOutbound == null )
    {
      InitialContext ctx = new InitialContext();
      // Lookup based on our local resource reference which
      // was mapped to the EJB at deployment time
      Object         obj = ctx.lookup("java:comp/env/ejb/KxoOutboundHome");
      KxoOutboundHome li = (KxoOutboundHome)PortableRemoteObject.narrow(obj,
                                                       KxoOutboundHome.class);
      kxoOutbound = li.create();
      LOG.info("Kxo Outbound Resolved EJB (\"ejb/KxoOutboundHome\")"); 
    }
    return;
  }

  private KxoSOAPEnvelope buildSoapRequest(String xmlDoc) throws Exception
  {
    KxoSOAPMsg soapMsg = new KxoSOAPMsg();

    try{
      // Add the CASP BusinessService name to invoke
      soapMsg.setRequest(businessService);

      // Set any header values 
      // Persistance off for request/response
      soapMsg.setHeader(KxoSOAPConstants.TAG_PERSISTENT, KxoSOAPConstants.OPT_NO);

      // timeout to 3 sec and eBP expiry to 1 sec
      soapMsg.setHeader(KxoSOAPConstants.TAG_TIMEOUT, "30000");
      soapMsg.setHeader(KxoSOAPConstants.TAG_EXPIRY, "10000");

      // Eg Tell the CEJO Forwarder to use API Profile..
      //soapMsg.setHeader(KxoSOAPConstants.TAG_PROFILE, "TestPutProfile1");
    
      // Tell the CEJO Forwarder to set a Message Descriptor/Format name..
      soapMsg.setHeader(KxoSOAPConstants.TAG_FORMAT, "kxcMsg00000002");

      // Finally add the Xml document to the SOAP body
      soapMsg.setBody(xmlDoc);

      // And return the soap envelope
      KxoSOAPEnvelope soapEnv = soapMsg.getEnvelope();
      return(soapEnv);
    }
    finally
    {
        soapMsg.destroy();
    }
  }

  private KxoSOAPEnvelope buildSoapDatagram(String xmlDoc) throws Exception
  {
    KxoSOAPMsg soapMsg = new KxoSOAPMsg();

    try{
      // Add the CASP BusinessService name to invoke
      soapMsg.setDatagram(businessService);

      // Set any header values 
      // Persistance off for request/response
      soapMsg.setHeader(KxoSOAPConstants.TAG_PERSISTENT, KxoSOAPConstants.OPT_YES);

      // Eg Tell the CEJO Forwarder to use API Profile..
      //soapMsg.setHeader(KxoSOAPConstants.TAG_PROFILE, "TestPutProfile1");

      // Tell the CEJO Forwarder to set a Message Descriptor/Format name..
      soapMsg.setHeader(KxoSOAPConstants.TAG_FORMAT, "kxcMsg00000002");

      // Finally add the Xml document to the SOAP body
      soapMsg.setBody(xmlDoc);

      // And return the soap envelope
      KxoSOAPEnvelope soapEnv = soapMsg.getEnvelope();
      return(soapEnv);
    }
    finally
    {
        soapMsg.destroy();
    }
  }

}

