//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoicxt.cpp,v 1.26 2004/02/18 16:35:08 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoicxt.cpp,v $
// $Revision: 1.26 $
//--------------------------------------------------------------------
// The internal implementation of the RomaClient, the
// RomaBusinessElement and the RomaStorageQueue public interfaces.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoicxt.hpp"
#include "kxcoierr.hpp"
#include "kxcoipxy.hpp"
#include "kxcosbl.hpp"
#include "kxcostq.hpp"
#include "kxcobe.hpp"
#include "kxcocl.hpp"
#include "kxcodefs.hpp"

TRACE_MODULE_VERSION($Revision: 1.26 $);


#define TRACE_BUFFER_DUMP(buf) \
    TRACE_DUMP(buf.pBuffer,buf.BufLen,<< "Buffer");


//====================================================================
// The RomaImplContext class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplContext::RomaImplContext() 
    : m_handle(NULL_HKXC), m_error(NULL), m_ilsIncoming(NULL_HKXCILS)
{
    TRACE_ENTRY(RomaImplContext::RomaImplContext, << TRACE(this));
    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
// If the object is connected, it will be disconnected on destruction.
//--------------------------------------------------------------------
RomaImplContext::~RomaImplContext()
{
    TRACE_ENTRY(RomaImplContext::~RomaImplContext, << TRACE(this));

    disconnect();
    delete m_error;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Establish connection.
// The callType parameter is used to distinguish client and server
// connection type.
//--------------------------------------------------------------------
bool RomaImplContext::connect(KXCULONG callType, 
                              const char* name, 
                              KXCULONG options, 
                              const RomaSecurityDef& secDef,
                              const char* model, 
                              const RomaILSData& ilsData,
                              const RomaDirectoryAccess& auth)
{
    TRACE_ENTRY(RomaImplContext::connect, << TRACE(this) 
                                          << TRACE(name) 
                                          << TRACE(options) 
                                          << TRACE(secDef)
                                          << TRACE(model)
                                          << TRACE(ilsData)
                                          << TRACE(auth));

    // Check that there is no connection.
    if (!checkDisconnected(callType))
    {
        TRACE_INT_RETURN(false);
    }

    // If the RomaDirectoryAccess contains user ID and password,
    // use them for the directory authentication. This should be done
    // prior to the context initialization.
    PKXCDIRACCESSOPT access = 
        (PKXCDIRACCESSOPT)RomaImplProxy::getImpl(auth);

    if ((access->Id || access->Password) &&
        kxcDirSetAccess(access) == FALSE)
    {
        setErrorStatus(false);
        TRACE_INT_RETURN(false);
    }

    // Setup the KXCAPPDEF structure with the specified parameters.
    KXCAPPDEF appDef = { KXCAPPDEF_DEFAULT };
    kxcSetAppElementName(&appDef, name);
    kxcSetAppOptions(&appDef, options);

    if (model)
    {
        strncpy((char*)appDef.ModelQName, model, KXCOBJECT_NAME_LENGTH);
    }

    appDef.pSecurity = (PKXCSECDEF)RomaImplProxy::getImpl(secDef);
    appDef.ILSData.hToILS = (HKXCILS)RomaImplProxy::getImpl(ilsData);

    // Initialize the context, i.e. establish connection.
    bool success = (callType == KXCOCALLTYPE_BECONNECT ?
                    kxcServerInitialize(&appDef, &m_handle) :
                    kxcClientInitialize(&appDef, &m_handle)) == TRUE;

    if (success)
    {
        // Save handle of ILS data for further use.
        m_ilsIncoming = appDef.ILSData.hFromILS;
    }
    else
    {
        m_handle = NULL_HKXC;
        m_ilsIncoming = NULL_HKXCILS;
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(m_handle != NULL_HKXC);
}

//--------------------------------------------------------------------
// Terminate the connection.
//--------------------------------------------------------------------
bool RomaImplContext::disconnect()
{
    TRACE_ENTRY(RomaImplContext::disconnect, << TRACE(this));

    bool success = true;

    m_ilsIncoming = NULL_HKXCILS;

    if (m_handle != NULL_HKXC)
    {
        success = (kxcTerminate(&m_handle) == TRUE);
        setErrorStatus(success);
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Return the last error information.
//--------------------------------------------------------------------
RomaError& RomaImplContext::getError() 
{ 
    // If there is no error, create empty RomaError object.
    // An application should determine error presence using the
    // anyError() method call.
    if (!m_error) 
    {
        m_error = new RomaError; 
    }

    return *m_error;
}

//--------------------------------------------------------------------
// Commit local transaction.
//--------------------------------------------------------------------
bool RomaImplContext::commitLocal()
{
    TRACE_ENTRY(RomaImplContext::commitLocal, << TRACE(this));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_COMMITLOCAL))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the KXCSYNC structure.
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    sync.TransactionType = KXCSYNC_LOCAL;

    // Commit transaction.
    bool success = (kxcCommit(m_handle, &sync) == TRUE);

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Rollback local transaction.
//--------------------------------------------------------------------
bool RomaImplContext::rollbackLocal()
{
    TRACE_ENTRY(RomaImplContext::rollbackLocal, << TRACE(this));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_ROLLBACKLOCAL))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the KXCSYNC structure.
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    sync.TransactionType = KXCSYNC_LOCAL;

    // Rollback transaction.
    bool success = (kxcRollback(m_handle, &sync) == TRUE);

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Close the specified theme.
//--------------------------------------------------------------------
bool RomaImplContext::closeTheme(const RomaTheme& theme)
{
    return closeTheme((HKXCTHEME)theme);
}

//--------------------------------------------------------------------
// Close the specified theme handle.
//--------------------------------------------------------------------
bool RomaImplContext::closeTheme(HKXCTHEME theme)
{
    TRACE_ENTRY(RomaImplContext::closeTheme, << TRACE(this)
                                             << TRACE(theme));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_CLOSETHEME))
    {
        TRACE_INT_RETURN(false);
    }

    bool success = true;

    if (theme != NULL_HKXCTHEME)
    {
        // Close the theme.
        success = (kxcCloseHandle(m_handle, &theme) == TRUE);
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Put the exception message.
//--------------------------------------------------------------------
bool RomaImplContext::putException(RomaExceptionMessage& message,
                                   const RomaExceptionOptions& options)
{
    TRACE_ENTRY(RomaImplContext::putException, << TRACE(this) 
                                               << TRACE(message) 
                                               << TRACE(options));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_PUTEXCEPTION))
    {
        TRACE_INT_RETURN(false);
    }

    // Set the service name if specified.
    KXCSERVICE service = { KXCSERVICE_DEFAULT };
    KXCSERVICE* servPtr = NULL;

    const RomaImplExceptionOptions& optImpl = 
        RomaImplProxy::getImpl(options);
    const char* servName = optImpl.getServiceName();

    if (servName)
    {
        kxcSetServiceName(&service, servName);
        servPtr = &service;
    }

    // Fill the C API structures with the message and
    // put options data.
    RomaImplExceptionMessage& msgImpl = RomaImplProxy::getImpl(message);
    KXCEXCEPTION exc = { KXCEXCEPTION_DEFAULT };
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    HKXCTHEME theme;

    optImpl.populate(po, sync, theme);
    msgImpl.populate(exc);
    msgImpl.populate(po);

    // Put the exception.
    bool success = (kxcPutException(m_handle, servPtr, (PKXCMSG)msgImpl,
                                    &exc, &po, &sync, &theme) == TRUE);

    if (success)
    {
        // If the operation succeeded, save the theme on the message.
        msgImpl.capture(theme);
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Report the error.
//--------------------------------------------------------------------
bool RomaImplContext::reportError(RomaExceptionMessage& message,
                                  KXCULONG& actions,
                                  const RomaReportOptions& options)
{
    TRACE_ENTRY(RomaImplContext::reportError, << TRACE(this) 
                                              << TRACE(message)
                                              << TRACE(options));

    actions = 0;

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_REPORTERROR))
    {
        TRACE_INT_RETURN(false);
    }

    // Fill the C API structures with the message and
    // report options data.
    RomaImplExceptionMessage& msgImpl = RomaImplProxy::getImpl(message);
    const RomaImplReportOptions& optImpl = RomaImplProxy::getImpl(options);

    KXCEPOPT ro = { KXCEPOPT_DEFAULT };
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    KXCEXCEPTION exc = { KXCEXCEPTION_DEFAULT };
    HKXCTHEME theme;

    optImpl.populate(ro, sync, theme);
    msgImpl.populate(exc);

    // Report the error.
    bool success = (kxcReportError(m_handle, &ro, (PKXCMSG)msgImpl, 
                                   &exc, &sync, &theme) == TRUE);

    if (success) 
    {
        // If the operation succeeded, save the theme on the message
        // and performed actions in the specified variable.
        msgImpl.capture(theme);
        actions = ro.Actions;
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Send the alert.
//--------------------------------------------------------------------
bool RomaImplContext::sendAlert(const RomaLogInformation& logInfo)
{
    TRACE_ENTRY(RomaImplContext::sendAlert, << TRACE(this) 
                                            << TRACE(logInfo));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SENDALERT))
    {
        TRACE_INT_RETURN(false);
    }

    // Send the alert.
    bool success = (kxcSendAlert(m_handle, 
        (PKXCLOGDATA)RomaImplProxy::getImpl(logInfo)) == TRUE);

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Write the specified information to the user log.
//--------------------------------------------------------------------
bool RomaImplContext::writeLog(const RomaLogInformation& logInfo)
{
    TRACE_ENTRY(RomaContextImpl::writeLog, << TRACE(this) 
                                           << TRACE(logInfo));

    // Write to log.
    bool success = (kxcWriteLog(
        (PKXCLOGDATA)RomaImplProxy::getImpl(logInfo)) == TRUE);

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Write the specified information to the system log.
//--------------------------------------------------------------------
bool RomaImplContext::writeSysLog(const RomaLogInformation& logInfo)
{
    TRACE_ENTRY(RomaImplContext::writeSysLog, << TRACE(this) 
                                              << TRACE(logInfo));

    // Write to log.
    bool success = (kxcWriteSysLog(
        (PKXCLOGDATA)RomaImplProxy::getImpl(logInfo)) == TRUE);

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Get data provided by ILSes invoked on initialization.
//--------------------------------------------------------------------
RomaILSData* RomaImplContext::getILSData()
{
    // If there is no ILS data, just return NULL.
    if (m_ilsIncoming == NULL_HKXCILS)
    {
        return NULL;
    }

    // Create the implemention instance from the ILS data handle
    // and create the public interface from the implemention instance.
    return RomaImplProxy::createPublic(
        new RomaImplILSData(m_ilsIncoming, this, m_handle));
}

//--------------------------------------------------------------------
// Delete the specified storage queue.
//--------------------------------------------------------------------
bool RomaImplContext::deleteStorageQueue(const char* queueName)
{
    TRACE_ENTRY(RomaImplContext::deleteStorageQueue, << TRACE(this) 
                                                     << TRACE(queueName));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_DELETESTORAGEQUEUE))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the KXCSTORQUEUEDEF structure.
    KXCSTORQUEUEDEF queueDef = { KXCSTORQUEUEDEF_DEFAULT };

    if (queueName)
    {
        strncpy((char*)queueDef.QueueName, queueName, 
            sizeof(queueDef.QueueName) - 1);
        queueDef.QueueName[sizeof(queueDef.QueueName) - 1] = 0;
    }

    // Delete the queue.
    bool success = (kxcDeleteStorageQueue(m_handle, &queueDef) == TRUE);

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Open the specified storage queue.
//--------------------------------------------------------------------
HKXCSTORQ RomaImplContext::openStorageQueue(const char* queueName, 
                                            KXCULONG options,
                                            const char* queueModel)
{
    TRACE_ENTRY(RomaImplContext::openStorageQueue, << TRACE(this)
                                                   << TRACE(queueName)
                                                   << TRACE(options)
                                                   << TRACE(queueModel));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SQOPEN))
    {
        TRACE_INT_RETURN(NULL_HKXCSTORQ);
    }

    // Setup the KXCSTORQUEUEDEF structure.
    KXCSTORQUEUEDEF queueDef = { KXCSTORQUEUEDEF_DEFAULT };
    kxcSetStoreQueueOptions(&queueDef, options);
    kxcSetStoreQueueQueueName(&queueDef, queueName);

    if (queueModel)
    {
        kxcSetStoreQueueModelQName(&queueDef, queueModel);
    }

    // Open the queue.
    HKXCSTORQ qh;
    bool success = (kxcOpenStorageQueue(m_handle, &queueDef, &qh) == TRUE);

    if (!success) 
    {
        qh = NULL_HKXCSTORQ;
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(qh);
}

//--------------------------------------------------------------------
// Close the specified storage queue.
//--------------------------------------------------------------------
bool RomaImplContext::closeStorageQueue(HKXCSTORQ queueHandle)
{
    TRACE_ENTRY(RomaImplContext::closeStorageQueue,
        << TRACE(this) << TRACE(queueHandle));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SQCLOSE))
    {
        TRACE_INT_RETURN(false);
    }

    // Close the queue.
    bool success = (kxcCloseStorageQueue(
        m_handle, &queueHandle) == TRUE);

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Put the message on the storage queue.
//--------------------------------------------------------------------
bool RomaImplContext::storeStorable(HKXCSTORQ queueHandle,
                                    RomaStorable& message,
                                    const RomaStoreOptions& options)
{
    TRACE_ENTRY(RomaImplContext::storeStorable, << TRACE(this) 
                                                << TRACE(queueHandle)
                                                << TRACE(message)
                                                << TRACE(options));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SQSTORE))
    {
        TRACE_INT_RETURN(false);
    }

    // Marshal the message.
    if (!RomaImplProxy::marshalForStorage(message))
    {
        KXCO_SET_CPP_ERROR(KXCORC_MARSHAL_STORABLE);
        reportCppError(KXCOCALLTYPE_SQSTORE);
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    HKXCTHEME theme;

    RomaImplStorable& msgImpl = RomaImplProxy::getImpl(message);
    const RomaImplStoreOptions& optImpl = RomaImplProxy::getImpl(options);

    optImpl.populate(po, sync, theme);
    msgImpl.populate(po, theme);

    // Put the message.
    bool success = (kxcPutToStorageQueue(m_handle, queueHandle,
                       (PKXCMSG)msgImpl, &po, &sync, &theme) == TRUE);

    if (success)
    {
        // If the operation succeeded, save the theme on the message.
        msgImpl.setTheme(theme);
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Get a message from the storage queue.
//--------------------------------------------------------------------
RomaStorable* RomaImplContext::retrieveStorable(HKXCSTORQ queueHandle,
                                  KXCMSG& message,
                                  const RomaRetrieveOptions& options,
                                  RomaStorageQueue& sq,
                                  bool noAutoAlloc)
{
    TRACE_ENTRY(RomaImplContext::retrieveStorable, << TRACE(this) 
                                                   << TRACE(queueHandle)
                                                   << TRACE(options)
                                                   << TRACE(noAutoAlloc));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SQRETRIEVE))
    {
        TRACE_INT_RETURN(NULL);
    }

    // Setup the C API structures.
    KXCGETOPT go = { KXCGETOPT_DEFAULT };
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    HKXCTHEME theme;

    RomaImplProxy::getImpl(options).populate(go, sync, theme);

    // Delete the last error.
    resetError();

    // Indicate whether the data buffers must be auto-allocated.
    if (noAutoAlloc)
    {
        go.ReceiveOptions &= ~KXCGOPT_AUTO_ALLOC;
    }
    else
    {
        go.ReceiveOptions |= KXCGOPT_AUTO_ALLOC;
    }

    // Get a message.
    bool success = (kxcGetFromStorageQueue(m_handle, queueHandle, 
                              &message, &go, &sync, &theme) == TRUE);

    if (!success) 
    {
        // If there is no message on the queue or a message is
        // suppressed, we don't set an error on the object, but
        // just return NULL. However, if we are working under a
        // syncpoint, we should report the KXCRC_MESSAGE_SUPPRESSED
        // error.
        if (go.Error.LastReturnCode != KXCRC_NO_MSG_AVAILABLE &&
            (go.Error.LastReturnCode != KXCRC_MESSAGE_SUPPRESSED ||
             sync.TransactionType != KXCSYNC_NONE)) 
        {
            // Set the last error on the object.
            setErrorStatus(false);
        }

        TRACE_INT_RETURN(NULL);
    }

    // Create the RomaStorable from the KXCMSG.
    RomaStorable* updated = NULL;
    RomaStorable* original = RomaImplProxy::createPublic(
        new RomaImplStorable(message, theme, *this));

    // Unmarshal the message.
    if (!RomaImplProxy::createStorable(sq, *original, updated) || !updated)
    {
        KXCO_SET_CPP_ERROR(KXCORC_CREATE_STORABLE);
        reportCppError(KXCOCALLTYPE_SQRETRIEVE, &message, theme, &sync);
    }

    // Delete the original message and return unmarshaled.
    delete original;
    TRACE_INT_RETURN(updated);
}

//--------------------------------------------------------------------
// Resets the last error depending on the argument.
// If true specified, then the last operation succeeded, so the object
// containing the last error information should be deleted.
// Otherwise, some error occured, so the last error information should
// be gathered in order to be returned on an application request.
//--------------------------------------------------------------------
void RomaImplContext::setErrorStatus(bool success)
{
    if (success)
    {
        delete m_error;
        m_error = NULL;
        return;
    }

    if (!m_error)
    {
        m_error = new RomaError;
    }

    // Gather the last error information.
    RomaImplProxy::getImpl(*m_error).capture();
}

//--------------------------------------------------------------------
// Fills the object containing the last error information with data of
// an error occured in the C++ API.
//--------------------------------------------------------------------
void RomaImplContext::setCppError(KXCULONG retcode,
                                  KXCULONG position,
                                  KXCULONG errorType,
                                  const char* method,
                                  KXCULONG line,
                                  const char* module)
{
    TRACE_ENTRY(RomaImplContext::setCppError, << TRACE(this) 
                                              << TRACE(retcode));

    // If the error object does not exsist, create it.
    if (!m_error)
    {
        m_error = new RomaError;
    }
    
    RomaImplError& impl = RomaImplProxy::getImpl(*m_error);

    // Set error properties.
    impl.setStatus(Roma::STATUS_PRIMARY);
    impl.setCompCode(Roma::SUBCLASS_PRIMARY, Roma::CC_FAILED);
    impl.setReturnCode(Roma::SUBCLASS_PRIMARY, retcode);
    impl.setReason(Roma::SUBCLASS_PRIMARY, 0);
    impl.setOperationType(Roma::SUBCLASS_PRIMARY, Roma::OP_KXC_OM);
    impl.setPosition(Roma::SUBCLASS_PRIMARY, position);
    impl.setType(Roma::SUBCLASS_PRIMARY, (Roma::ErrorType)errorType);
    impl.setLineNum(Roma::SUBCLASS_PRIMARY, line);

    // Set detailed error description.
    const char* shortName = strrchr(module, '/');

    if (!shortName) 
    {
        shortName = strrchr(module, '\\');
    }

    if (shortName) 
    {
        module = shortName + 1;
    }

    impl.setAdditionalInfo(Roma::SUBCLASS_PRIMARY, module, 
        KXCO_BUILD, method);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Report an error occured in the C++ API to an Error Analysis
// Routine according to the application's error profile.
//--------------------------------------------------------------------
void RomaImplContext::reportCppError(KXCULONG callType,
                                     KXCMSG* message,
                                     HKXCTHEME theme,
                                     KXCSYNC* sync)
{
    TRACE_ENTRY(RomaImplContext::reportCppError, << TRACE(this) 
                                                 << TRACE(callType));

    if (!anyError())
    {
        // No error, nothing to do.
        TRACE_VOID_RETURN;
    }

    // Setup error logging options.
    KXCEPOPT eo = { KXCEPOPT_DEFAULT };
    eo.ErrorOptions = KXCEPOPT_USEDEFAULT;
    eo.DefaultAction = KXCEP_ACTION_LOG;

    // Setup exception message details.
    KXCEXCEPTION exc = { KXCEXCEPTION_DEFAULT };
    exc.ExceptionFlags = KXCEXC_TIMESTAMP | KXCEXC_SYSTEMDATA |
                         KXCEXC_ERRORDATA | KXCEXC_ROMADATA;
    exc.ErrorData.CallType = callType;
    RomaImplProxy::getImpl(*m_error).populate(exc.ErrorData.Error);

    // Report the error/
    if (!kxcReportError(m_handle, &eo, message, &exc, sync, &theme))
    {
        // The error reporting failed, so write details to the trace log.
        RomaError last;
        RomaImplProxy::getImpl(last).capture();

        TRACE_WARNING(<< "kxcReportError() failed");
        TRACE_WARNING(<< last.errorText());

        if (last.status() == Roma::STATUS_BOTH)
        {
            TRACE_WARNING(<< last.errorText(Roma::SUBCLASS_SECONDARY));
        }
    }
    else
    {
        TRACE_EVENT(<< TRACE(eo.DefaultAction) 
                    << TRACE(eo.Actions));
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Raise the error if the object is disconnected.
//--------------------------------------------------------------------
bool RomaImplContext::checkConnected(KXCULONG callType)
{
    TRACE_ENTRY(RomaImplContext::checkConnected, << TRACE(this));

    bool success = (m_handle != NULL_HKXC);

    if (!success)
    {
        KXCO_SET_CPP_ERROR(KXCORC_NOT_CONNECTED);
        reportCppError(callType);
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Raise the error if the object is connected.
//--------------------------------------------------------------------
bool RomaImplContext::checkDisconnected(KXCULONG callType)
{
    TRACE_ENTRY(RomaImplContext::checkDisconnected, << TRACE(this));

    bool success = (m_handle == NULL_HKXC);

    if (!success)
    {
        KXCO_SET_CPP_ERROR(KXCORC_ALREADY_CONNECTED);
        reportCppError(callType);
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Inserts the business service into the workflow or replaces
// the workflow with the business service.
//--------------------------------------------------------------------
bool RomaImplContext::insertWorkflow(RomaTheme& newTheme,
                                     const RomaTheme& curTheme,
                                     const RomaBusinessService& bs,
                                     bool close, bool replace)
{
    HKXCTHEME theme = (HKXCTHEME)curTheme;

    TRACE_ENTRY(RomaImplContext::insertWorkflow,
        << TRACE_HEX(theme) << TRACE(close) << TRACE(replace));

    // Check that the object is connected.
    if (!checkConnected(
        replace ? KXCOCALLTYPE_REPLACEWORKFLOW : KXCOCALLTYPE_INSERTWORKFLOW))
    {
        TRACE_INT_RETURN(false);
    }

    KXCSERVICE serv;
    RomaImplProxy::getImpl(bs).populate(serv);

    KXCBUFFER buf = { KXCBUFFER_DEFAULT };
    buf.pBuffer = &serv;
    buf.BufLen = sizeof(serv);

    TRACE_BUFFER_DUMP(buf);

    bool success = (kxcBuildTheme(m_handle, theme,
                        (close ? KXCTOPT_CLOSETHEME : 0) |
                        (replace ? KXCTOPT_REPLACEWFLOW : KXCTOPT_INSERTWFLOW),
                        &buf, &theme) == TRUE);

    if (success)
    {
        if (close)
        {
            RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
        }

        RomaImplProxy::getImpl(newTheme) = theme;
        TRACE_EVENT(<< "New " << TRACE_HEX(theme));
    }

    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Sets the business flow information on the context.
//--------------------------------------------------------------------
bool RomaImplContext::setBusinessFlow(const RomaBusinessFlow& bf)
{
    TRACE_ENTRY(RomaImplContext::setBusinessFlow, << "");

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SETBUSFLOW))
    {
        TRACE_INT_RETURN(false);
    }

    KXCBUSFLOW busflow;
    RomaImplProxy::getImpl(bf).populate(busflow);

    KXCBUFFER buf = { KXCBUFFER_DEFAULT };
    buf.pBuffer = &busflow;
    buf.BufLen = sizeof(busflow);

    TRACE_BUFFER_DUMP(buf);

    HKXCTHEME theme;
    bool success = (kxcBuildTheme(m_handle, NULL_HKXCTHEME,
                        KXCTOPT_SETBUSFLOW, &buf, &theme) == TRUE);

    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Creates a new theme that is copy of the specified theme.
//--------------------------------------------------------------------
bool RomaImplContext::copyTheme(RomaTheme& newTheme,
                                const RomaTheme& curTheme)
{
    HKXCTHEME theme = (HKXCTHEME)curTheme;

    TRACE_ENTRY(RomaImplContext::copyTheme, << TRACE_HEX(theme));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_COPYTHEME))
    {
        TRACE_INT_RETURN(false);
    }

    bool success = (kxcBuildTheme(m_handle, theme,
        KXCTOPT_COPYTHEME, NULL, &theme) == TRUE);

    if (success)
    {
        RomaImplProxy::getImpl(newTheme) = theme;
        TRACE_EVENT(<< "New " << TRACE_HEX(theme));
    }

    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Merges ETE information into the theme.
//--------------------------------------------------------------------
bool RomaImplContext::mergeEteInfo(RomaTheme& newTheme,
                                   const RomaTheme& curTheme,
                                   const RomaEteInfo& ete,
                                   bool close)
{
    HKXCTHEME theme = (HKXCTHEME)curTheme;
    RomaImplEteInfo* eteImpl = RomaImplProxy::getImplPtr(ete);

    TRACE_ENTRY(RomaImplContext::mergeEteInfo, 
        << TRACE_HEX(theme) << TRACE_HEX(eteImpl));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_MERGEETEINFO))
    {
        TRACE_INT_RETURN(false);
    }

    if (eteImpl == NULL)
    {
        bool res = true;

        if (close)
        {
            newTheme = curTheme;
            RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
            setErrorStatus(true);
        }
        else
        {
            res = copyTheme(newTheme, curTheme);
        }

        TRACE_INT_RETURN(res);
    }

    KXCBUFFER buf = { KXCBUFFER_DEFAULT };
    buf.pBuffer = (void*)eteImpl->getData(buf.BufLen);

    TRACE_BUFFER_DUMP(buf);

    bool success = (kxcBuildTheme(m_handle, theme,
        KXCTOPT_MERGEETEINFO | (close ? KXCTOPT_CLOSETHEME : 0),
        &buf, &theme) == TRUE);

    if (success)
    {
        if (close)
        {
            RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
        }

        RomaImplProxy::getImpl(newTheme) = theme;
        TRACE_EVENT(<< "New " << TRACE_HEX(theme));
    }

    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Extracts theme information.
//--------------------------------------------------------------------
bool RomaImplContext::queryTheme(RomaThemeInfo& info,
                                 const RomaTheme& srcTheme,
                                 bool client)
{
    HKXCTHEME theme = (HKXCTHEME)srcTheme;

    TRACE_ENTRY(RomaImplContext::queryTheme, 
        << TRACE_HEX(theme) << TRACE(client));

    // Check that the object is connected.
    if (!checkConnected(KXCCALLTYPE_QUERYTHEME))
    {
        TRACE_INT_RETURN(false);
    }

    KXCBUFFER buf = { KXCBUFFER_DEFAULT };
    KXCULONG flags = (client ? KXCTID_REQWFLOW : KXCTID_RSPWFLOW) |
            KXCTID_THEMEID | KXCTID_RETRY | KXCTID_PTRAILER |
            KXCTID_ETEINFO | KXCTID_CONFIRM | KXCTID_VERSION_CURR;

    bool success = (kxcQueryTheme(
        m_handle, theme, flags, &buf) == TRUE);

    if (success)
    {
        TRACE_BUFFER_DUMP(buf);

        RomaImplProxy::setImpl(
            info, new RomaImplThemeInfo(buf.pBuffer, client));
    }

    kxcFreeBuffer(NULL_HKXC, &buf);
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Creates a new theme from the specified theme information.
//--------------------------------------------------------------------
bool RomaImplContext::buildTheme(RomaTheme& newTheme,
                                 const RomaTheme& curTheme,
                                 const RomaThemeInfo& info,
                                 unsigned options,
                                 bool close,
                                 bool client)
{
    HKXCTHEME theme = (HKXCTHEME)curTheme;
    RomaImplThemeInfo* ti = RomaImplProxy::getImplPtr(info);

    TRACE_ENTRY(RomaImplContext::buildTheme, 
        << TRACE_HEX(theme) << TRACE_HEX(options)
        << TRACE(close) << TRACE(client) << TRACE_HEX(ti));

    // Check that the object is connected.
    if (!checkConnected(KXCCALLTYPE_BUILDTHEME))
    {
        TRACE_INT_RETURN(false);
    }

    if (ti == NULL)
    {
        bool res = true;

        if (close)
        {
            newTheme = curTheme;
            RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
            setErrorStatus(true);
        }
        else
        {
            res = copyTheme(newTheme, curTheme);
        }

        TRACE_INT_RETURN(res);
    }

    HKXCTHEME res = NULL_HKXCTHEME;

    if (close)
    {
        res = theme;
    }
    // copy theme because the original must be preserved
    else if (!kxcBuildTheme(m_handle, theme, KXCTOPT_COPYTHEME, NULL, &res))
    {
        setErrorStatus(false);
        TRACE_INT_RETURN(false);
    }

    KXCBUFFER buf = { KXCBUFFER_DEFAULT };

    // set retry count

    if ((options & Roma::THEMEINFO_RETRYCOUNT) != 0)
    {
        KXCULONG retry = ti->getRetryCount();
        buf.pBuffer = &retry;
        buf.BufLen = sizeof(retry);

        TRACE_EVENT( << "Changing retry count to " << retry );

        if (!kxcBuildTheme(m_handle, res,
            KXCTOPT_SETRETRY | KXCTOPT_CLOSETHEME, &buf, &res))
        {
            setErrorStatus(false);
            TRACE_INT_RETURN(false);
        }

        if (close)
        {
            RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
            close = false;
        }
    }

    // set confirmation flags

    if ((options & Roma::THEMEINFO_CONFIRMFLAGS) != 0)
    {
        unsigned cf = ti->getConfirmFlags();
        KXCCONFIRMSTATUS conf;
        conf.Version = KXCCONFIRMSTATVERS;
        conf.CoA = ((cf & KXCPOPT_COA) != 0);
        conf.CoD = ((cf & KXCPOPT_COD) != 0);

        buf.pBuffer = &conf;
        buf.BufLen = sizeof(conf);

        TRACE_EVENT( << "Changing confirmation flags: " 
            << TRACE(conf.CoA) << TRACE(conf.CoD) );

        if (!kxcBuildTheme(m_handle, res,
                KXCTOPT_SETCONFIRM | KXCTOPT_CLOSETHEME, &buf, &res))
        {
            setErrorStatus(false);
            TRACE_INT_RETURN(false);
        }

        if (close)
        {
            RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
            close = false;
        }
    }

    // set theme id

    if ((options & Roma::THEMEINFO_THEMEID) != 0)
    {
        TRACE_EVENT( << "Changing theme ID" );

        buf.pBuffer = (void*)((const void*)info.getThemeId());
        buf.BufLen = Roma::ID_LENGTH;

        TRACE_BUFFER_DUMP(buf);

        if (!kxcBuildTheme(m_handle, res,
                KXCTOPT_SETTHEMEID | KXCTOPT_CLOSETHEME, &buf, &res))
        {
            setErrorStatus(false);
            TRACE_INT_RETURN(false);
        }

        if (close)
        {
            RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
            close = false;
        }
    }

    // set workflow

    if ((options & Roma::THEMEINFO_WORKFLOW) != 0)
    {
        TRACE_EVENT( << "Changing workflow" );

        RomaImplWorkflow* wflow = ti->getWorkflow();
        RomaImplRouteList* list = wflow->getRoutes();
        unsigned num = list->getCount();

        TRACE_EVENT( << "Number of routes: " << num );

        if (num)
        {
            size_t bsize = sizeof(KXCWROUTE) * num + sizeof(KXCWFLOW);

            TRACE_EVENT( << "Required buffer size: " << bsize );

            char* mem = new char[bsize];
            PKXCWFLOW pFlow = (PKXCWFLOW)mem;
            pFlow->Version = KXCWFLOW_VERSION;
            pFlow->NumRoutes = num;

            RomaImplRoute* route = list->getRoot();
            
            for (num = 0; route != NULL; 
                route = route->getNextItem(), ++num)
            {
                route->populate(pFlow->Routes[num]);
            }

            wflow->getWorkflowService()->populate(pFlow->Service);

            buf.pBuffer = mem;
            buf.BufLen = bsize;

            TRACE_BUFFER_DUMP(buf);

            if (!kxcBuildTheme(m_handle, res, KXCTOPT_CLOSETHEME |
                (client ? KXCTOPT_SETREQWFLOW : KXCTOPT_SETRSPWFLOW),
                &buf, &res))
            {
                delete[] mem;
                setErrorStatus(false);
                TRACE_INT_RETURN(false);
            }

            delete[] mem;

            if (close)
            {
                RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
                close = false;
            }
        }
    }

    // set trailers

    if ((options & Roma::THEMEINFO_TRAILERS) != 0)
    {
        TRACE_EVENT( << "Changing trailers" );

        RomaImplTrailerList* list = ti->getTrailers();
        RomaImplTrailer* trailer = list->getRoot();
        unsigned num = 0;
        size_t bsize = sizeof(KXCTHEMEPTRLHDR);

        // calculate required buffer size
        for (; trailer != NULL;
            trailer = trailer->getNextItem(), ++num)
        {
            unsigned long dlen = 0;
            trailer->getData(dlen);
            bsize += sizeof(KXCTHEMEPTRLDATA) + kxcRoundLength(dlen);
        }

        TRACE_EVENT( << "Number of trailers: " << num );

        if (num)
        {
            TRACE_EVENT( << "Required buffer size: " << bsize );

            char* mem = new char[bsize];
            PKXCTHEMEPTRLHDR pHdr = (PKXCTHEMEPTRLHDR)mem;
            pHdr->Version = KXCTHEMEPTRLHDRVERS;
            pHdr->Number = num;

            char* ptr = (char*)(pHdr + 1);
            trailer = list->getRoot();

            for (; trailer != NULL; trailer = trailer->getNextItem())
            {
                PKXCTHEMEPTRLDATA pData = (PKXCTHEMEPTRLDATA)ptr;
                pData->Version = KXCTHEMEPTRLDATAVERS;
                pData->Type = trailer->getType();
                pData->Method = trailer->getMethod();
                pData->Status = 0;
                ptr += sizeof(KXCTHEMEPTRLDATA);

                unsigned long dlen = 0;
                const void* td = trailer->getData(dlen);
                memcpy(ptr, td, dlen);
                pData->BufLen = dlen;
                ptr += kxcRoundLength(dlen);
            }

            buf.pBuffer = mem;
            buf.BufLen = bsize;

            TRACE_BUFFER_DUMP(buf);

            if (!kxcBuildTheme(m_handle, res,
                KXCTOPT_CLOSETHEME | KXCTOPT_SETPTRAILER, &buf, &res))
            {
                delete[] mem;
                setErrorStatus(false);
                TRACE_INT_RETURN(false);
            }

            delete[] mem;

            if (close)
            {
                RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
                close = false;
            }
        }
    }

    // set ETE info

    if ((options & Roma::THEMEINFO_ETEINFO) != 0)
    {
        TRACE_EVENT( << "Changing ETE info" );

        RomaImplEteInfo* ete = ti->getEteInfo();

        if (ete == NULL)
        {
            TRACE_EVENT( << "ETE info is NULL" );
        }
        else
        {
            buf.pBuffer = (void*)ete->getData(buf.BufLen);

            if (buf.pBuffer == NULL || !buf.BufLen)
            {
                TRACE_EVENT( << "No ETE data" );
            }
            else
            {
                TRACE_BUFFER_DUMP(buf);

                if (!kxcBuildTheme(m_handle, res,
                    KXCTOPT_CLOSETHEME | KXCTOPT_SETETEINFO, &buf, &res))
                {
                    setErrorStatus(false);
                    TRACE_INT_RETURN(false);
                }

                if (close)
                {
                    RomaImplProxy::getImpl(curTheme) = NULL_HKXCTHEME;
                }
            }
        }
    }

    RomaImplProxy::getImpl(newTheme) = res;
    setErrorStatus(true);
    TRACE_INT_RETURN(true);
}

//--------------------------------------------------------------------
// Extracts message information.
//--------------------------------------------------------------------
bool RomaImplContext::interpretMessage(RomaMessageInfo& result,
                                       const RomaImplMessage& msg,
                                       unsigned options)
{
    TRACE_ENTRY(RomaImplContext::interpretMessage, << TRACE_HEX(options));

    // Check that the object is connected.
    if (!checkConnected(KXCCALLTYPE_INTERPRETMSG))
    {
        TRACE_INT_RETURN(false);
    }

    PKXCMSG pMsg = (PKXCMSG)msg;
    TRACE_DUMP(pMsg, sizeof(KXCMSG),<< "KXCMSG");

    KXCMSGINFO info = { KXCMSGINFO_DEFAULT };

    bool success = (kxcInterpretMessage(
        m_handle, options, pMsg, &info) == TRUE);

    if (success)
    {
        TRACE_DUMP(&info, sizeof(KXCMSGINFO),<< "KXCMSGINFO");

        RomaImplProxy::setImpl(result, new RomaImplMsgInfo(info, *this));
        kxcFreeBuffer(NULL_HKXC, &info.Exception.UserData.UserBuf);
    }

    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}


//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplContext::trace(RomaTraceStream& s) const
{
    s << TRACE(m_handle);
}


//====================================================================
// The RomaImplClient class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplClient::RomaImplClient() 
{
}

//--------------------------------------------------------------------
// Send the request.
//--------------------------------------------------------------------
bool RomaImplClient::put(RomaClientRequest& request)
{
    TRACE_ENTRY(RomaImplClient::put, << TRACE(this)
                                     << TRACE(request));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_CLIENTPUT))
    {
        TRACE_INT_RETURN(false);
    }

    // Marshal the message.
    if (!RomaImplProxy::marshalRequest(request))
    {
        KXCO_SET_CPP_ERROR(KXCORC_MARSHAL_REQUEST);
        reportCppError(KXCOCALLTYPE_CLIENTPUT);
        TRACE_INT_RETURN(false);
    }

    // Put the marshaled request.
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    bool success = put(RomaImplProxy::getImpl(request), po);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Put the marshaled request.
//--------------------------------------------------------------------
bool RomaImplClient::put(RomaImplClientRequest& reqImpl, KXCPUTOPT& po)
{
    TRACE_ENTRY(RomaImplClient::put, << TRACE(this));

    // Setup the C API structures.
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    HKXCTHEME theme;
    reqImpl.populate(po, sync, theme);

    KXCSERVICE service = { KXCSERVICE_DEFAULT };
    kxcSetServiceName(&service, reqImpl.getServiceName());

    // Put the message.
    bool success = (kxcPutRequest(m_handle, &service,
                      (PKXCMSG)reqImpl, &po, &sync, &theme) == TRUE);

    if (success) 
    {
        // Save updated message properties.
        reqImpl.capture(po, theme);
        reqImpl.setIncomingILSData(po.ILSData.hFromILS, this);
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Setup the C API structures for the publish/subscribe operation.
//--------------------------------------------------------------------
bool RomaImplClient::preparePubSub(KXCPUTOPT& po, 
                                   KXCPSLIST& list,
                                   const RomaSubject& subject, 
                                   int operation, 
                                   int switches) const
{
    TRACE_ENTRY(RomaImplClient::preparePubSub, << TRACE(this));

    // Check the operation value.
    switch (operation) 
    {
    case KXCPSOPT_NONE:
    case KXCPSOPT_PUBLISH:
    case KXCPSOPT_SUBSCRIBE:
    case KXCPSOPT_UNSUBSCRIBE:
    case KXCPSOPT_DELETEPUB :
    case KXCPSOPT_REQUESTUPDATE :
        break;

    default:
        TRACE_INT_RETURN(false);
    }

    // Check the switches value.
    if (switches & ~(KXCPSOPT_RETAINPUBLICATION | KXCPSOPT_NEWPUBSONLY |
                     KXCPSOPT_ONREQUEST) != 0)
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the KXCPSLIST structure.
    kxcSetPSListDefault(&list);
    RomaImplProxy::getImpl(subject).populate(list.aSubject);
    list.NumSubjects = 1;
    list.TotalLen += sizeof(KXCSUBJECT);

    // Setup the KXCPUTOPT structure.
    kxcSetPutPSOptions(&po, operation | switches);
    kxcSetPutPSbuffer(&po, &list);

    TRACE_INT_RETURN(true);
}

//--------------------------------------------------------------------
// Publish the message on the specified subject.
//--------------------------------------------------------------------
bool RomaImplClient::publish(RomaClientRequest& publication, 
                             const RomaSubject& subject)
{
    TRACE_ENTRY(RomaImplClient::publish, << TRACE(this)
                                         << TRACE(publication)
                                         << TRACE(subject));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_PUBLISH))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };

    if (!preparePubSub(po, list, subject, KXCPSOPT_PUBLISH, KXCPSOPT_NONE))
    {
        TRACE_INT_RETURN(false);
    }

    // Marshal the message.
    if (!RomaImplProxy::marshalRequest(publication))
    {
        KXCO_SET_CPP_ERROR(KXCORC_MARSHAL_REQUEST);
        reportCppError(KXCOCALLTYPE_PUBLISH);
        TRACE_INT_RETURN(false);
    }

    // Put the publication.
    bool success = put(RomaImplProxy::getImpl(publication), po);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Publish the publication.
//--------------------------------------------------------------------
bool RomaImplClient::publish(RomaPublication& publication)
{
    TRACE_ENTRY(RomaImplClient::publish, << TRACE(this)
                                         << TRACE(publication));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_PUBLISH))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    RomaImplPublication& impl = RomaImplProxy::getImpl(publication);

    if (!preparePubSub(po, list, impl.getSubject(), KXCPSOPT_PUBLISH, 
            impl.getRetain() ? KXCPSOPT_RETAINPUBLICATION : KXCPSOPT_NONE))
    {
        TRACE_INT_RETURN(false);
    }

    // Marshal the message.
    if (!RomaImplProxy::marshalRequest(publication))
    {
        KXCO_SET_CPP_ERROR(KXCORC_MARSHAL_REQUEST);
        reportCppError(KXCOCALLTYPE_PUBLISH);
        TRACE_INT_RETURN(false);
    }

    // Put the publication.
    bool success = put(impl, po);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Subscribe to publications on the specified subject.
//--------------------------------------------------------------------
bool RomaImplClient::subscribe(RomaClientRequest& request, 
                               const RomaSubject& subject)
{
    TRACE_ENTRY(RomaImplClient::subscribe, << TRACE(this)
                                           << TRACE(request) 
                                           << TRACE(subject));
 
    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SUBSCRIBE))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };

    if (!preparePubSub(po, list, subject, KXCPSOPT_SUBSCRIBE, KXCPSOPT_NONE))
    {
        TRACE_INT_RETURN(false);
    }

    // Put the subscription.
    bool success = put(RomaImplProxy::getImpl(request), po);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Subscribe to publications.
//--------------------------------------------------------------------
bool RomaImplClient::subscribe(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaImplClient::subscribe, << TRACE(this)
                                           << TRACE(subscription));
 
    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_SUBSCRIBE))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    RomaImplSubscription& impl = RomaImplProxy::getImpl(subscription);

    int switches = 
        impl.getNewPubsOnly() ? KXCPSOPT_NEWPUBSONLY : KXCPSOPT_NONE;

    if (impl.getOnRequest())
    {
        switches |= KXCPSOPT_ONREQUEST;
    }

    if (!preparePubSub(po, list, impl.getSubject(), 
            KXCPSOPT_SUBSCRIBE, switches))
    {
        TRACE_INT_RETURN(false);
    }

    // Put the subscription.
    bool success = put(impl, po);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Request updated publications.
//--------------------------------------------------------------------
bool RomaImplClient::requestUpdates(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaImplClient::requestUpdates, << TRACE(this)
                                                << TRACE(subscription));
 
    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_REQUESTUPDATES))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    RomaImplSubscription& impl = RomaImplProxy::getImpl(subscription);

    // Put the request.
    bool success = (preparePubSub(po, list, impl.getSubject(), 
        KXCPSOPT_REQUESTUPDATE, KXCPSOPT_NONE) && put(impl, po));

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Delete retained publication.
//--------------------------------------------------------------------
bool RomaImplClient::deletePublication(RomaPublication& publication)
{
    TRACE_ENTRY(RomaImplClient::deletePublication, << TRACE(this)
                                                   << TRACE(publication));
 
    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_DELETEPUBLICATION))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    RomaImplPublication& impl = RomaImplProxy::getImpl(publication);

    // Put the request.
    bool success = (preparePubSub(po, list, impl.getSubject(), 
        KXCPSOPT_DELETEPUB, KXCPSOPT_NONE) && put(impl, po));

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Unsubscribe from publications on the specified subject.
//--------------------------------------------------------------------
bool RomaImplClient::unsubscribe(RomaClientRequest& request, 
                                 const RomaSubject& subject)
{
    TRACE_ENTRY(RomaImplClient::unsubscribe, << TRACE(this)
                                             << TRACE(request)
                                             << TRACE(subject));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_UNSUBSCRIBE))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };

    // Put the request.
    bool success = (preparePubSub(po, list, subject, KXCPSOPT_UNSUBSCRIBE, 
        KXCPSOPT_NONE) && put(RomaImplProxy::getImpl(request), po));

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Unsubscribe from publication.
//--------------------------------------------------------------------
bool RomaImplClient::unsubscribe(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaImplClient::unsubscribe, << TRACE(this)
                                             << TRACE(subscription));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_UNSUBSCRIBE))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    RomaImplSubscription& impl = RomaImplProxy::getImpl(subscription);

    // Put the request.
    bool success = (preparePubSub(po, list, impl.getSubject(), 
        KXCPSOPT_UNSUBSCRIBE, KXCPSOPT_NONE) && put(impl, po));

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Unsubscribe from all publications.
//--------------------------------------------------------------------
bool RomaImplClient::unsubscribeAll(RomaSubscription& subscription)
{
    TRACE_ENTRY(RomaImplClient::unsubscribeAll, << TRACE(this)
                                                << TRACE(subscription));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_UNSUBSCRIBEALL))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    KXCPSLIST list;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };

    kxcSetPutPSOptions(&po, KXCPSOPT_UNSUBSCRIBEALL);
    kxcSetPSListDefault(&list);
    list.NumSubjects = 0;
    kxcSetPutPSbuffer(&po, &list);

    // Put the request.
    bool success = put(RomaImplProxy::getImpl(subscription), po);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Get a message from the queue.
// Message data buffers will be auto-allocated.
//--------------------------------------------------------------------
RomaClientResponse* RomaImplClient::get(RomaClient& cl,
                                        RomaGetOptions& opt)
{
    TRACE_ENTRY(RomaImplClient::get, << TRACE(this) << TRACE(opt));

    // Setup the C API structures.
    KXCMSG msg = { KXCMSG_DEFAULT };
    KXCGETOPT go = { KXCGETOPT_DEFAULT };
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    HKXCTHEME theme;
    RomaImplGetOptions& impl = RomaImplProxy::getImpl(opt);

    impl.populate(go, sync, theme);
    go.ReceiveOptions |= KXCGOPT_AUTO_ALLOC;

    // Get a message.
    RomaClientResponse* response = realGet(cl, msg, go, sync, theme);

    if (response)
    {
        // Save updated properties of get options.
        impl.capture(go, theme);
    }

    TRACE_INT_RETURN(response);
}

//--------------------------------------------------------------------
// Get a message from the queue.
// Message data will be placed into the specified buffers.
//--------------------------------------------------------------------
RomaClientResponse* RomaImplClient::get(RomaClient& cl,
                                        RomaGetOptions& opt,
                                        void* msgBuffer, int& msgLength,
                                        void* tagBuffer, int& tagLength)
{
    TRACE_ENTRY(RomaImplClient::get, << TRACE(this) 
                                     << TRACE(opt)
                                     << TRACE(msgBuffer)
                                     << TRACE(tagBuffer));

    // Setup the C API structures.
    KXCMSG msg = { KXCMSG_DEFAULT };
    KXCGETOPT go = { KXCGETOPT_DEFAULT };
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    HKXCTHEME theme;

    RomaImplProxy::getImpl(opt).populate(go, sync, theme);
    go.ReceiveOptions &= ~KXCGOPT_AUTO_ALLOC;

    kxcSetMsgBuffer(&msg, msgLength, msgBuffer);
    kxcSetTagBuffer(&msg, tagLength, tagBuffer);

    // Get a message.
    RomaClientResponse* response = realGet(cl, msg, go, sync, theme);

    if (response)
    {
        // Save updated properties of get options.
        RomaImplProxy::getImpl(opt).capture(go, theme);
    }
    else
    {
        // If the KXCRC_BUFFER_TOO_SMALL error occured, we should
        // provide the application with the required buffer length.
        msgLength = msg.MsgLen;
        tagLength = msg.TagLen;
    }

    TRACE_INT_RETURN(response);
}

//--------------------------------------------------------------------
// Get a message from the queue.
//--------------------------------------------------------------------
RomaClientResponse* RomaImplClient::realGet(RomaClient& cl,
                                            KXCMSG& msg, 
                                            KXCGETOPT& go,
                                            KXCSYNC& sync, 
                                            HKXCTHEME theme)
{
    TRACE_ENTRY(RomaImplClient::realGet, << TRACE(this));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_CLIENTGET))
    {
        TRACE_INT_RETURN(NULL);
    }

    // If a wait interval is set, make sure we indicate that it
    // should be used.
    if (go.WaitLen)
    {
        go.ReceiveOptions |= KXCGOPT_WAIT;
    }

    // Delete the last error.
    resetError();

    // Get a message.
    if (!kxcGetResponse(m_handle, &msg, &go, &sync, &theme))
    {
        // If there is no message on the queue or a message is
        // suppressed, we don't set an error on the object, but
        // just return NULL. However, if we are working under a
        // syncpoint, we should report the KXCRC_MESSAGE_SUPPRESSED
        // error.
        if (go.Error.LastReturnCode != KXCRC_NO_MSG_AVAILABLE &&
            (go.Error.LastReturnCode != KXCRC_MESSAGE_SUPPRESSED ||
             sync.TransactionType != KXCSYNC_NONE)) 
        {
            // Set the last error on the object.
            setErrorStatus(false);
        }

        TRACE_INT_RETURN(NULL);
    }

    // Create the RomaImplClientResponse from the KXCMSG.
    RomaClientResponse* response = NULL;
    RomaClientResponse* original = RomaImplProxy::createClientResponse(
        new RomaImplMsgIncoming(msg, theme, go.ILSData.hFromILS, *this));

    // Unmarshal the message.
    if (!RomaImplProxy::createResponse(cl, *original, response) ||
        !response)
    {
        KXCO_SET_CPP_ERROR(KXCORC_CREATE_RESPONSE);
        reportCppError(KXCOCALLTYPE_CLIENTGET, &msg, theme, &sync);
    }

    delete original;
    TRACE_INT_RETURN(response);
}


//====================================================================
// The RomaImplElement class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplElement::RomaImplElement() : m_termIndicator(false)
{
}

//--------------------------------------------------------------------
// Put the message.
//--------------------------------------------------------------------
bool RomaImplElement::put(RomaBeResponse& response)
{
    TRACE_ENTRY(RomaImplElement::put, << TRACE(this) 
                                      << TRACE(response));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_BEPUT))
    {
        TRACE_INT_RETURN(false);
    }

    // Marshal the message.
    if (!RomaImplProxy::marshalResponse(response))
    {
        KXCO_SET_CPP_ERROR(KXCORC_MARSHAL_RESPONSE);
        reportCppError(KXCOCALLTYPE_BEPUT);
        TRACE_INT_RETURN(false);
    }

    // Setup the C API structures.
    HKXCTHEME theme;
    KXCPUTOPT po = { KXCPUTOPT_DEFAULT };
    KXCSYNC syncDef  = { KXCSYNC_DEFAULT };
    RomaImplElementResponse& impl = RomaImplProxy::getImpl(response);
    impl.populate(po, syncDef, theme);

    // Put the message.
    bool success = (kxcPutResponse(m_handle, (PKXCMSG)impl, 
                           &po, &syncDef, &theme) == TRUE);

    if (success) 
    {
        // Save a termination indicator state and updated message
        // properties.
        m_termIndicator = ((po.Status & KXCPOPTSTAT_TERMINATE) != 0);
        impl.capture(po, theme);
        impl.setIncomingILSData(po.ILSData.hFromILS, this);
    }

    // If the operation failed, set the last error on the object.
    setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Get a message from the queue.
// Message buffers will be auto-allocated.
//--------------------------------------------------------------------
RomaBeRequest* RomaImplElement::get(RomaBusinessElement& elem,
                                    RomaGetOptions& opt)
{
    TRACE_ENTRY(RomaImplElement::get, << TRACE(this));

    KXCMSG message = { KXCMSG_DEFAULT };

    // Get a message.
    RomaBeRequest* request = realGet(message, opt, elem);

    TRACE_INT_RETURN(request);
}

//--------------------------------------------------------------------
// Get a message from the queue.
// Message data will be placed into the specified buffers.
//--------------------------------------------------------------------
RomaBeRequest* RomaImplElement::get(RomaBusinessElement& elem,
                                    RomaGetOptions& opt,
                                    void* msgBuffer, 
                                    int& msgLength,
                                    void* tagBuffer, 
                                    int& tagLength)
{
    TRACE_ENTRY(RomaImplElement::get, << TRACE(this)
        << TRACE(msgBuffer) << TRACE(tagBuffer));

    // Setup the KXCMSG structure.
    KXCMSG message = { KXCMSG_DEFAULT };
    kxcSetMsgBuffer(&message, msgLength, msgBuffer);
    kxcSetTagBuffer(&message, tagLength, tagBuffer);

    // Get a message.
    RomaBeRequest* request = realGet(message, opt, elem, false);

    if (!request) 
    {
        // If the KXCRC_BUFFER_TOO_SMALL error occured, we should
        // provide the application with the required buffer length.
        msgLength = message.MsgLen;
        tagLength = message.TagLen;
    }

    TRACE_INT_RETURN(request);
}

//--------------------------------------------------------------------
// Get a message from the queue.
//--------------------------------------------------------------------
RomaBeRequest* RomaImplElement::realGet(KXCMSG& message, 
                                        RomaGetOptions& opt,
                                        RomaBusinessElement& elem,
                                        bool bAutoAlloc)
{
    TRACE_ENTRY(RomaImplElement::realGet, << TRACE(this));

    // Check that the object is connected.
    if (!checkConnected(KXCOCALLTYPE_BEGET))
    {
        TRACE_INT_RETURN(NULL);
    }

    // Setup the C API structures.
    KXCGETOPT go = { KXCGETOPT_DEFAULT };
    KXCSYNC sync = { KXCSYNC_DEFAULT };
    HKXCTHEME theme;
    const RomaImplGetOptions& impl = RomaImplProxy::getImpl(opt);

    impl.populate(go, sync, theme);

    // Delete the last error.
    resetError();
    
    // Indicate whether the message buffers must be auto-allocated.
    if (bAutoAlloc)
    {
        go.ReceiveOptions |= KXCGOPT_AUTO_ALLOC;
    }
    else
    {
        go.ReceiveOptions &= ~KXCGOPT_AUTO_ALLOC;
    }

    // Get a message.
    bool success = (kxcGetRequest(m_handle, &message, 
                         &go, &sync, &theme) == TRUE);

    if (!success)
    {
        // If there is no message on the queue or a message is
        // suppressed, we don't set an error on the object, but
        // just return NULL. However, if we are working under a
        // syncpoint, we should report the KXCRC_MESSAGE_SUPPRESSED
        // error.
        if (go.Error.LastReturnCode != KXCRC_NO_MSG_AVAILABLE &&
            (go.Error.LastReturnCode != KXCRC_MESSAGE_SUPPRESSED ||
             sync.TransactionType != KXCSYNC_NONE)) 
        {
            TRACE_WARNING(<< "kxcGetRequest() failed");
            setErrorStatus(false);
        }

        TRACE_INT_RETURN(NULL);
    }

    // Save a termination indicator state.
    m_termIndicator = ((go.Status & KXCGOPTSTAT_TERMINATE) != 0);

    RomaBeRequest* updated = NULL;
    RomaBeRequest* original = RomaImplProxy::createBeRequest(
        new RomaImplMsgIncoming(message, theme, go.ILSData.hFromILS, *this));

    // Unmarshal the message.
    if (!RomaImplProxy::createRequest(elem, *original, updated) ||
        !updated)
    {
        KXCO_SET_CPP_ERROR(KXCORC_CREATE_REQUEST);
        reportCppError(KXCOCALLTYPE_BEGET, &message, theme, &sync);
    }
    else
    {
        // Save updated properties of get options.
        RomaImplProxy::getImpl(opt).capture(go, theme);
    }

    delete original;
    TRACE_INT_RETURN(updated);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplElement::trace(RomaTraceStream& s) const
{
    RomaImplContext::trace(s);
    s << TRACE(m_termIndicator);
}


//====================================================================
// The RomaImplStorageQueue class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplStorageQueue::RomaImplStorageQueue() 
    : m_handle(NULL_HKXCSTORQ), m_context(NULL)
{
}

//--------------------------------------------------------------------
// Destructor.
// Closes the queue.
//--------------------------------------------------------------------
RomaImplStorageQueue::~RomaImplStorageQueue()
{
    // Close the queue if it's open.
    if (isOpen())
    {
        close();
    }

    // Release the shared context.
    if (m_context)
    {
        m_context->releaseInstance();
    }
}

//--------------------------------------------------------------------
// Open the storage queue.
//--------------------------------------------------------------------
bool RomaImplStorageQueue::open(RomaImplContext& context, 
                                const char* queueName,
                                KXCULONG options,
                                const char* queueModel)
{
    TRACE_ENTRY(RomaImplStorageQueue::open, << TRACE(this)
                                            << TRACE(context));

    // Check whether the queue is open already.
    if (m_context && (m_handle != NULL_HKXCSTORQ))
    {
        context.KXCO_SET_CPP_ERROR(KXCORC_ALREADY_OPEN);
        context.reportCppError(KXCOCALLTYPE_SQOPEN);
        TRACE_INT_RETURN(false);
    }

    // Open the queue using the context.
    m_handle = context.openStorageQueue(queueName, options, queueModel);

    bool success = (m_handle != NULL_HKXCSTORQ);

    if (success)
    {
        // Capture the instance of the specified context.
        if (m_context)
        {
            m_context->releaseInstance();
        }

        context.lockInstance();
        m_context = &context;
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Close the storage queue.
//--------------------------------------------------------------------
bool RomaImplStorageQueue::close()
{
    TRACE_ENTRY(RomaImplStorageQueue::close, << TRACE(this));

    bool success = 
        isOpen() ? m_context->closeStorageQueue(m_handle) : true;

    if (success)
    {
        m_handle = NULL_HKXCSTORQ;

        // Release the shared context.
        if (m_context)
        {
            m_context->releaseInstance();
            m_context = NULL;
        }
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Store a message on the storage queue.
//--------------------------------------------------------------------
bool RomaImplStorageQueue::store(RomaStorable& message,
                                 const RomaStoreOptions& options)
{
    TRACE_ENTRY(RomaImplStorageQueue::store, << TRACE(this));

    // Check that the queue is open.
    if (!checkOpen(KXCOCALLTYPE_SQSTORE))
    {
        TRACE_INT_RETURN(false);
    }

    // Store the message.
    bool success = m_context->storeStorable(m_handle, message, options);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Retrieve a message from the queue.
// Message data will be placed into the specified buffer.
//--------------------------------------------------------------------
RomaStorable*
RomaImplStorageQueue::retrieve(RomaStorageQueue& sq,
                               void* buffer, 
                               int& length,
                               const RomaRetrieveOptions& options)
{
    TRACE_ENTRY(RomaImplStorageQueue::retrieve, << TRACE(this) 
                                                << TRACE(buffer)
                                                << TRACE(length)
                                                << TRACE(options));

    // Check that the queue is open.
    if (!checkOpen(KXCOCALLTYPE_SQRETRIEVE))
    {
        TRACE_INT_RETURN(NULL);
    }

    // Setup the KXCMSG structure.
    KXCMSG msg = { KXCMSG_DEFAULT };
    kxcSetMsgBuffer(&msg, length, buffer);

    // Retrieve a message.
    RomaStorable* stor = 
        m_context->retrieveStorable(m_handle, msg, options, sq, true);

    if (!stor) 
    {
        // If the KXCRC_BUFFER_TOO_SMALL error occured, we should
        // provide the application with the required buffer length.
        length = msg.MsgLen;
        TRACE_EVENT(<< "Required buffer size: " << length);
    }

    TRACE_INT_RETURN(stor);
}

//--------------------------------------------------------------------
// Retrieve a message from the queue.
// Message buffers will be auto-allocated.
//--------------------------------------------------------------------
RomaStorable*
RomaImplStorageQueue::retrieve(RomaStorageQueue& sq, 
                               const RomaRetrieveOptions& options)
{
    TRACE_ENTRY(RomaImplStorageQueue::retrieve, << TRACE(this) 
                                                << TRACE(options));

    // Check that the queue is open.
    if (!checkOpen(KXCOCALLTYPE_SQRETRIEVE))
    {
        TRACE_INT_RETURN(NULL);
    }

    // Retrieve a message.
    KXCMSG msg = { KXCMSG_DEFAULT };
    RomaStorable* stor = 
        m_context->retrieveStorable(m_handle, msg, options, sq, false);

    TRACE_INT_RETURN(stor);
}

//--------------------------------------------------------------------
// Raise the error if the queue is not open.
//--------------------------------------------------------------------
bool RomaImplStorageQueue::checkOpen(KXCULONG callType)
{
    TRACE_ENTRY(RomaImplStorageQueue::checkOpen, << TRACE(this));

    bool success = true;

    if (!isOpen())
    {
        success = false;

        // Set the error on the context.
        if (m_context)
        {
            m_context->KXCO_SET_CPP_ERROR(KXCORC_NOT_OPEN);
            m_context->reportCppError(callType);
        }
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Indicates whether the queue is open.
//--------------------------------------------------------------------
bool RomaImplStorageQueue::isOpen() const
{
    if (!m_context || m_handle == NULL_HKXCSTORQ)
    {
        return false;
    }

    // Do test query.
    KXCULONG options = KXCCID_ELEMENTTYPE;
    KXCBUFFER info = { KXCBUFFER_DEFAULT };
    HKXC h = (HKXC)(*m_context);
    KXCBOOL status = kxcQueryContext(h, options, &info);

    kxcFreeBuffer(NULL_HKXC, &info);
    return (status == TRUE);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplStorageQueue::trace(RomaTraceStream& s) const
{
    s << TRACE(m_handle) << TRACE(m_context);
}

