//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcorda.cpp,v 1.12 2004/01/22 17:53:26 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcorda.cpp,v $
// $Revision: 1.12 $
//--------------------------------------------------------------------
// The RomaDirectoryItem and the RomaDirectoryAgent public interfaces.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoidir.hpp>
#include <kxcoipxy.hpp>
#include <kxcorda.hpp>

TRACE_MODULE_VERSION($Revision: 1.12 $);


//====================================================================
// The RomaDirectoryItem class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaDirectoryItem::RomaDirectoryItem()
{
    m_impl = new RomaImplDirItem;
}

//--------------------------------------------------------------------
// Constructor creating the object using an existing implementation
// instance.
//--------------------------------------------------------------------
RomaDirectoryItem::RomaDirectoryItem(RomaImplDirItem* impl) 
    : m_impl(impl)
{
}

//--------------------------------------------------------------------
// Constructor.
// Creates the item with the distinguished name.
//--------------------------------------------------------------------
RomaDirectoryItem::RomaDirectoryItem(const char* dn,
                                     const char* objectClass)
{
    m_impl = new RomaImplDirItem(dn, objectClass);
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaDirectoryItem::~RomaDirectoryItem()
{
    m_impl->releaseInstance();
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaDirectoryItem::RomaDirectoryItem(const RomaDirectoryItem& di)
    : m_impl(di.m_impl)
{
    m_impl->lockInstance();
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaDirectoryItem& 
RomaDirectoryItem::operator=(const RomaDirectoryItem& di)
{
    KXCO_ASSIGN_SHARED(di);
}

//--------------------------------------------------------------------
// Set a distinguished name.
//--------------------------------------------------------------------
void RomaDirectoryItem::setDN(const char* dn)
{
    m_impl->setDN(dn);
}

//--------------------------------------------------------------------
// Add an attribute to the item.
//--------------------------------------------------------------------
void RomaDirectoryItem::addAttribute(const char* name, 
                                     const char* value)
{
    m_impl->addAttribute(name, value);
}

//--------------------------------------------------------------------
// Get a distinguished name.
//--------------------------------------------------------------------
const char* RomaDirectoryItem::dn() const
{
    return m_impl->getDN();
}

//--------------------------------------------------------------------
// Get a number of attributes.
//--------------------------------------------------------------------
int RomaDirectoryItem::attributes() const
{
    return m_impl->getNumber();
}

//--------------------------------------------------------------------
// Get a name of an attribute with the specified index.
//--------------------------------------------------------------------
const char* RomaDirectoryItem::attributeName(int a) const
{
    return m_impl->getAttrName(a);
}

//--------------------------------------------------------------------
// Get a value of an attribute with the specified index.
//--------------------------------------------------------------------
const char* RomaDirectoryItem::attributeValue(int a) const
{
    return m_impl->getAttrValue(a);
}

//--------------------------------------------------------------------
// Get a value of an attribute with the specified name and index.
//--------------------------------------------------------------------
const char* 
RomaDirectoryItem::attributeValue(const char* name, int i) const
{
    return m_impl->getAttrValue(name, i);
}

//--------------------------------------------------------------------
// Specify true if the item must already be present in the directory
// before RomaDirectoryAgent::storeItem() is called, i.e. the item
// must be replaced, not added.
//--------------------------------------------------------------------
void RomaDirectoryItem::setItemMustExist(bool f)
{
    m_impl->setMustExist(f);
}

//--------------------------------------------------------------------
// Specify true if the item must not be present in the directory
// before RomaDirectoryAgent::storeItem() is called, i.e. the item
// must be added, not replaced.
//--------------------------------------------------------------------
void RomaDirectoryItem::setItemMustNotExist(bool f)
{
    m_impl->setMustExist(!f);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaDirectoryItem::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaDirectoryItem, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}


//====================================================================
// The RomaDirectoryAgent class.
// Provides ability to retrieve and update contents of a LDAP
// directory. It uses already established connection of a RomaClient
// or a RomaBusinessElement.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaDirectoryAgent::RomaDirectoryAgent()
{
    TRACE_ENTRY(RomaDirectoryAgent::RomaDirectoryAgent, 
        << TRACE(this));

    m_impl = new RomaImplDirAgent;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor. 
// Performs items lookup in a directory using a connection of the
// specified RomaClient.
//--------------------------------------------------------------------
RomaDirectoryAgent::RomaDirectoryAgent(RomaClient& client,
                                       const char* startingPoint,
                                       const char* searchName,
                                       Roma::SearchType searchType,
                                       Roma::SearchOption option)
{
    TRACE_ENTRY(RomaDirectoryAgent::RomaDirectoryAgent, 
        << TRACE(this));

    m_impl = new RomaImplDirAgent;
    m_impl->lookup(RomaImplProxy::getImpl(client), startingPoint, 
        searchName, searchType, option);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor. 
// Performs items lookup in a directory using a connection of the
// specified RomaBusinessElement.
//--------------------------------------------------------------------
RomaDirectoryAgent::RomaDirectoryAgent(RomaBusinessElement& element,
                                       const char* startingPoint,
                                       const char* searchName,
                                       Roma::SearchType searchType,
                                       Roma::SearchOption option)
{
    TRACE_ENTRY(RomaDirectoryAgent::RomaDirectoryAgent, 
        << TRACE(this));

    m_impl = new RomaImplDirAgent;
    m_impl->lookup(RomaImplProxy::getImpl(element), startingPoint, 
        searchName, searchType, option);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaDirectoryAgent::~RomaDirectoryAgent()
{
    TRACE_ENTRY(RomaDirectoryAgent::~RomaDirectoryAgent, 
        << TRACE(this));

    delete m_impl;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Lookup items in a directory using a connection of the specified
// RomaClient.
//--------------------------------------------------------------------
bool RomaDirectoryAgent::lookup(RomaClient& client,
                                const char* startingPoint,
                                const char* searchName,
                                Roma::SearchType searchType,
                                Roma::SearchOption option)
{
    TRACE_ENTRY(RomaDirectoryAgent::lookup, << TRACE(this));

    bool success = m_impl->lookup(RomaImplProxy::getImpl(client), 
        startingPoint, searchName, searchType, option);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Lookup items in a directory using a connection of the specified
// RomaBusinessElement.
//--------------------------------------------------------------------
bool RomaDirectoryAgent::lookup(RomaBusinessElement& element,
                                const char* startingPoint,
                                const char* searchName,
                                Roma::SearchType searchType,
                                Roma::SearchOption option)
{
    TRACE_ENTRY(RomaDirectoryAgent::lookup, << TRACE(this));

    bool success = m_impl->lookup(RomaImplProxy::getImpl(element), 
        startingPoint, searchName, searchType, option);

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Get a number of found items.
//--------------------------------------------------------------------
int RomaDirectoryAgent::matches()
{
    TRACE_ENTRY(RomaDirectoryAgent::matches, << TRACE(this));

    int res = m_impl->getMatches();

    TRACE_INT_RETURN(res);
}

//--------------------------------------------------------------------
// Get a number of found items and indicate whether the dirctory was
// modified since the last lookup.
//--------------------------------------------------------------------
int RomaDirectoryAgent::matches(bool& modified)
{
    TRACE_ENTRY(RomaDirectoryAgent::matches(bool&), << TRACE(this));

    int res = m_impl->getMatches(modified);

    TRACE_INT_RETURN(res);
}

//--------------------------------------------------------------------
// Get an item by the index.
//--------------------------------------------------------------------
RomaDirectoryItem* RomaDirectoryAgent::item(int i)
{
    return m_impl->getItem(i);
}

//--------------------------------------------------------------------
// Get an item by the index and indicate whether the dirctory was
// modified since the last lookup.
//--------------------------------------------------------------------
RomaDirectoryItem* RomaDirectoryAgent::item(int i, bool& modified)
{
    return m_impl->getItem(i, modified);
}

//--------------------------------------------------------------------
// Calls the action() method for each found item.
//--------------------------------------------------------------------
int RomaDirectoryAgent::processAll()
{
    TRACE_ENTRY(RomaDirectoryAgent::processAll, << TRACE(this));

    bool b;
    int n = m_impl->getMatches(b);
    int i = 0;

    for (; i < n; ++i) 
    {
        RomaDirectoryItem* item = m_impl->getItem(i, b);
        b = action(item);
        delete item;

        if (!b)
        {
            break;
        }
    }

    TRACE_INT_RETURN(i);
}

//--------------------------------------------------------------------
// Should be overriden by an application to perform an item processing
// on the processAll() method invocation. The default implementation
// does nothing.
//--------------------------------------------------------------------
bool RomaDirectoryAgent::action(RomaDirectoryItem*)
{
    return true;
}

//--------------------------------------------------------------------
// Remove the item from the directory.
//--------------------------------------------------------------------
bool RomaDirectoryAgent::deleteItem(const char* dn)
{
    return m_impl->deleteItem(dn, NULL, NULL);
}

//--------------------------------------------------------------------
// Remove the item from the directory.
//--------------------------------------------------------------------
bool RomaDirectoryAgent::deleteItem(const char* dn,
			                        const char* id,
			                        const char* pw)
{
    return m_impl->deleteItem(dn, id, pw);
}

//--------------------------------------------------------------------
// Store the item in the directory.
//--------------------------------------------------------------------
bool RomaDirectoryAgent::storeItem(RomaDirectoryItem& item)
{
    return m_impl->storeItem(item, NULL, NULL);
}

//--------------------------------------------------------------------
// Store the item in the directory.
//--------------------------------------------------------------------
bool RomaDirectoryAgent::storeItem(RomaDirectoryItem& item,
			                       const char* id,
			                       const char* pw)
{
    return m_impl->storeItem(item, id, pw);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaDirectoryAgent::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaDitecoryAgent, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

