//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaBaseGetOptions.java
//
// Defines      : class RomaBaseGetOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates the common options by which a Roma message
* is retrieved.
*/
class RomaBaseGetOptions
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // theme allocation control mask
  final int THEME_USAGE = Kxc.POPT_NEWTHEME | Kxc.POPT_USETHEME;
  
  RomaSecurityDef  m_sec = null;
  private long m_incomingILSDataHandle = Kxc.NULL_ILS_DATA_HANDLE;
  long m_outgoingILSDataHandle = Kxc.NULL_ILS_DATA_HANDLE;
  private int m_status;
  
  byte       m_msgId[];
  byte       m_correlId[];
  int        m_waitInterval = Kxc.GOWAIT_INDEFINITE;
  int        m_opts         = Kxc.GOPT_NONE;
  RomaTheme  m_theme        = new RomaTheme();
  String     m_apiProfileName;
  String     m_errorProfileName;
  int        m_subjectStyle;    
  
  private RomaSyncpointDef m_sync = new RomaSyncpointDef( Roma.SYNC_PROFILE );
  
  /**
  * Selects whether or not a new theme will be allocated
  * when applying these options
  * 
  * @param flag  if true theme already associated with the 
  * RomaMessage will be used. If false the new theme 
  * allocation will be done.
  */
  public void setUseTheme(boolean b)
  {
    m_opts &= ~THEME_USAGE;
    if (b)
      m_opts |= Kxc.POPT_USETHEME;
  }
  
  /**
  * Selects whether not to get message by matching theme
  * when applying these options
  * 
  * @param flag  if true the theme will be ignored  
  *							If false the theme will be matched 
  *  
  */
  public void setAnyTheme(boolean b)
  {
    setUseTheme(!b);	
  }
  
  /**
  * Sets the API profile name.
  * 
  * @param name  the name of the API profile.
  */
  public void setApiProfileName( String name )
  {
    m_apiProfileName = name;
  }
  
  /**
  * Indicates that flags and options specified in the API profile
  * should be used for the Roma put operation.
  * <p>
  * If this option is chosen but the specified profile is not
  * available then default values are used for the put options.
  */
  public void useApiProfile( boolean b )
  {
    if (b)
      m_opts |= Kxc.POPT_USEDEFAULT;
    else
      m_opts &= ~Kxc.POPT_USEDEFAULT;
  }
  
  /**
  * Sets the error profile name.
  * 
  * @param name  the name of the error profile.
  */
  public void setErrorProfileName( String name )
  {
    m_errorProfileName = name;
  }
  
  //------------------------------------------------------------------
  // OPERATION is the product of logically OR'ing together all of the
  // options associated with message read-in. The resulting mask 
  // can be logically AND'd with the m_opts member so that only those
  // bits in the m_opts member associated with message getting are
  // altered.
  //------------------------------------------------------------------
  private final int OPERATION = Roma.GET_FIRST | Roma.PEEK_FIRST |
    Roma.PEEK_NEXT | Roma.PEEK_SAME | Roma.GET_PEEKED;
  
  /**
  * Set only the receive options of the object.
  *
  * @param op  the new message receive options. It is expected that
  *            this will be one of the following values :-
  *            <ul>
  *            <li> GET_FIRST
  *            <li> PEEK_FIRST
  *            <li> PEEK_NEXT
  *            <li> PEEK_SAME
  *            <li> GET_PEEKED
  *            </ul>
  *
  */
  public void setOperation( int op )
  {
    m_opts &= ~OPERATION;
    m_opts |= (op & OPERATION);
  }
  
  //------------------------------------------------------------------
  // SELECTOR is the product of logically OR'ing together all of the
  // options associated with message finding. The resulting mask 
  // can be logically AND'd with the m_opts member so that only those
  // bits in the m_opts member associated with message finding are
  // altered.
  //------------------------------------------------------------------
  private final int SELECTOR = Roma.BY_MSGID|Roma.BY_CORRELID|Roma.BY_THEME;
  
  /**
  * Sets the message selection criteria.
  *
  * @param sel  the new message selection options.
  *             It is expected that
  *             this will be one of the following values :-
  *             <ul>
  *             <li> BY_NONE
  *             <li> BY_THEME
  *             <li> BY_MSGID
  *             <li> BY_CORRELID
  *             <li> BY_MSGID_AND_CORRELID
  *             <li> BY_MSGID_AND_THEME
  *             </ul>
  * 
  */
  public void setSelector( int sel )
  {
    m_opts &= ~SELECTOR;
    m_opts |= (sel & SELECTOR);
  }
  
  /**
  * Sets the theme.
  *
  * @param theme  the new theme.
  * 
  */
  public void setTheme( RomaTheme theme )
  {
    m_theme = theme;
  }
  
  /**
  * Returns current security buffer associated with this context.
  */
  byte[] getSecurityBuffer()
    throws RomaError
  {
    if ( m_sec == null )
      return null;
    return m_sec.getBuffer();
  }
  
  /**
  * Sets the message identifier.
  *
  * @param msgId  the new message identifier value
  *
  * @see #msgId()
  */
  public void setMsgId(byte[] msgId)
  {
    m_msgId = msgId;
  }
  
  /**
  * Sets the correlation identifier.
  *
  * @param correlId  the new correlation identifier value.
  *
  * @see #correlId()
  */
  public void setCorrelId(byte[] correlId)
  {
    m_correlId = correlId;
  }
  
  /**
  * Sets the wait interval in milliseconds.
  *
  * @param waitInterval  the new wait interval value in milliseconds.
  *
  */
  public void setWaitInterval( int waitInterval )
  {
    m_waitInterval = waitInterval;
  }
  
  /**
  * Sets whether to wait without timing out
  *
  * @param flag  true for indefinite wait
  *
  */
  public void setWaitIndefinite( boolean flag )
  {
    m_waitInterval = ( flag ) ? Roma.GOWAIT_INDEFINITE : 0;
  }
  
  /**
  * Specifies whether or not a get failure is to result in a 
  * message send to the Exception Message Queue. 
  * <p>
  * If an API profile is to be used ( selected by invoking the
  * useApiProfile() method and verifiable by querying the
  * isApiProfileUsed() method ) then this setting will be overridden
  * either by the profile or, if not found, the default for this 
  * option.
  *
  * @param b  true if no message should be sent to the Exception 
  *           Message Queue upon a get operation failure.
  */
  public void setNoEms( boolean b )
  {
    if (b)
      m_opts |= Kxc.GOPT_NOEMS;
    else
      m_opts &= ~Kxc.GOPT_NOEMS;
  }
  
  /**
  * Specify if a notification message is to be sent to the original
  * sender if the get failed.
  * <p>
  * If an API profile is to be used ( selected by invoking the
  * useApiProfile() method and verifiable by querying the
  * isApiProfileUsed() method ) then this setting will be overridden
  * either by the profile or, if not found, the default for this 
  * option.
  *
  * @param b  true if no message should be sent to the sender 
  *           upon a get operation failure.
  */
  public void setNoNotification( boolean b )
  {
    if (b)
      m_opts |= Kxc.GOPT_NONOTIFY;
    else
      m_opts &= ~Kxc.GOPT_NONOTIFY;
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_sync = sync;
  }
  
  /**
  * Return sync point definitiion
  */
  RomaSyncpointDef syncpointDef( )
  {
    return m_sync;
  }
  
  // Following methods are for internal use
  
  /**
  * Called from a RomaContext instance when it prepares to do
  * one of the following actions :-
  * <ul>
  * <li> invoke an error analysis routine.
  * <li> get a message
  * <li> put a message
  * </li>
  * Receives a KxcContext object and performs a series of updates to
  * it using the current state of this instance. This is done prior
  * to the KxcContext object being used in a Roma activity
  *
  * @param ctx  instance of a Roma context abstraction that takes
  *             part in low level operations.
  */
  void prepare( KxcContext ctx )
  {
    KxcGetCtrl ctrl = ctx.getCtrl();

    // set ids
    ctrl.setMsgId    ( m_msgId );
    ctrl.setCorrelId ( m_correlId );
    
    // set wait interval
    ctrl.setWaitInterval( m_waitInterval );
    int opts = m_opts;

    // ors WAIT flag if interval is specified
    if ( m_waitInterval != 0 )
      opts |= Kxc.GOPT_WAIT;
    
    // sets options
    ctrl.setOpts            ( opts );
    ctrl.setApiProfileName  ( m_apiProfileName );
    ctrl.setErrorProfileName( m_errorProfileName );
    ctrl.setSubjectStyle    ( m_subjectStyle );
    ctrl.setThemeHandle     ( m_theme != null? m_theme.handle():
    
    Kxc.NULL_HANDLE );
    ctx.putCtrl().setILSDataHandle( m_outgoingILSDataHandle );
    
    // set security buffer if one is defined
    byte[] secBuf;
    if ( m_sec == null )
      secBuf = null;
    else 
    {	
      try {
        secBuf = m_sec.getBuffer();
      }
      catch (RomaError rx) {
        secBuf = null;
      }
    }
    ctx.putCtrl().setSecurity( secBuf );
  }
  
  /**
  * Process message get operation results.
  *
  * @param ctx KxcContext
  */
  void processGet( KxcContext ctx )
  {
    KxcPutCtrl ctrl = ctx.putCtrl();
    m_incomingILSDataHandle =  ctrl.ilsDataHandle();
    m_status = ctrl.putgetStatus();
  }
  
  /**
  * Return status of get operation
  *
  */
  int status()
  {
    return m_status;
  }
}
