//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaClient.java
//
// Defines      : class RomaClient
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;
import java.util.*;

/**
* This class represents the application's client-side use of Roma.
* <p>
* A single application is allowed to instantiate multiple objects
* of the RomaClient class.
* <p>
* A RomaClient can issue requests to a Business Service and can get
* responses.
* <p>
* A client can also commit and rollback transactions.
* <p>
* It is intended that the RomaClient class be extended by
* application programmers to differentiate between different kinds
* of responses. To do this the application programmer must provide
* a new implementation of the createResponse() method that
* determines what kind of RomaClientResponse has been received
* by looking at the incoming message attributes such as the
* message format or message identifier. The application programmer
* may then create an appropriate type of RomaClientResponse 
* that has application-specific instructions for the unmarshalling
* of the message data.
*
* @see RomaClientResponse
*/
public class RomaClient extends RomaContext
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2003";
  
  private KxcClient m_kxcCli;
  
  /**
  * Constructs a new RomaClient object but does not attach
  * to the Roma network.
  * <p>
  * Call method connect() to attach to the Roma network.
  */
  public RomaClient()
  {
  }
  
  /**
  * Constructs a named Client and attaches to the Roma network
  * using default application options and with no security 
  * definition.
  *
  * @param name  name of the new Client.
  * @exception  RomaError is thrown on operation failure.
  */
  public RomaClient( String name )
    throws RomaError
  {
    connect( name, Roma.APP_NONE, null, null, null, null );
  }
  
  /**
  * Constructs a named Client, with the specified application
  * options and attaches to the Roma network.
  *
  * @param name    name of the new Client.
  * @param opts    application options
  * 
  * @exception     RomaError is thrown on operation failure.
  *
  */
  public RomaClient( String          name,
                     int             opts )
    throws RomaError
  {
    connect( name, opts, null, null, null, null );
  }
  
  /**
  * Constructs a named Client, with the specified application
  * options and security definition, and attaches to the Roma
  * network.
  *
  * @param name    name of the new Client.
  * @param opts    application options
  * @param sec     security options
  * 
  * @exception     RomaError is thrown on operation failure.
  *
  */
  public RomaClient( String          name,
                     int             opts,
                     RomaSecurityDef sec )
    throws RomaError
  {
    connect( name, opts, sec, null, null, null );
  }
  
  /**
  * Constructs a named Client, with the specified application
  * options security definition, and model queue name and then
  * attaches to the Roma network.
  *
  * @param name name of the new Client.
  * @param opts application options
  * @param sec security options
  * @param modelQName model queue name
  * 
  * @exception RomaError is thrown on operation failure.
  *
  */
  public RomaClient( String          name,
                     int             opts,
                     RomaSecurityDef sec,
                     String          modelQName )
    throws RomaError
  {
    connect( name, opts, sec, modelQName, null, null );
  }

  /**
  * Constructs a named Client, with the specified application
  * options, security definition, model queue name and data to
  * hand off to the transport data structure ILS, and attaches 
  * to the Roma network.
  *
  * @param name name of the new Client.
  * @param opts application options
  * @param sec security options
  * @param modelQName model queue name
  * @param ilsData data to hand off to the ILS
  * 
  * @exception RomaError is thrown on operation failure.
  *
  */
  public RomaClient( String          name,
                     int             opts,
                     RomaSecurityDef sec,
                     String          modelQName,
                     RomaILSData     ilsData )
    throws RomaError
  {
    connect( name, opts, sec, modelQName, ilsData, null );
  }

  /**
  * Constructs a named Client, with the specified application
  * options, security definition, model queue name, data to
  * hand off to the transport data structure ILS and LDAP authentication
  * information, and attaches to the Roma network.
  *
  * @param name name of the new Client.
  * @param opts application options
  * @param sec security options
  * @param modelQName model queue name
  * @param ilsData data to hand off to the ILS
  * @param dirAccess LDAP authentication information
  * 
  * @exception RomaError is thrown on operation failure.
  *
  */
  public RomaClient( String          name,
                     int             opts,
                     RomaSecurityDef sec,
                     String          modelQName,
                     RomaILSData     ilsData,
                     RomaDirectoryAccess dirAccess )
    throws RomaError
  {
    connect( name, opts, sec, modelQName, ilsData, dirAccess );
  }
  
  /**
  * Attaches a named Client to the Roma network. Default application
  * options, no security definition and no ILS data is used in the
  * connect operation.
  *
  * @param name    name of the Client.name
  * @exception     RomaError is thrown on operation failure
  */
  public void connect( String name )
    throws RomaError
  {
    connect( name, 
             Roma.APP_NONE, 
             null, 
             null, 
             null,
             null );
  }
  
  /**
  * Attach to the Roma network. The specified application options
  * security definition are associated with it.
  *
  * @param name    name of the Client
  * @param opts    application options
  *
  * @exception     RomaError is thrown on operation failure
  */
  public void connect( String name,
                       int    opts )
    throws RomaError
  {
    connect( name, opts, null, null, null, null );
  }
  
  /**
  * Attach to the Roma network. The specified application options
  * and security definition are associated with it.
  *
  * @param name    name of the Client
  * @param opts    application options
  * @param sec     security definition
  *
  * @exception     RomaError is thrown on operation failure
  */
  public void connect( String          name,
                       int             opts,
                       RomaSecurityDef sec )
    throws RomaError
  {
    connect( name, opts, sec, null, null, null );
  }
  
  /**
  * Attach to the Roma network. The specified application options,
  * security definition and model name name are associated with it.
  *
  * @param name    name of the Client
  * @param opts    application options
  * @param sec     security definition
  * @param modelQName model queue name
  *
  * @exception     RomaError is thrown on operation failure
  */
  public void connect( String          name,
                       int             opts,
                       RomaSecurityDef sec,
                       String          modelQName )
    throws RomaError
  {
    connect( name, opts, sec, modelQName, null, null );
  }

  /**
  * Attach to the Roma network. The specified application options,
  * security definition, model queue name and ILS data are associated
  * with it.
  *
  * @param name    name of the Client
  * @param opts    application options
  * @param sec     security definition
  * @param modelQName model queue name
  * @param ilsData data to hand off to the ILS
  *
  * @exception     RomaError is thrown on operation failure
  */
  public void connect( String          name,
                       int             opts,
                       RomaSecurityDef sec,
                       String          modelQName,
                       RomaILSData     ilsData )
    throws RomaError
  {
    connect( name, opts, sec, modelQName, ilsData, null );
  }

  /**
  * Attach to the Roma network. The specified application options,
  * security definition, model queue name, ILS data and
  * LDAP authentication information are associated with it.
  *
  * @param name name of the Client
  * @param options application options
  * @param secDef security definition
  * @param modelQName model queue name
  * @param ilsData data to hand off to the ILS
  * @param dirAccess LDAP authentication information
  *
  * @exception     RomaError is thrown on operation failure
  */
  public void connect( String              name,
                       int                 options,
                       RomaSecurityDef     secDef,
                       String              modelQName,
                       RomaILSData         ilsData,
                       RomaDirectoryAccess dirAccess )
    throws RomaError
  {
    checkDisconnected();
    KxcConnection f = nativeConn();

    KxcClient cli = f.newClient(
        name,
        options,
        secDef == null ? null : secDef.getBuffer(),
        modelQName,
        ilsData == null ? Kxc.NULL_ILS_DATA_HANDLE : ilsData.getILSDataHandle(),
        dirAccess == null ? null : dirAccess.getUserId(),
        dirAccess == null ? null : dirAccess.getPassword());

    RomaContext.kxcOper( cli.init() );
    m_kxcCli = cli;
    
    setConnected( name, options );
  }
  
  /**
  * Puts a request to a Business Service.
  * <p>
  * This method calls the ClientRequest class' marshalRequest()
  * method immediately before issuing the request. If the put
  * succeeds the RomaClientRequest instance is updated with
  * the generated theme and/or the message id and correlation id
  * if the user did not specified them on the request.
  *
  * @param req  the RomaClientRequest to be put. This provides the
  *   options and the data for the put operation.
  *
  * @exception     RomaError is thrown on operation failure
  */
  public void put( RomaClientRequest req )
    throws RomaError
  {
    checkConnected();
    
    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svcName = req.serviceName();
    
    /*
    * Supply an implementation for the PutAdapter.put() method.
    */
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the
      // put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.put service
        RomaContext.kxcOper( ((KxcClient)ctx).put( svcName ) );
      }
    };// end of anonymous inner class
    
    try {
      req.marshalRequest();
    } 
    catch ( IOException iox ) {
      RomaError err = new RomaError("");
      err.m_primInfo.setCompCode(Roma.CC_FAILED);
      err.m_primInfo.setReturnCode(Roma.RC_MARSHALLING_ERROR);
      throw err;
    }

    // invokes base put message service
    putMsg( pa, req.m_Message, req.putOptions() );
  }
  
  /**
  * Gets the response to an earlier Client using default options.
  * supplied get options. 
  * <p>
  * If the wait interval has expired a 
  * null RomaClientResponse is returned.
  *
  * @exception  RomaError is thrown on operation failure
  *
  */
  public RomaClientResponse get( )
    throws RomaError
  {
    RomaGetOptions opts = new RomaGetOptions();
    return get(opts);
  }
  
  /**
  * Gets the response to an earlier Client request using 
  * supplied options. 
  * If the get options have not been used to
  * select a specific response the first available response is
  * retrieved.
  * 
  * If the wait interval has expired a 
  * null RomaClientResponse is returned.
  *
  * @param opt  chooses the response
  * @exception  RomaError is thrown on operation failure
  */
  public RomaClientResponse get( RomaGetOptions opt )
    throws RomaError
  {
    RomaInteger noLen = new RomaInteger(-1); 
    // -1 length means auto-allocate
    return get(opt, null, noLen, null, noLen);
  }
  
  /**
  * Gets the response to an earlier Client request request using 
  * supplied options.  
  * The supplied message and tag buffers will be used to receive data.
  * 
  * If the wait interval has expired a 
  * null RomaClientResponse is returned.
  *
  * @param opt  chooses the response
  * @param msgBuf  buffer to receive message
  * @param msgBufSize  size of message buffer.
  * @param tagBuf  buffer to receive tag data 
  * @param tagBufSize  size of buffer to receive tag data
  *
  * @exception  RomaError is thrown on operation failure
  */
  public RomaClientResponse get( RomaGetOptions opt,
                                 byte           msgBuf[],
                                 RomaInteger    msgBufSize, 
                                 byte           tagBuf[],
                                 RomaInteger    tagBufSize )
    throws RomaError
  {
    checkConnected();
    
    //----------------------------------------------------------------
    // Supply an implementation for the GetAdapter.get() method.
    //----------------------------------------------------------------
    GetAdapter ga = new GetAdapter()
    {
      public boolean get( KxcContext ctx )
        throws RomaError
      {
        if ( ( ctx.getCtrl().opts() &
          (Kxc.GOPT_USEMSGID|Kxc.GOPT_USECORRELID|Kxc.GOPT_USETHEME)) == 0)
        {
          // force any theme flag if BY_THEME not specified
          ctx.getCtrl().maskOpts( Kxc.GOPT_ANYTHEME,
            Kxc.GOPT_ANYTHEME );
        }
        
        RomaContext.kxcOper( ctx.get() );
        return true;
      }
    };
    
    RomaMessage msg = new RomaMessage();
    // copy the buffer sizes to protect
    RomaInteger msgLen = new RomaInteger( msgBufSize.intValue() );
    RomaInteger tagLen = new RomaInteger( tagBufSize.intValue() );
    // Use buffers supplied instead of Roma auto-allocating
    msg.setMsgBuffer( msgBuf, msgLen );
    msg.setTagBuffer( tagBuf, tagLen );

    try {
      if ( getMsg( ga, msg, opt.getBaseOptions() ))
      {
        RomaClientResponse clientResponse = new RomaClientResponse( msg );
        return createResponse( clientResponse );
      }
      return null;
    }
    catch( RomaError apix ) {
      if ( apix.primaryInfo().returnCode() == Roma.RC_BUFFER_TOO_SMALL  ||
        apix.primaryInfo().returnCode() == Roma.RC_MESSAGE_SUPPRESSED )
      {
        // update buffer sizes to the required message lengths
        msgBufSize.setValue( msgLen.intValue() );
        tagBufSize.setValue( tagLen.intValue() );
      }
      throw apix;
    }
  }
  
  /**
  * Analagous to a put, but the request is treated as a publication
  * on the specified subject.  New applications should use the
  * version of publish that takes a RomaPublication as the parameter.
  * This version of the interface may be deprecated in a later 
  * version of eBP.
  * <p>
  *
  * @param req     defined service name and the data to be published.
  * @param subject specifies the subject for this publication.
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs
  */
  public void publish( RomaClientRequest req, RomaSubject subject )
    throws RomaError
  {
    checkConnected();
    
    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svcName = req.serviceName();
    final String mysubject = subject.subject();
    final int mystyle = subject.style();
    
    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.publish service
        RomaContext.kxcOper(
          ((KxcClient)ctx).publish( svcName, mystyle, mysubject, 0)
          );
      }
    };// end of anonymous inner class
    
    try {
      req.marshalRequest();
    } 
    catch ( IOException iox ) {
      RomaError err = new RomaError("");
      err.m_primInfo.setCompCode(Roma.CC_FAILED);
      err.m_primInfo.setReturnCode(Roma.RC_MARSHALLING_ERROR);
      throw err;
    }

    // invokes base put message service
    putMsg( pa, req.m_Message, req.putOptions() );
  }
  
  /**
  * Subscribe to the specified subject. Any data in the request message
  * is ignored.  New applications should use the version of subscribe
  * that takes a RomaSubscription as the parameter.  This version of 
  * the interface may be deprecated in a later version of eBP.
  * <p>
  *
  * @param req     defines the service name.
  * @param subject the subject to be subscribed to (can include wildcards)
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs.
  */
  public void subscribe( RomaClientRequest req, 
                         RomaSubject       subject )
    throws RomaError
  {
    checkConnected();
    
    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svc = req.serviceName();
    final String mysubject = subject.subject();
    final int mystyle = subject.style();
    
    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.subscribe service
        RomaContext.kxcOper(
          ((KxcClient)ctx).subscribe(svc, mystyle, mysubject, 0)
          );
      }
    };// end of anonymous inner class
    
    try {
      req.marshalRequest();
    }
    catch ( IOException iox ) {
      RomaError err = new RomaError("");
      err.m_primInfo.setCompCode(Roma.CC_FAILED);
      err.m_primInfo.setReturnCode(Roma.RC_MARSHALLING_ERROR);
      throw err;
    }

    // invokes base put message service
    putMsg( pa, req.m_Message, req.putOptions() );
  }
  
  /**
  * Unsubscribe to the specified subject. Any data in the request message
  * is ignored.  New applications should use the version of subscribe
  * that takes a RomaSubscription as the parameter.  This version of the
  * interface may be deprecated in a later version of eBP.
  * <p>
  *
  * @param req     defines the service name.
  * @param subject the subject to be unsubscribed to (can include wildcards)
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs.
  */
  public void unsubscribe( RomaClientRequest req, 
                           RomaSubject       subject )
    throws RomaError
  {
    checkConnected();
    
    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svc = req.serviceName();
    final String mysubject = subject.subject();
    final int mystyle = subject.style();
    
    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.unsubscribe service
        RomaContext.kxcOper(
          ((KxcClient)ctx).unsubscribe(svc, mystyle, mysubject)
          );
      }
    };// end of anonymous inner class
    
    try {
      req.marshalRequest();
    } catch ( IOException iox )
    {
      RomaError err = new RomaError("Marshalling error");
      err.m_primInfo.setCompCode(Roma.CC_FAILED);
      err.m_primInfo.setReturnCode(Roma.RC_MARSHALLING_ERROR);
      throw err;
    }
    // invokes base put message service
    putMsg( pa, req.m_Message, req.putOptions() );
  }

  // Version of publish that takes a RomaPublication object
  /**
  * Analagous to a put, but the publication is "broadcast" to 
  * any application that has subscribed to receive messages on
  * the subject contained within the RomaPublication object. 
  * Publications can be retained.
  * <p>
  *
  * @param pub     defined service name and the data to be published.
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs
  */
  public void publish (RomaPublication pub)
    throws RomaError
  {
    checkConnected();

    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svcName   = pub.serviceName();
    final String mysubject = pub.m_Sub.subject();
    final int mystyle      = pub.m_Sub.style();

    int myoptions = Roma.PS_NONE;

    //----------------------------------------------------------------
    // Set the retain flag if needed.
    //----------------------------------------------------------------
    if (pub.m_Retain)
      myoptions = Roma.PUB_RETAIN;

    final int myfoptions = myoptions;

    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.publish service
        RomaContext.kxcOper(
          ((KxcClient)ctx).publish( svcName, mystyle, mysubject, myfoptions)
                           );
      }
    };// end of anonymous inner class
    try {
      pub.marshalRequest();
    } catch ( IOException iox )
    {
      RomaError err = new RomaError("");
      err.m_primInfo.setCompCode(Roma.CC_FAILED);
      err.m_primInfo.setReturnCode(Roma.RC_MARSHALLING_ERROR);
      throw err;
    }
    // invokes base put message service
    putMsg( pa, pub.m_CR.m_Message, pub.m_CR.putOptions() );
  }

  // Version of subscribe that takes a RomaSubscription object
  /**
  * Subscribe to receive publications sent with the subject
  * defined within the RomaSubscription object.  The subscription
  * object can also define that client should only get new 
  * publication messages, and/or only get publications when the
  * client requests them.
  * <p>
  *
  * @param subscr  defines the service name and subject.
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs.
  */
  public void subscribe (RomaSubscription subscr)
    throws RomaError
  {
    checkConnected();

    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svc       = subscr.serviceName();
    final String mysubject = subscr.m_Sub.subject();
    final int mystyle      = subscr.m_Sub.style();

    int myoptions = Roma.PS_NONE;

    //----------------------------------------------------------------
    // Set the new publications only and on request flags if needed.
    //----------------------------------------------------------------
    if (subscr.m_NewPubs)
      myoptions += Roma.SUB_NEWPUBS;
    if (subscr.m_onRequest)
      myoptions += Roma.SUB_ONREQ;

    final int myfoptions = myoptions;

    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.subscribe service
        RomaContext.kxcOper(
          ((KxcClient)ctx).subscribe(svc, mystyle, mysubject, myfoptions)
                           );
      }
    };// end of anonymous inner class
    
    // invokes base put message service
    putMsg( pa, subscr.m_CR.m_Message, subscr.m_CR.putOptions() );
  }

  // Version of unsubscribe that takes a RomaSubscription object
  /**
  * Unsubscribe to receive publications sent with the subject
  * defined within the RomaSubscription object.  
  * <p>
  *
  * @param subscr  defines the service name and subject.
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs.
  */
  public void unsubscribe (RomaSubscription subscr)
    throws RomaError
  {
    checkConnected();

    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svc       = subscr.serviceName();
    final String mysubject = subscr.m_Sub.subject();
    final int mystyle      = subscr.m_Sub.style();

    final int myfoptions = Roma.PS_NONE;

    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.subscribe service
        RomaContext.kxcOper(
          ((KxcClient)ctx).unsubscribe(svc, mystyle, mysubject)
                           );
      }
    };// end of anonymous inner class

    // invokes base put message service
    putMsg( pa, subscr.m_CR.m_Message, subscr.m_CR.putOptions() );
  }

  /**
  * Request any new publications since the last request update 
  * request.
  * <p>
  *
  * @param subscr  defines the service name and subject
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs.
  */
  public void requestUpdates (RomaSubscription subscr)
    throws RomaError
  {
    checkConnected();

    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svc       = subscr.m_CR.serviceName();
    final String mysubject = subscr.m_Sub.subject();
    final int mystyle      = subscr.m_Sub.style();

    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.reqUpd service
        RomaContext.kxcOper(
          ((KxcClient)ctx).reqUpd(svc, mystyle, mysubject)
                           );
      }
    };// end of anonymous inner class

    // invokes base put message service
    putMsg(pa,
           subscr.m_CR.m_Message,
           subscr.m_CR.putOptions());
  }

  /**
  * Deletes a retained publication.
  * <p>
  *
  * @param pub     defines the service name and subject 
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs.
  */
  public void deletePublication (RomaPublication pub)
    throws RomaError
  {
    checkConnected();

    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svc       = pub.m_CR.serviceName();
    final String mysubject = pub.m_Sub.subject();
    final int mystyle      = pub.m_Sub.style();

    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.delPub service
        RomaContext.kxcOper(
          ((KxcClient)ctx).delPub(svc, mystyle, mysubject)
                           );
      }
    };// end of anonymous inner class
    try {
      pub.m_CR.marshalRequest();
    }
    catch ( IOException iox ) {
      RomaError err = new RomaError("Marshalling error");
      err.m_primInfo.setCompCode(Roma.CC_FAILED);
      err.m_primInfo.setReturnCode(Roma.RC_MARSHALLING_ERROR);
      throw err;
    }

    // Take a copy of the message as we have to set the message
    // buffer to null.
    RomaMessage t_Msg        = new RomaMessage(pub.m_CR.m_Message);
    t_Msg.setMsgBuffer (null, 0);


    // invokes base put message service
    putMsg(pa, 
           t_Msg, 
           pub.m_CR.putOptions());
  }

  /**
  * Unsubscribes for all subjects.
  * <p>
  *
  * @param subscr  defines the service name - the subject is ignored
  * @exception     RomaError is thrown if a Roma operation fails or
  *                a marshalling error occurs.
  */
  public void unsubscribeAll (RomaSubscription subscr)
    throws RomaError
  {
    checkConnected();

    // initializes final local variables
    // used by the PutAdapter anonymous class
    final String svc = subscr.serviceName();

    //----------------------------------------------------------------
    // Supply an implementation for the PutAdapter.put() method.
    //----------------------------------------------------------------
    PutAdapter pa = new PutAdapter()
    {
      // provides Roma client dependent part of the put service
      public void put( KxcContext ctx )
        throws RomaError
      {
        // next, invoke KxcClient.unSub service
        RomaContext.kxcOper(
          ((KxcClient)ctx).unsubAll(svc)
                           );
      }
    };// end of anonymous inner class

    // invokes base put message service
    putMsg (pa, 
            subscr.m_CR.m_Message, 
            subscr.m_CR.putOptions());
  }

  // End of PUBLISH - SUBSCRIBE 

  /**
  * Gets any data an inline service has made available to the 
  * application on initialization.  Currently, only transport 
  * data structure ILSs are able to provide information at this 
  * point.
  *
  * @exception  RomaError is thrown on operation failure
  */
  public RomaILSData getILSData()
    throws RomaError
  {
   if (m_kxcCli.ILSDataHandle() == Kxc.NULL_ILS_DATA_HANDLE)
      return null;
    else
      return new RomaILSData(m_kxcCli.ILSDataHandle());
  }

  /**
   *  Returns information on the specified theme.
   *
   *  @param theme Source theme
   *  @exception RomaError is thrown if the operation fails
   */
  public RomaThemeInfo queryTheme(RomaTheme theme)
  throws RomaError
  {
     checkConnected();
     return jniQueryTheme(kxcCtx().getContextHandle(), theme.handle(), true);
  }

  /**
   *  Creates a new theme from the source theme and the specified theme
   *  information.
   *
   *  @param theme Source theme
   *  @param info Theme information
   *  @param options Options indicating what parts of the theme information to
   *         use for the theme build. It is THEMEINFO_ALL or a combination of
   * <br><ul>
   * <li><code>Roma.THEMEINFO_THEMEID</code> - use theme ID</li>
   * <li><code>Roma.THEMEINFO_RETRYCOUNT</code> - use retry count</li>
   * <li><code>Roma.THEMEINFO_WORKFLOW</code> - use workflow information</li>
   * <li><code>Roma.THEMEINFO_ETEINFO</code> - use ETE information</li>
   * <li><code>Roma.THEMEINFO_TRAILERS</code> - use persistent trailers</li>
   * <li><code>Roma.THEMEINFO_CONFIRMFLAGS</code> - use confirmation flags</li>
   * </ul>
   *  @param close If true, closes the source theme
   *  @exception RomaError is thrown if the operation fails
   */
  public RomaTheme buildTheme(RomaTheme theme, 
                              RomaThemeInfo info,
                              int options,
                              boolean close)
  throws RomaError
  {
    return buildTheme(theme, info, options, close, true);
  }

  /**
   *  Returns detailed information on the specified message.
   *
   *  @param msg Source message
   *  @param options Feature options, should be zero or one of
   * <br><ul>
   * <li><code>Roma.INTMSG_THEME</code> - extract the message theme. The application is responsible for the theme closing.</li>
   * </ul>
   *  @exception RomaError is thrown if the operation fails
   */
  public RomaMessageInfo interpretMessage(RomaClientResponse msg, int options)
  throws RomaError
  {
     return interpretMessage(msg.m_Message, options);
  }
  
  /**
  * Invoked in get() methods once the response has been retrieved
  * from the Roma network.
  * <p>
  * It is intended that this method is overridden in subclasses to
  * differentiate between different types of response.
  * <p>
  * Users can construct subclasses of RomaClientResponse classes
  * that deal with the application-specific response data as
  * appropriate.
  * <p>
  * The overriddden createResponse() method can construct
  * the appropriate type of RomaClientResponse using the
  * RomaClientResponse instance data.
  *
  * @param resp RomaClientResponse containing raw response data.
  *
  * @return a RomaClientResponse or null if failure
  * has occurred when constructing the RomaClientResponse subclass.
  */
  protected  RomaClientResponse createResponse( RomaClientResponse resp )
  {
    return new RomaClientResponse( resp );
  }
  
  /**
  * Delegates underlying KxcClient object reference to
  * the internal package implementation.
  */
  KxcContext kxcCtx()
  {
    return m_kxcCli;
  }
}

//--------------------------------------------------------------------
// End of file. 
//--------------------------------------------------------------------
