//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaClientRequest.java
//
// Defines      : class RomaClientRequest
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
* This class encapsulates requests made by a Roma client to a Roma
* business service. It allows the application to set the options and
* the service required when making a client request.
* This class also provides a framework for associating application
* specific data with a client request.
* <p>The ClientRequest class may be sub-classed to provide for
* application specific data. Any subclasses must override
* the marshalRequest() method to stream application data into the
* message data buffer.
*
* @see #marshalRequest()
*/
public class RomaClientRequest
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2003";

  // RomaClientRequest has a RomaMessage
  // Package visibility so other classes can access
  RomaMessage m_Message = new RomaMessage();

  RomaPutOptions putOptions()
  { 
    return m_po; 
  }
  
  private String m_svcName;
  private RomaPutOptions m_po = new RomaPutOptions(true);
  
  /**
  * Constructs request with default options and with the name of a
  * Roma businesss service specified.
  * 
  * @param svcName  name of a Roma Business Service.
  */
  public RomaClientRequest( String svcName )
  {
    m_svcName = svcName;
  }
  
  /**
  * Constructs request with default options, with the name of a
  * Roma businesss service specified and user message buffer.
  * 
  * @param svcName name of a Roma Business Service.
  * @param msgBuf byte array containing the message.
  * @param msgLen length of the message. 
  */
  public RomaClientRequest( String svcName,
                            byte   msgBuf[],
                            int    msgLen )
  {
    m_svcName = svcName;
    m_Message.setMsgBuffer( msgBuf, msgLen );  
  }
  
  /**
  * Constructs request with default options, with the name of a
  * Roma businesss service specified and user message buffer.
  * 
  * @param svcName name of a Roma Business Service.
  * @param msgBuf byte array containing the message.
  * @param msgLen length of the message. 
  * @param tagBuf byte array containing the tag data.
  * @param tagLen length of the tag data. 
  */
  public RomaClientRequest( String svcName,
                            byte   msgBuf[],
                            int    msgLen,
                            byte   tagBuf[],
                            int    tagLen )
  {
    m_svcName = svcName;
    m_Message.setMsgBuffer( msgBuf, msgLen );  
    m_Message.setTagBuffer( tagBuf, tagLen );  
  }
  
  /**
  * Returns the name of the business service this request is 
  * intended for. Uses package visibility - not public
  */
  String serviceName()
  {
    return m_svcName;
  }
  
  /**
  * Sets the name of the Roma Business Service that a put operation
  * will send this request to.
  *
  * @param svcName  Roma Business Service name.
  */
  public void setServiceName( String svcName )
  {
    m_svcName = svcName;
  }
  
  /**
  * Invoked from the RomaClient.put() method immediately prior to 
  * attempting to issue the request to the Roma network.
  * Is intended that this method be overridden in derived classes
  * to provide an implementation for marshalling application
  * specific data into the associated message buffer.
  * <p> If this method fails the put operation will not be carried 
  * out.
  *
  * @see RomaClient#put(RomaClientRequest)
  * @exception IOException is thrown on error.
  */
  public void marshalRequest() throws IOException
  {
  }

  /**
  * Sets the put selector. This specifies which identifiers
  * contained in the message should be used when the
  * message is put.
  * @param sel the new value of the put selector.
  * <p>
  * The value will be one of the following fields :-
  * <ul>
  *   <li>USE_NONE
  *   <li>USE_MSGID
  *   <li>USE_CORRELID
  *   <li>USE_MSGID_AND_CORRELID
  * </ul>
  */
  public void setSelector( int sel )
  { 
    m_po.setSelector( sel ); 
  }
  
  /**
  * Controls whether an MSMQ message can be placed outside syncpoint
  * control. If the application attempts to put a message to MSMQ inside  
  * syncpoint control but the target queue does not support transactions,
  * the call is normally rejected with error code RC_NONXACTIONAL.
  * The application can call this method with true flag to force an MSMQ
  * message to be placed outside syncpoint control.
  * @param flag  one of :-
  * <ul>
  *  <li>true - allow MSMQ messages to be placed outside syncpoint control
  *  <li>false - don't allow MSMQ messages to be placed outside syncpoint
  *              control
  * </ul>
  */
  public void setNoSyncAllowed( boolean flag )
  { 
    m_po.setNoSyncAllowed( flag ); 
  }
  
  /**
  * Associates an API profile with the put operation.
  *
  * @param profile  the name of the API profile.
  */
  public void setApiProfileName( String profile )
  { 
    m_po.setApiProfileName( profile ); 
  }
  
  /**
  * Associates an error profile with the put operation.
  *
  * @param profile  the name of the error profile.
  */
  public void setErrorProfileName( String profile )
  { 
    m_po.setErrorProfileName( profile ); 
  }
  
  /**
  * Specify if error profiling is to be suppressed
  *   
  * @param b true to suppress error profiling
  */
  public void setSuppressErrorProfile( boolean b )
  {
    m_po.setSuppressErrorProfile(b);
	} 
    
  /**
   * Indicates that flags and options specified in the API profile
   * should be used for the Roma put operation.
   * <p>
   * If this option is chosen but the specified profile is not
   * available then default values are used for the put options.
  */
  public void setUseApiProfile( boolean b )
  { 
    m_po.useApiProfile( b ); 
  }
  
  /**
  * Enables or disables the option to send the message to an
  * exception message service if the put operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> perform a send to an
  *                exception message service if put fails.
  *    <li> false - allow a message to be sent to an exception
  *                 message service if the put fails.
  * </ul>
  */
  public void setNoEms( boolean select )
  { 
    m_po.setNoEms( select ); 
  }
  
  /**
  * Enables or disables the option to send a notification message
  * back to the sender if the message get operation fails.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true  - <strong>do not</strong> allow a notification
  *                 message to be sent on a get failure.
  *    <li> false - <strong>do </strong> allow a notification
  *                 message to be sent.
  * </ul>
  */
  public void setNoNotification( boolean select )
  { 
    m_po.setNoNotification( select ); 
  }
  
  /**
  * Enables or disables the option to send the message to a 
  * message transformation service if the next component in the
  * workflow cannot accept the message format.
  *
  * @param select  one of :-
  * <ul>
  *    <li> true - <strong>do not</strong> allow the message to
  *                sent to the transformation service.
  *    <li> false - allow the message to be routed via a
  *                 transformation service if necessary.
  * </ul>
  */
  public void    setNoTransformation( boolean select )
  {
    m_po.setNoTransformation( select ); 
  }
  
  /**
  * Specify if termination is supported
  *   
  * @param b true to indicate termination is supported.
  */
  public void setTermSupported( boolean b )
	{ 
    m_po.setTermSupported( b ); 
  }
		
  /**
  * Specify whether a new theme should be allocated.
  * <p>
  * @param b  if true then a new theme is allocated.
  */
  public void setNewTheme(boolean b) 
  { 
    m_po.setNewTheme(b); 
  }
  
  /**
  * Specify whether the theme should be retained for future operations.
  * <p>
  * @param b  if true then the theme is retained.
  */
  public void setRetainTheme(boolean b)
  { 
    m_po.setRetainTheme(b); 
  }

  /**
  * Specify whether the theme should be closed automatically when the
  * operation completes.
  * <p>
  * @param b  if true then the theme will be closed.
  */
  public void setCloseTheme(boolean b)
  { 
    m_po.setCloseTheme(b); 
  }
  
  /**
  * Specify whether an existing theme should be used.
  * <p>
  * @param b  if true then the existing theme is used.
  */
  public void setUseTheme(boolean b)
  { 
    m_po.setUseTheme(b); 
  }
  
  /**
  *  set or clear confirmation of delivery flag 
  */
  public void setCOD( boolean flag )
  {
    m_po.setCOD( flag );
  }
  
  /**
  *  set or clear confirmation of arrival flag 
  */
  public void setCOA( boolean flag )
  {
    m_po.setCOA( flag );
  }
  
  /**
  * Sets the datagram flag.
  * Controls type of the request sent to server.
  * <p>
  * If an API profile is to be used ( selected by invoking the
  * useApiProfile() method and verifiable by querying the
  * isApiProfileUsed() method ) then this setting will be
  * overridden either by the profile or, if not available, the
  * default for this option.
  *
  * @param b  one of :-
  * <ul>
  *   <li>true - send DATAGRAM
  *   <li>false - send REQUEST
  * </ul>
  */
  public void setDatagram(boolean b)
  { 
    m_po.setDatagram(b); 
  }
  
  /**
  * Set the message priority.
  * @param priority  this can be one of the following :-
  * <ul>
  * <li>PRIORITY_DEFAULT - use the default priority
  *     from the transport.
  * <li>PRIORITY_PROFILE - use the priority as specified in the 
  *     associated Put API profile. If no such profile or attribute,
  *     use PRIORITY_DEFAULT.
  * <li>a specific integer meaningful to the underlying transport
  *     e.g. 0..9 for MQSeries and BEAMQ,  0..7 for MSMQ 
  * </ul>
  */
  public void setPriority( int priority )
  {
    m_Message.setPriority(priority);
  }
  
  /**
  * Set a new value for the message perisitence.
  * @param  persistence  this can be one of the following :-
  *   <ul>
  *   <li>PERSISTENCE_YES
  *   <li>PERSISTENCE_NO
  *   <li>PERSISTENCE_PROFILE - use the persistence value as
  *       specified in the associated Put API profile. If no such
  *       profile or attribute, use PERSISTENCE_NO.
  *   </ul>
  */
  public void setPersistence( int persistence )
  {
    m_Message.setPersistence(persistence);
  }
  
  /**
  * Set a new message format value.
  * @param format  identifies the format of message. If the 
  *   value FORMAT_PROFILE is used then the message format value
  *   is retrieved from the associated Put API profile. If the
  *   profile is not available or else has no format attribute 
  *   value then the message format value FORMAT_BINARY is used.
  */
  public void setFormat( int format )
  {
    m_Message.setFormat(format);
  }
  
  /**
  * Sets a new message feedback code.
  * @param feedback  new message feedback value.
  */
  public void setFeedback( int feedback )
  {
    m_Message.setFeedback(feedback);
  }
  
  /**
  * Set a new message expiry time.
  * @param expiry  the new expiry time value
  *   <strong>in seconds</strong> or EXPIRY_NEVER or EXPIRY_PROFILE.
  *   If EXPIRY_PROFILE is specified then
  *   the expiry value is retrieved from the associated Put API
  *   profile. If the profile is not available or else has no expiry
  *   attribute set, the EXPIRY_NEVER constant is used.
  */
  public void setExpiry( int expiry )
  {
    m_Message.setExpiry(expiry);
  }
  
  /**
  * Set a new message codepage value. 
  * @param codepage  the new codepage value. If CODEPAGE_PROFILE is
  *   specified then the codepage will be retrieved from the
  *   associated Put API profile. If the profile is not available or
  *   else has no codepage attribute set, the default value of 
  *   CODEPAGE_USER will be used.
  */
  public void setCodepage( int codepage )
  {
    m_Message.setCodepage(codepage);
  }
  
  /**
  * Set the message encoding.
  * @param  encoding  the new encoding indicator. If this is 
  *   ENCODING_PROFILE then the value is retrieved from the 
  *   associated Put API profile. If the profile is not available
  *   or else has no encoding attribute set, the default value
  *   ENCODING_DEFAULT is used.
  */
  public void setEncoding( int encoding )
  {
    m_Message.setEncoding(encoding);
  }
  
  /**
  * Set the message identifier.
  * @param  msgId  a byte array containing the new message id value.
  * The msg id must not exceed kxc.MSG_ID_LENGTH.
  */
  public void setMsgId( byte[] msgId )
  {
    m_Message.setMsgId(msgId);
  }
  
  /**
  * Return the message identifier in a byte array.
  */
  public byte[] msgId()
  {
    return m_Message.msgId();
  }
  
  /**
  * Set the message correlation identifier.
  * @param  correlId  the new correlation id.
  */
  public void setCorrelId( byte[] correlId )
  {
    m_Message.setCorrelId(correlId);
  }
  
  /**
  * Return the message correlation identifier.
  */
  public byte[] correlId()
  {
    return m_Message.correlId();
  }
  
  /**
  * Sets the format name.
  *
  * @param formatName the new format name.
  */
  public void setFormatName( String formatName )
  {
    m_Message.setFormatName( formatName );
  }
  
  /**
  * Update the message theme.
  *
  * @param theme RomaTheme object.
  */
  public void setTheme( RomaTheme theme )
  {
    m_Message.setTheme( theme );
  }
  
  /**
  * Return the message theme. Theme can be set internally when
  * the message is being sent or received or else explicitly by
  * the user.
  */
  public RomaTheme theme()
  {
    return m_Message.theme();
  }
  
  /**
  * Set the name of this message
  * @param messageName name to be used
  */
  public void setMsgName( String messageName )
  {
    m_Message.setMessageName(messageName);
  }
  
  /**
  * Return message status
  */
  public int status( )
  {
    return m_Message.status();
  }
  
  /**
  * Set buffer containing the message data and the length of the
  * message for a PUT request. To send an empty message set length
  * to 0 and buffer to null
  *
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message.
  *
  * @return true if successfull,
  *         false if datalength < 0 or datalength = 0 and buffer != null
  */
  public boolean setMsgBuffer( byte buffer[],
                               int  dataLength )
  {
    return m_Message.setMsgBuffer( buffer, dataLength );
  }

  /**
  * Set buffer containing the message data for a PUT request.
  * To send an empty message set buffer to null.
  *
  * @param buffer     byte array containing message data.
  *
  * @return true if successfull.
  */
  public boolean setMsgBuffer(byte[] buffer)
  {
    return m_Message.setMsgBuffer(buffer, buffer.length);
  }

  /**
  * Set buffer containing the message data for a PUT request.
  *
  * @param buffer   String containing message data.
  *
  * @return true if successfull.
  */
  public boolean setMsgBuffer(String buffer)
  {
    return setMsgBuffer(buffer.getBytes());
  }

  /**
  * Set buffer containing the tag data and the length of the
  * tag for a PUT request.
  * @param buffer     byte array containing tag data.
  * @param dataLength length of the tag data.
  *
  * @return true if successfull,
  *         false if datalength < 0 or datalength = 0 and buffer != null
  */
  public boolean setTagBuffer( byte buffer[],
                               int  dataLength )
  {
    return m_Message.setTagBuffer( buffer, dataLength );
  }

  /**
  * Set buffer containing the tag data for a PUT request.
  *
  * @param buffer     byte array containing tag data.
  *
  * @return true if successfull.
  */
  public boolean setTagBuffer(byte[] buffer)
  {
    return m_Message.setTagBuffer(buffer, buffer.length);
  }

  /**
  * Set buffer containing the tag data for a PUT request.
  *
  * @param buffer  String containing tag data.
  *
  * @return true if successfull.
  */
  public boolean setTagBuffer(String buffer)
  {
    return setTagBuffer(buffer.getBytes());
  }

  /**
  * Set security definition
  *
  * @param secDef security definition to be sent with message
  *
  */
  public void setSecurityDef( RomaSecurityDef secDef )
  {
    m_Message.setSecurityDef( secDef );
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_po.setSyncpointDef( sync );
  }
  
  /**
  * Set outgoing ILS data
  *
  * @param  ilsData The ILS data to be sent with message 
  */
  public void setILSData(RomaILSData ilsData)
  {							 
    m_po.m_outgoingILSDataHandle = ilsData.getILSDataHandle( );
  }
  
  /**
  * Get incoming ILS data
  *
  */
  public RomaILSData getILSData()
  {							 
    if (m_po.m_incomingILSDataHandle == Kxc.NULL_ILS_DATA_HANDLE)
      return null;
    else
      return new RomaILSData( m_po.m_incomingILSDataHandle );
  }
}
