//--------------------------------------------------------------------
//
// Copyright !Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaContext.java
//
// Defines      : class RomaContext
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.util.*;

/**
* This is a base class that encapsulates the common options between
* a RomaClient and a RomaBusinessElement.
* 
* @see RomaClient
* @see RomaBusinessElement
*/
abstract public class RomaContext {
  
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2003";
  
  private String           m_appName;
  private int              m_appOpts;
  private boolean          m_connected;
  
  /**
  * Returns true if connection established, false otherwise.
  */
  public boolean isConnected()
  {
    return m_connected;
  }
  
  /**
  * Commits any local Roma transactions outstanding on current
  * thread. This method doesn't rely on the context having been
  * set in the syncpoint definition.
  *
  * @exception RomaError is thrown on error.
  */
  public void commitLocal()
    throws RomaError
  {
    checkConnected();
    kxcOper( kxcCtx().commit( Kxc.SYNC_LOCAL, 0 ) );
  }
  
  /**
  * Rolls back any local Roma transactions outstanding on the
  * current thread. This method doesn't rely on the context
  * having been set in the syncpoint definition, it always rollbacks
  * the Roma local transaction.
  *
  * @exception RomaError is thrown on error.
  */
  public void rollbackLocal()
    throws RomaError
  {
    checkConnected();
    kxcOper( kxcCtx().rollback( Kxc.SYNC_LOCAL, 0 ) );
  }
  
  /**
  * Deletes storage queue. (temporarily removed. RCH)
  *
  * @param queueName name of storage queue to delete.
  *
  * @exception RomaError  reports errors returned by the
  *                           Roma pipe.
  */
  public void deleteStorageQueue( String queueName )
    throws RomaError
  {
    checkConnected();
    kxcOper( kxcCtx().deleteQueue( queueName ) );
  }
  
  /**
  * Closes a Roma theme.
  *
  * @param theme  a Roma theme previously allocated for this
  *               context. If the operation is successful this will
  *               be reset to null.
  *
  * @exception RomaError is thrown on error.
  */
  public void closeTheme( RomaTheme theme )
    throws RomaError
  {
    if ( !theme.isNull() )
    {
      checkConnected();
      kxcOper( kxcCtx().closeHandle( theme.handle() ) );
    }
  }
  
  /**
  * Terminates the connection to Roma (if connected).
  *
  * @exception RomaError is thrown on error.
  */
  public void disconnect()
    throws RomaError
  {
    if ( m_connected )
    {
      try {
        kxcOper( kxcCtx().disconnect() );
      } 
      finally {
        m_connected = false;
      }
    }
  }
  
  /**
  * Writes an exception message to the exception message queue.
  *
  * @param msg  the exception message.
  * @param opt  put options associated with the operation.
  * 
  * @exception RomaError is thrown on error.
  */
  public void putException( RomaExceptionMessage msg,
                            RomaExceptionOptions opt )
    throws RomaError
  {
    checkConnected();
    final String               svcName = opt.serviceName();
    final RomaExceptionMessage msg2    = msg;
    
    /*
    Supply an overriding implementation for the PutAdapter.put()
    method.
    */
    PutAdapter pa = new PutAdapter()
    {
      public void put( KxcContext ctx )
        throws RomaError
      {
        msg2.syncPutCtrl( ctx.putCtrl() );
        kxcOper( ctx.putException( svcName ) );
      }
    }; // end of anonymous inner class

    // transfer theme from options to message
    if (opt.theme() != null)
      msg.m_Message.setTheme( opt.theme() );
    
    // call preparePut/processPut to ensure error data is handled
    msg.preparePut(kxcCtx());
    putMsg( pa, msg.m_Message, opt.putOptions() );
    msg.processPut(kxcCtx());
  }
  
  /**
  * Writes an exception message to the exception message queue with
  * default RomaExceptionOptions.
  *
  * @param msg  the exception message.
  *
  * @exception RomaError is thrown on error.
  */
  public void putException( RomaExceptionMessage msg )
    throws RomaError
  {
    putException( msg, new RomaExceptionOptions() );
  }
  
  /**
  * Used to invoke an error analysis routine directly.
  *
  * @param msg  the exception message.
  * @param opt  options associated with the invocation.
  * @param carriedOutActions  actions carried out
  * @exception RomaError is thrown on error.
  */
  public void reportError( RomaExceptionMessage  msg,
                           RomaInteger           carriedOutActions,
                           RomaReportOptions     opt )
    throws RomaError
  {
    checkConnected();
    opt.prepare( kxcCtx() );
    kxcCtx().reportCtrl().setSyncpoint( opt.syncpointDef().type(),
                                        opt.syncpointDef().context() );
    
    // transfer theme from options to message
    if (opt.theme() != null)
      msg.m_Message.setTheme( opt.theme() );
    
    msg.report( kxcCtx() );
    
    opt.process( kxcCtx() );
    
    carriedOutActions.setValue(kxcCtx().reportCtrl().actions());	
  }
  
  /**
  * Invoke an error analysis routine directly with default
  * RomaReportOptions.
  *
  * @param msg  the exception message.
  * @param carriedOutActions  actions carried out
  * @exception  RomaError is thrown on error.
  */
  public void reportError( RomaExceptionMessage  msg, 
                           RomaInteger           carriedOutActions )
    throws RomaError
  {
    reportError( msg, carriedOutActions, new RomaReportOptions( ) );
  }
  
  /**
  * Writes entry to the system log.
  *
  * @param info information for the log.
  *
  * @exception RomaError is thrown on error.
  */
  public static void writeLog( RomaLogInformation info )
    throws RomaError
  {
    KxcConnection.LogWriter logWriter = nativeConn().logWriter();
    info.setPeer( logWriter );
    kxcOper( logWriter.writeLog() );
  }

  /**
   * Writes entry to the system log.
   *
   * @param info information for the log.
   *
   * @exception RomaError is thrown on error.
   */
  public static void writeSysLog( RomaLogInformation info )
    throws RomaError
  {
    KxcConnection.LogWriter logWriter = nativeConn().logWriter();
    info.setPeer( logWriter );
    kxcOper( logWriter.writeSysLog() );
  }
  
  /**
  * Sends an alert message.
  *
  * @param info the alert.
  *
  * @exception RomaError is thrown on error.
  */
  public void sendAlert( RomaLogInformation info )
    throws RomaError
  {
    checkConnected();
    info.setPeer( kxcCtx().logData() );
    kxcOper( kxcCtx().sendAlert() );
  }
  
   /**
   * Inserts the business service into the workflow of the theme
   * immediately after the current location in the workflow.
   *
   * @param theme Source theme
   * @param bs Business Service to insert
   * @param close If true, closes the source theme
   * @return New theme with inserted workflow
   * @exception RomaError is thrown if the operation fails
   */
   public RomaTheme insertWorkflow(RomaTheme theme,
                                   RomaBusinessService bs,
                                   boolean close)
   throws RomaError
   {
     checkConnected();
     long h = jniInsertWorkflow(kxcCtx().getContextHandle(),
        theme.handle(), bs.getName(), close, false);

     if (close)
     {
        theme.setNull();
     }

     return new RomaTheme(h);
   }

   /**
   * Replaces all remaining servers in the workflow with the business
   * service, leaving the client at the end of the workflow.
   *
   * @param theme Source theme
   * @param bs Business Service replacing the workflow
   * @param close If true, closes the source theme
   * @return New theme with replaced workflow
   * @exception RomaError is thrown if the operation fails
   */
   public RomaTheme replaceWorkflow(RomaTheme theme,
                                    RomaBusinessService bs,
                                    boolean close)
   throws RomaError
   {
     checkConnected();
     long h = jniInsertWorkflow(kxcCtx().getContextHandle(),
        theme.handle(), bs.getName(), close, true);

     if (close)
     {
        theme.setNull();
     }

     return new RomaTheme(h);
   }

   /**
   * Sets the business flow information on the context. This information
   * will be included into the workflow route information on the next
   * {@link RomaContext#replaceWorkflow(RomaTheme,RomaBusinessService,boolean)}
   * or {@link RomaContext#insertWorkflow(RomaTheme,RomaBusinessService,boolean)}
   * calls.
   *
   * @param bf Business Flow information
   * @exception RomaError is thrown if the operation fails
   */
   public void setBusinessFlow(RomaBusinessFlow bf)
   throws RomaError
   {
     checkConnected();
     jniSetBusFlow(kxcCtx().getContextHandle(),
        bf.getFlowName(), bf.getOperationName());
   }

   /**
   * Creates a new theme ID with exactly the same information as
   * the existing theme.
   *
   * @param theme Source theme
   * @return Copy of the specified theme
   * @exception RomaError is thrown if the operation fails
   */
   public RomaTheme copyTheme(RomaTheme theme)
   throws RomaError
   {
     checkConnected();
     return new RomaTheme(
        jniCopyTheme(kxcCtx().getContextHandle(), theme.handle()));
   }

   /**
   * Merges the end-to-end and in-flight information with the end-to-end
   * and in-flight information that already exists for the message and
   * can be extracted with the
   * {@link RomaBusinessElement#queryTheme(RomaTheme)}
   * or {@link RomaClient#queryTheme(RomaTheme)}
   * method.
   *
   * @param theme Source theme
   * @param ete End-to-end and in-flight information to merge into
   *            the specified theme
   * @param close If true, closes the source theme
   * @return New theme with merged ETE information
   * @exception RomaError is thrown if the operation fails
   */
   public RomaTheme mergeEteInfo(RomaTheme theme,
                                 RomaEteInfo ete,
                                 boolean close)
   throws RomaError
   {
     checkConnected();
     long h = jniMergeEteInfo(kxcCtx().getContextHandle(),
        theme.handle(), ete.getData(), close);

     if (close)
     {
        theme.setNull();
     }

     return new RomaTheme(h);
   }
  
  RomaMessageInfo interpretMessage(RomaMessage msg, int options)
  throws RomaError
  {
     checkConnected();

     if (options != 0 && options != Roma.INTMSG_THEME)
     {
        throw new RomaError("Invalid options value");
     }

     long ts = msg.timeStamp().getTime();
     return jniInterpretMsg(kxcCtx().getContextHandle(), options,
         msg.msgType(), msg.priority(), msg.persistence(), msg.format(),
         msg.feedback(), msg.expiry(), msg.codepage(), msg.encoding(),
         msg.msgId(), msg.correlId(), msg.status(), msg.msgBuffer(),
         msg.tagBuffer(), msg.backoutCount(), ts / 1000, ts % 1000,
         msg.formatName(), msg.subject(), msg.messageName());
  }

  RomaTheme buildTheme(RomaTheme theme, 
                       RomaThemeInfo info,
                       int options,
                       boolean close,
                       boolean client)
  throws RomaError
  {
     checkConnected();

     if (options == 0 || (options & ~Roma.THEMEINFO_ALL) != 0)
     {
        throw new RomaError("Invalid options value");
     }

     // Make sure theme ID is valid
     if ((options & Roma.THEMEINFO_THEMEID) != 0 &&
         info.getThemeId() == null)
     {
        throw new RomaError("Theme ID is null");
     }

     if ((options & Roma.THEMEINFO_TRAILERS) != 0)
     {
        // Make sure trailers are valid
        LinkedList trailers = info.getTrailers();

        if (trailers == null || trailers.isEmpty())
        {
            throw new RomaError("No trailers in the theme information");
        }

        ListIterator iter = trailers.listIterator(0);

        while (iter.hasNext())
        {
            Object o = iter.next();

            if (o == null)
            {
                throw new RomaError(
                    "The trailers list contains a null object");
            }

            if ((o instanceof RomaTrailer) == false)
            {
                throw new RomaError("The trailers list contains " +
                    o.getClass().getName() + 
                    " object that cannot be casted to RomaTrailer");
            }
        }
    }

    if ((options & Roma.THEMEINFO_WORKFLOW) != 0)
    {
        // Make sure workflow is valid
        RomaWorkflow wf = info.getWorkflow();

        if (wf == null || wf.getWorkflowService() == null)
        {
            throw new RomaError("No workflow in the theme information");
        }

        LinkedList routes = wf.getRoutes();

        if (routes == null || routes.isEmpty())
        {
            throw new RomaError(
                "No workflow routes in the theme information");
        }

        ListIterator iter = routes.listIterator(0);

        while (iter.hasNext())
        {
            Object o = iter.next();

            if (o == null)
            {
                throw new RomaError(
                    "The routes list contains a null object");
            }

            if ((o instanceof RomaRoute) == false)
            {
                throw new RomaError("Routes list contains " +
                    o.getClass().getName() + 
                    " object that cannot be casted to RomaRoute");
            }
        }
    }

    long handle = jniBuildTheme(kxcCtx().getContextHandle(),
        theme.handle(), info, options, close, client);

    if (close)
    {
        theme.setNull();
    }

    return new RomaTheme(handle);
  }

  /**
  * Sets connection params.
  */
  void setConnected( String appName,
                     int    appOpts )
    throws RomaError
  {
    m_appName   = appName;
    m_appOpts   = appOpts;
    m_connected = true;
  }
  
  /**
  * Checks whether connection is valid.
  * Throws RomaError exception if invalid.
  *
  * @see #checkDisconnected()
  */
  void checkConnected()
    throws RomaError
  {
    if ( !m_connected )
      throw new RomaError( "not connected" );
  }
  
  /**
  * Checks for no connection.
  * Throws RomaError exception if a connection exists.
  *
  * @see #checkConnected()
  *
  */
  void checkDisconnected()
    throws RomaError
  {
    if ( m_connected )
      throw new RomaError( "Already connected" );
  }
  
  /**
  * Generic get message service implementation.
  *
  * @param ga         GetAdapter object implementing
  *                   specific details.
  * @param msg        message to get from queue
  * @param opts       get options
  * @param sync       syncpoint definition
  * @return           true if message retrieved successfully,
  *                   false if RC_NO_MSG_AVAILABLE flag
  *                   received
  *
  * @exception RomaError if a Roma API call fails
  */
  boolean getMsg( GetAdapter         ga,
                  RomaMessage        msg,
                  RomaBaseGetOptions opts )
    throws RomaError
  {
    // prepare message
    try {
      msg.prepareGet( kxcCtx() );
      opts.prepare( kxcCtx() );
      
      // prepare rest of the control
      kxcCtx().getCtrl().setSyncpoint( opts.syncpointDef().type(), 
                                       opts.syncpointDef().context() );
      // call adapter
      if ( ga.get( kxcCtx() ) )
      {
        // process results
        msg.processGet( kxcCtx() );
        opts.processGet( kxcCtx() );
      }
      return true;
    }
    catch( RomaError apix ) {
      if ( apix.primaryInfo().returnCode() == Roma.RC_NO_MSG_AVAILABLE )
        return false;
      
      if ( apix.primaryInfo().returnCode() == Roma.RC_BUFFER_TOO_SMALL  ||
        apix.primaryInfo().returnCode() == Roma.RC_MESSAGE_SUPPRESSED )
      {
        // In this case must set the actual user and tag buffer lengths
        // required that will have been returned from pipe call via JNI.
        msg.setReturnedBufferLengths( kxcCtx().msg().msgLength(),
                                      kxcCtx().msg().tagLength() );
      }
      throw apix;
    }
  }
  
  /**
  * Generic put message service.
  * Assumes that message is already marshalled.
  *
  * @param pa    PutAdapter object implementing context
  *              specific put details
  * @param msg   message to be put on queue
  * @param opts  put options
  * @param sync  syncpoint definition
  *
  * @exception RomaError    if Roma API call fails
  */
  void putMsg( PutAdapter         pa,
               RomaMessage        msg,
               RomaBasePutOptions opts )
    throws RomaError
  {
    // prepare message
    msg.preparePut( kxcCtx() );
    
    // prepare options
    opts.prepare( kxcCtx() );
    kxcCtx().putCtrl().setSecurity( msg.getSecurityBuffer() );
    kxcCtx().putCtrl().setSyncpoint( opts.syncpointDef().type(),
                                     opts.syncpointDef().context() );
    // call adapter
    pa.put( kxcCtx() );
    
    // process results
    msg.processPut( kxcCtx() );
    opts.processPut( kxcCtx() );
  }
  
  /**
  * Abstract method returning context implementation object
  * (of either KxcClient or KxcServer class type).
  */
  abstract KxcContext kxcCtx();
  
  /**
  * Returns native connection object.
  */
  static KxcConnection nativeConn()
  {
    return KxcConnection.nativeConnection();
  }
  
  /**
  * Carry out some low level Roma activity. Examples of such
  * activity include performing a transaction commit or rollback ;
  * doing a network connection ; doing a message get or put.
  * <p>
  * This is a very key method in the bindings as all Roma 
  * work is accomplished by calling this method with an argument
  * that describes the action to be performed.
  * 
  * @param oper  encapsulates the action and arguments that
  *              uniquely describe the operation to be performed.
  * @exception RomaError  if Roma operation fails
  *
  * @see RomaBusinessElement
  * @see RomaClient
  * @see RomaExceptionMessage
  * @see RomaStorageQueue
  * @see com.candle.roma.kxc.raw.KxcOper
  */
  static void kxcOper( KxcOper oper )
    throws RomaError
  {
    try {
      oper.invoke();
    } 
    catch ( KxcOper.ApiException apix ) {
      throw new RomaError( apix.error() );
    } 
    catch ( Exception x ) {
      throw new RomaError( x.getMessage() );
    }
  }


  native RomaMessageInfo jniInterpretMsg(
    long ctx, int opts, int type, int prio, int pers, int format,
    int fback, int expiry, int cp, int enc, byte[] mid, byte[] cid,
    int status, byte[] mbuf, byte[] tbuf, int backout, long sec, long msec,
    String fname, String sname, String mname);

  native RomaThemeInfo jniQueryTheme(
    long ctx, long th, boolean client);

  native long jniBuildTheme(
    long ctx, long th, RomaThemeInfo info, int options,
    boolean close, boolean client);

  native long jniInsertWorkflow(
    long ctx, long th, String sn, boolean close, boolean replace);

  native long jniCopyTheme(
    long ctx, long th);

  native long jniMergeEteInfo(
    long ctx, long th, byte[] data, boolean close);

  native void jniSetBusFlow(
    long ctx, String bf, String op);
}

//--------------------------------------------------------------------
// End of file. RomaContext.java.
//--------------------------------------------------------------------
