//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaErrorInformation.java
//
// Defines      : class RomaErrorInformation
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.KxcSubError;
import java.io.*;

/**
* This class introduces Roma error information.
*/
class RomaErrorInformation implements Roma
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2003";
  
  int    m_compCode   = 0;
  int    m_returnCode = 0;
  int    m_opType     = 0;
  int    m_reason     = 0;
  int    m_position   = 0;
  String m_location   = "";
  String m_shortText  = "";
  String m_fullText   = "";
  int    m_type       = 0;
  
  /**
  * Default constructor.
  * Note : not public and so instances of this class cannot be
  *        created by users.
  */
  RomaErrorInformation()
  {
  }
  
  /**
  * Initializes RomaErrorInformation object using KxcSubError
  * object type.
  * Note : not public and so instances of this class cannot be
  *        created by users.
  */
  RomaErrorInformation( KxcSubError err )
  {
    m_compCode    = err.compCode();
    m_returnCode  = err.returnCode();
    m_opType      = err.opType();
    m_reason      = err.reason();
    m_position    = err.position();
    m_location    = err.location();
    m_type        = err.type();
    m_fullText    = err.fullText();
    m_shortText   = err.shortText();
  }
  
  /**
  * Returns completion code.
  *
  * @see Roma
  */
  public int compCode()
  {
    return m_compCode;
  }
  
  /**
  * Sets the completion code value.
  *
  * @param compCode the new completion code value.
  * @see Roma
  */
  public void setCompCode( int compCode )
  {
    m_compCode = compCode;
  }
  
  /**
  * Returns the return code.
  *
  * @see Roma
  */
  public int returnCode()
  {
    return m_returnCode;
  }
  
  /**
  * Sets the return code value.
  *
  * @param returnCode the new return code value.
  * @see Roma
  */
  public void setReturnCode( int returnCode )
  {
    m_returnCode = returnCode;
  }
  
  /**
  * Returns the type of the failed operation.
  *
  * @see Roma
  */
  public int operationType()
  {
    return m_opType;
  }
  
  /**
  * Sets the operation type value.
  *
  * @param opType  the new operation type identifier.
  * @see Roma
  */
  public void setOperationType( int opType )
  {
    m_opType = opType;
  }
  
  /**
  * Returns the reason code from the transport layer
  * where the failure occurred. For example, if the MQSeries
  * transport was being used, this value would be the
  * reason code from MQSeries.
  */
  public int reason()
  {
    return m_reason;
  }
  
  /**
  * Sets the reason code.
  *
  * @param reason  the new reason code value.
  */
  public void setReason( int reason )
  {
    m_reason = reason;
  }
  
  /**
  * Returns error position to help identify the failing code.
  */
  public int position()
  {
    return m_position;
  }
  
  /**
  * Sets error position.
  *
  * @param position the error position
  */
  public void setPosition( int position )
  {
    m_position = position;
  }
  
  /**
  * Sets error type.
  *
  * @param position the error position
  */
  public void setErrorType( int type )
  {
    m_type = type;
  }
  
  /**
  * Returns additional error location information.
  */
  public String additionalInfo()
  {
    return m_location;
  }
  
  /**
  * Sets additional information.
  *
  * @param location additional error location information.
  */
  public void setAdditionalInfo( String location )
  {
    m_location = location;
  }
  
  /**
  * Returns String object with full error description.
  */
  public String fullText()
  {
    return m_fullText;
  }
  
  /**
  * Returns String object with abbreviated error description.
  */
  public String shortText()
  {
    return m_shortText;
  }
  
  /**
  * Returns error type code.
  *
  * @see Roma
  */
  public int type()
  {
    return m_type;
  }
  
  /**
  * Returns representation of this object as a String. Necessary
  * if an object of this class is written to an output stream or
  * if used with the string concatenation operator.
  * <p>
  */
  public String toString()
  {
    return m_fullText != null? m_fullText: "";
  }
  
  /**
  * Receives a KxcSubError object and sets it up using the state
  * of this object.
  *
  * @param err  KxcSubError instance - ie, an encapsulation of a
  *             KXCSUBERROR structure in the C level layer.
  * 
  * @see #RomaErrorInformation( KxcSubError )
  * 
  */
  void setPeer( KxcSubError err )
  {
    err.setCompCode   ( m_compCode );
    err.setReturnCode ( m_returnCode );
    err.setOpType     ( m_opType );
    err.setReason     ( m_reason );
    err.setPosition   ( m_position );
    err.setLocation   ( m_location );
    err.setType       ( m_type );
  }

    RomaErrorInformation(int comp, int ret, int oper, int reason,
            int pos, int type, String loc, String sh, String full)
    {
        m_compCode = comp;
        m_returnCode = ret;
        m_opType = oper;
        m_reason = reason;
        m_position = pos;
        if (loc != null) m_location  = loc;
        if (sh != null) m_shortText = sh;
        if (full != null) m_fullText = full;
        m_type = type;
    }
}

