//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaRetrieveOptions.java
//
// Defines      : class RomaRetrieveOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;
import com.candle.roma.kxc.raw.*;

/**
* The RomaRetrieveOptions encapsulates the options available
* when retrieving a message from a storage queue.
*
*/
public class RomaRetrieveOptions 
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // get options has a BaseGetOptions, with package visibility
  RomaBaseGetOptions m_go = new RomaBaseGetOptions();
  
  /**
  * Construct a new RomaRetrieveOptions with default options.
  */
  public RomaRetrieveOptions()
  {
  }
  
  /**
  * Specify whether the theme should be closed automatically when the
  * operation completes.
  * <p>
  * @param flag  if true then the theme will be closed.
  */
  public void setCloseTheme(boolean flag)
  {
    if (flag) {
      m_go.m_opts |=  Kxc.GOPT_CLOSETHEME;
      m_go.m_opts &= ~Kxc.GOPT_RETAINTHEME;
    }
    else {
      m_go.m_opts &= ~Kxc.GOPT_CLOSETHEME;
      m_go.m_opts |=  Kxc.GOPT_RETAINTHEME;
    }
  }
  
  /**
  * Specify whether the theme should be retained for future operations.
  * <p>
  * @param flag  if true then the theme is retained.
  */
  public void setRetainTheme(boolean flag)
  {
    setCloseTheme(!flag);
  }
  
  /**
  * Sets an indefinite wait interval.
  */
  public void setWaitIndefinite()
  {
    m_go.m_waitInterval = Kxc.GOWAIT_INDEFINITE;
  }
  
  // Methods in base get options
  
  /**
  * Selects whether or not a new theme will be allocated
  * when applying these options
  * 
  * @param flag  if true theme already associated with the 
  * RomaMessage will be used. If false the new theme 
  * allocation will be done.
  */
  public void setUseTheme(boolean flag)
  {
    m_go.setUseTheme( flag );
  }
  
  /**
  * Selects whether not to get message by matching theme
  * when applying these options
  * 
  * @param flag  if true the theme will be ignored  
  *							If false the theme will be matched 
  *  
  */
  public void setAnyTheme(boolean flag)
  {
    m_go.setUseTheme(!flag);	
  }
  
  /**
  * Sets the API profile name.
  * 
  * @param name  the name of the API profile.
  */
  public void setApiProfileName( String name )
  {
    m_go.setApiProfileName( name );
  }
  
  /**
  * Indicates that flags and options specified in the API profile
  * should be used for the Roma put operation.
  * <p>
  * If this option is chosen but the specified profile is not
  * available then default values are used for the put options.
  * @param flag :-
  * <ul>
  *  <li>true - use API profile
  *  <li>false - do not use API profile
  * </ul>
  */
  public void useApiProfile( boolean flag )
  {
    m_go.useApiProfile( flag );
  }
  
  /**
  * Sets the error profile name.
  * 
  * @param name  the name of the error profile.
  */
  public void setErrorProfileName( String name )
  {
    m_go.setErrorProfileName( name );
  }
  
  /**
  * Set only the receive options of the object.
  *
  * @param op  the new message receive options. It is expected that
  *            this will be one of the following values :-
  *            <ul>
  *            <li> GET_FIRST
  *            <li> PEEK_FIRST
  *            <li> PEEK_NEXT
  *            <li> PEEK_SAME
  *            <li> GET_PEEKED
  *            </ul>
  */
  public void setOperation( int op )
  {
    m_go.setOperation( op );
  }
  
  /**
  * Sets the message selection criteria.
  *
  * @param sel  the new message selection options.
  *             It is expected that
  *             this will be one of the following values :-
  *             <ul>
  *             <li> BY_NONE
  *             <li> BY_THEME
  *             <li> BY_MSGID
  *             <li> BY_CORRELID
  *             <li> BY_MSGID_AND_CORRELID
  *             <li> BY_MSGID_AND_THEME
  *             </ul>
  * 
  */
  public void setSelector( int sel )
  {
    m_go.setSelector( sel );
  }
  
  /**
  * Sets the theme.
  *
  * @param theme  the new theme.
  * 
  */
  public void setTheme( RomaTheme theme )
  {
    m_go.setTheme( theme );
  }
  
  /**
  * Sets the message identifier.
  *
  * @param msgId  the new message identifier value
  *
  */
  public void setMsgId(byte[] msgId)
  {
    m_go.setMsgId( msgId );
  }
  
  /**
  * Sets the correlation identifier.
  *
  * @param correlId  the new correlation identifier value.
  *
  */
  public void setCorrelId(byte[] correlId )
  {
    m_go.setCorrelId( correlId );  
  }
  
  /**
  * Sets the wait interval in milliseconds.
  *
  * @param waitInterval  the new wait interval value in milliseconds.
  *
  */
  public void setWaitInterval( int waitInterval )
  {
    m_go.setWaitInterval( waitInterval );
  }
  
  /**
  * Sets whether to wait without timing out
  *
  * @param flag  true for indefinate wait
  *
  */
  public void setWaitIndefinite( boolean flag )
  {
    m_go.setWaitIndefinite( flag );
  }
  
  /**
  * Specifies whether or not a get failure is to result in a 
  * message send to the Exception Message Queue. 
  *
  * @param flag :-
  * <ul>
  *  <li>true - no message to be sent to the Exception Queue upon get failure
  *  <li>false - message to be sent
  * </ul>
  */
  public void setNoEms( boolean b )
  {
    m_go.setNoEms( b );
  }
  
  /**
  * Specify if a notification message is to be sent to the original
  * sender if the get failed.
  *
  * @param flag :-
  * <ul>
  *  <li>true - no message to be sent to the sender upon get failure
  *  <li>false - message to be sent to the sender
  * </ul>
  */
  public void setNoNotification( boolean flag )
  {
    m_go.setNoNotification( flag );
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_go.setSyncpointDef( sync );
  }
  
  // end of base get options methoods	
  
  /**
  * get base options
  * 
  */
  RomaBaseGetOptions getBaseOptions()
  {
    return  m_go;	
  }
}
