//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2001-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcopbl.hpp,v 1.9 2003/02/21 22:48:06 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcopbl.hpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------
// The RomaPublication public interface.
//--------------------------------------------------------------------

#ifndef __KXCOPBL_HPP__
#define __KXCOPBL_HPP__

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcosyn.hpp>
#include <kxcosec.hpp>
#include <kxcomid.hpp>
#include <kxcoils.hpp>
#include <kxcosub.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplPublication;


//--------------------------------------------------------------------
// The RomaPublication class.
// 
// This class is used to publish messages. Objects of this and derived
// classes are used by the RomaClient::publish() and
// the RomaClient::deletePublication() methods.
//--------------------------------------------------------------------
class KXCODLL RomaPublication : public RomaTypeBase,
                                public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructs the publication with the default options and the
    // specified service name and subject.
    // 
    // Parameters:
    //   (in) serviceName - a name of a business service.
    //   (in) subject     - a subject for the publication.
    //----------------------------------------------------------------
    RomaPublication(const char* serviceName,
                    const RomaSubject& subject);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaPublication();

    //----------------------------------------------------------------
    // Copy and assignment.
    //
    // Note: these methods don't create an independent copy of the
    // message, but just share common implementation instance.
    //----------------------------------------------------------------
    RomaPublication(const RomaPublication&);
    RomaPublication& operator = (const RomaPublication&);

    //----------------------------------------------------------------
    // Set a business service name.
    // 
    // The service name must be defined before the object will be
    // used.
    //----------------------------------------------------------------
    void setServiceName(const char* name);

    //----------------------------------------------------------------
    // Set theme options.
    //
    // If a new theme is requested, a new theme will be created for
    // each request. Otherwise the specified theme will be used.
    //  
    // If close theme is requested, the theme is closed after the
    // request is sent. If retain theme is requested, the theme is
    // retained for future use.
    //  
    // These selections may be overridden if an API profile is
    // defined.
    //----------------------------------------------------------------
    void setNewTheme(bool f = true);
    void setUseTheme(bool f = true);
    void setRetainTheme(bool f = true);
    void setCloseTheme(bool f = true);

    //----------------------------------------------------------------
    // Controls the type of the message:
    //   true  - DATAGRAM
    //   false - REQUEST (default)
    // 
    // However, if an API profile is used (isApiProfileUsed() == true)
    // then either the default or the profile value is used.
    //----------------------------------------------------------------
    void setDatagram(bool f = true);

    //----------------------------------------------------------------
    // Set or clear the following flags:
    //    Confirmation of delivery (COD)
    //    Confirmation of arrival  (COA)
    //----------------------------------------------------------------
    void setCOD(bool f = true);
    void setCOA(bool f = true);

    //----------------------------------------------------------------
    // Set a security definition to be used with this message.
    //----------------------------------------------------------------
    void setSecurityDef(const RomaSecurityDef& securityDef);

    //-----------------------------------------------------------------
    // Get/set the Roma Theme ID.
    //-----------------------------------------------------------------
    const RomaTheme& theme() const;
    void setTheme(const RomaTheme& theme);

    //-----------------------------------------------------------------
    // Get/set the Roma Message ID.
    //-----------------------------------------------------------------
    const RomaMsgId& msgId() const;
    void setMsgId(const RomaMsgId& msgId);

    //-----------------------------------------------------------------
    // Get/set the Correlation ID.
    //-----------------------------------------------------------------
    const RomaMsgId& correlId() const;
    void setCorrelId(const RomaMsgId& correlId);

    //-----------------------------------------------------------------
    // Set the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority
    //-----------------------------------------------------------------
    void setPriority(int priority);

    //-----------------------------------------------------------------
    // Set the message persistence to one of ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //-----------------------------------------------------------------
    void setPersistence(Roma::Persistence persistence);

    //-----------------------------------------------------------------
    // Set the format number.
    //  If FORMAT_PROFILE is specified then format is retrieved from 
    //  the API profile, defaulting to FORMAT_BINARY.
    //-----------------------------------------------------------------
    void setFormat(int format);

    //-----------------------------------------------------------------
    // Set the feedback code.
    //  If FEEDBACK_PROFILE is specified then code is retrieved from
    //  the API profile, defaulting to FEEDBACK_NONE.
    //-----------------------------------------------------------------
    void setFeedback(int feedback);

    //-----------------------------------------------------------------
    // Set the expiry interval.
    //  If EXPIRY_PROFILE is specified then interval is retrieved from
    //  the API profile, defaulting to EXPIRY_NEVER.
    //-----------------------------------------------------------------
    void setExpiry(int expiry);

    //-----------------------------------------------------------------
    // Set the codepage.
    //  If CODEPAGE_PROFILE is specified then codepage number is
    //  retrieved from the API profile, defaulting to CODEPAGE_USER
    //-----------------------------------------------------------------
    void setCodepage(int codepage);

    //-----------------------------------------------------------------
    // Set the encoding.
    //  If ENCODING_PROFILE is specified then encoding is
    //  retrieved from the API profile, defaulting to ENCODING_DEFAULT
    //-----------------------------------------------------------------
    void setEncoding(int encoding);

    //-----------------------------------------------------------------
    // Set the format name to the specified string and format name to
    // FORMAT_USENAME.
    //-----------------------------------------------------------------
    void setFormatName(const char* name);

    //-----------------------------------------------------------------
    // Set the message name.
    // Defines a message name passed on to the Application Management 
    // inline service.
    //-----------------------------------------------------------------
    void setMsgName(const char* name);

    //-----------------------------------------------------------------
    // Set a message data buffer.
    //-----------------------------------------------------------------
    bool setMsgBuffer(const void* buffer, int length);

    //----------------------------------------------------------------
    // Set the put selector.
    //
    // The selector specifies which identifiers contained in the
    // RomaMessage (msg id, correl id) to use when the message
    // is sent.
    //
    // The selector is specified by one of the following constants:
    //   USE_NONE     - doesn't set any specific selector
    //   USE_MSGID    - use Message Id as the selector
    //   USE_CORRELID - use Correlation Id as the selector
    //   USE_MSGID_AND_CORRELID - use both
    //----------------------------------------------------------------
    void setSelector(Roma::PutSelector selector);

    //----------------------------------------------------------------
    // Allows/disallows putting messages under transaction to a
    // non-transactional queue.
    //----------------------------------------------------------------
    void setNoSyncAllowed(bool f = true);

    //----------------------------------------------------------------
    // Set an API profile name. Supply null to suppress use.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Set an error profile name.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // If true specified, Roma does not put the message to the
    // exception message service on an error.
    //----------------------------------------------------------------
    void setNoEms(bool f = true);

    //----------------------------------------------------------------
    // If notification is allowed, Roma can send responses straight
    // back to the client, ignoring any further business components.
    // This may be overriden by an API profile.
    // The following method allows the application to specify that
    // notification messages may NOT be sent back to the client.
    //---------------------------------------------------------------- 
    void setNoNotification(bool f = true);

    //----------------------------------------------------------------
    // If true specified, Roma does not send the message to the 
    // Transformation Business Service if the next component in the 
    // workflow cannot understand the message format being sent.  
    // If the recipient will not accept this message format, and this 
    // option is selected, Roma treats the Put operation as an error. 
    //----------------------------------------------------------------
    void setNoTransformation(bool f = true);

    //----------------------------------------------------------------
    // The Application Management inline service can return an
    // indication that the application is no longer needed.  (e.g. if
    // the workload has decreased such that some instances of the
    // application are no longer needed.) Only applies to servers.
    //
    // This method allows a Business Element to register to receive
    // these indications (which can still be ignored).
    //----------------------------------------------------------------
    void setTermSupported(bool f = true);

    //----------------------------------------------------------------
    // This method allows an application to prevent the intervention
    // of any Error Analysis Routines defined in the configuration.
    //----------------------------------------------------------------
    void setSuppressErrorProfile(bool f = true);

    //----------------------------------------------------------------
    // Set the ILS Data definition to be used with this request.
    //----------------------------------------------------------------
    void setILSData(const RomaILSData& ILSDataSet);

    //----------------------------------------------------------------
    // Get any inline service data related to this request.
    //----------------------------------------------------------------
    RomaILSData* getILSData() const;

    //----------------------------------------------------------------
    // Set a syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);

    //----------------------------------------------------------------
    // Set a subject.
    //----------------------------------------------------------------
    void setSubject(const RomaSubject&);

    //----------------------------------------------------------------
    // Set the retain indicator.  If true specified, this means that
    // the publication will be available for applications that
    // subscribe after the message has been published.
    //----------------------------------------------------------------
    void setRetain(bool f = true);


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Marshal the publication.
    // 
    // This method is intended to be overridden in derived classes.
    // The RomaClient::publish() method calls this method before sending
    // the request. If the method returns false, the publish() operation
    // doesn't send the request and returns an error.
    //---------------------------------------------------------------- 
    virtual bool marshalRequest();

    //-----------------------------------------------------------------
    // Default constructor.
    //-----------------------------------------------------------------
    RomaPublication();

    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplPublication* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOPBL_HPP__

