//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosbl.hpp,v 1.13 2003/02/21 22:48:07 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosbl.hpp,v $
// $Revision: 1.13 $
//--------------------------------------------------------------------
// The RomaStorable public interface.
//--------------------------------------------------------------------

#ifndef __KXCOSBL_HPP
#define __KXCOSBL_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcothm.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplStorable;


//--------------------------------------------------------------------
// The RomaStorable class.
//
// Applications use a RomaStorageQueue object to work with storage 
// queues (see the Roma OO Programmer's Reference). The objects which 
// are stored and retrieved are of type RomaStorable.
//
// Since RomaStorable objects can be both stored and retrieved, the
// RomaStorable class implements methods to both set and get the 
// various message attributes. (Contrast with ClientRequest which 
// can only be put, or ClientResponse which can only be received.)
//
// RomaStorable objects have a single data buffer (again, contrast
// with ClientRequest etc which have separate Msg and Tag buffers).
//--------------------------------------------------------------------
class KXCODLL RomaStorable : public RomaTypeBase, 
                             public RomaTraceable,
                             public RomaAllocator
{
public:
    //----------------------------------------------------------------
	// Default constructor.
    //----------------------------------------------------------------
    RomaStorable();

    //----------------------------------------------------------------
    // Construct with the specified data buffer.
    //
    // Parameters:
    //  [
    //   (in) data   - pointer to data buffer
    //   (in) length - length of (relevant) data in the buffer
    //  ]
    //----------------------------------------------------------------
    RomaStorable(const void* data, int length);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaStorable();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaStorable(const RomaStorable&);
    RomaStorable& operator = (const RomaStorable&);

    //----------------------------------------------------------------
    // Get/set the Roma Theme ID.
    //----------------------------------------------------------------
    const RomaTheme& theme() const;
    void setTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Get/set the Roma Message ID.
    //----------------------------------------------------------------
    const RomaMsgId& msgId() const;
    void setMsgId(const RomaMsgId& msgId);

    //----------------------------------------------------------------
    // Get/set the Roma Correlation ID.
    //----------------------------------------------------------------
    const RomaMsgId& correlId() const;
    void setCorrelId(const RomaMsgId& correlId);

    //----------------------------------------------------------------
    // Set the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority
    //----------------------------------------------------------------
    void setPriority(int priority);

    //----------------------------------------------------------------
    // Set the message persistence to one of ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //----------------------------------------------------------------
    void setPersistence(Roma::Persistence persistence);

    //----------------------------------------------------------------
    // Set the format number.
    //  If FORMAT_PROFILE is specified then format is retrieved from
    //  the API profile, defaulting to FORMAT_BINARY.
    //----------------------------------------------------------------
    void setFormat(int format);

    //----------------------------------------------------------------
    // Set the expiry interval.
    //  If EXPIRY_PROFILE is specified then interval is retrieved from
    //  the API profile, defaulting to EXPIRY_NEVER.
    //----------------------------------------------------------------
    void setExpiry(int expiry);

    //----------------------------------------------------------------
    // Set the codepage.
    //  If CODEPAGE_PROFILE is specified then codepage number is
    //  retrieved from the API profile, defaulting to CODEPAGE_USER
    //----------------------------------------------------------------
    void setCodepage(int codepage);

    //----------------------------------------------------------------
    // Set the encoding.
    //  If ENCODING_PROFILE is specified then encoding is
    //  retrieved from the API profile, defaulting to ENCODING_DEFAULT
    //----------------------------------------------------------------
    void setEncoding(int encoding);

    //----------------------------------------------------------------
    // Set the format name to the specified string and format name to
    // FORMAT_USENAME.
    //----------------------------------------------------------------
    void setFormatName(const char* name);

    //----------------------------------------------------------------
    // Set/get the message data buffer.
    //----------------------------------------------------------------
    bool setMsgBuffer(const void* buffer, int dataLength);
    void* msgBuffer() const;
    int msgDataLength() const;

    //----------------------------------------------------------------
    // Get the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority
    //----------------------------------------------------------------
    int priority() const;

    //----------------------------------------------------------------
    // Get the message persistence. Values can be ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //----------------------------------------------------------------
    Roma::Persistence persistence() const;

    //----------------------------------------------------------------
    // Get the format number.
    //----------------------------------------------------------------
    int format() const;

    //----------------------------------------------------------------
    // Get the expiry interval.
    //----------------------------------------------------------------
    int expiry() const;

    //----------------------------------------------------------------
    // Get the codepage.
    //----------------------------------------------------------------
    int codepage() const;

    //----------------------------------------------------------------
    // Get the encoding.
    //----------------------------------------------------------------
    int encoding() const;

    //----------------------------------------------------------------
    // Get the backout count.
    //----------------------------------------------------------------
    int backoutCount() const;

    //----------------------------------------------------------------
    // Get the format name.
    //----------------------------------------------------------------
    const char* formatName() const;

    //----------------------------------------------------------------
    // Get the message timestamp.
    //----------------------------------------------------------------
    RomaTimestamp timestamp() const;


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Marshal the storable.
    //
    // This method is intended to be overriden in derived classes.
    // The RomaStorageQueue::store() method calls this method before
    // storing the message. If the method returns false, the store()
    // operation doesn't store the message and returns an error.
    //----------------------------------------------------------------
    virtual bool marshalForStorage();

    //----------------------------------------------------------------
    // Constructor that allows to create the object using an existing
    // implementation instance.
    //----------------------------------------------------------------
    RomaStorable(RomaImplStorable*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplStorable* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOSBL_HPP

