//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosbs.hpp,v 1.11 2003/02/21 22:48:07 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosbs.hpp,v $
// $Revision: 1.11 $
//--------------------------------------------------------------------
// The RomaSubscription public interface.
//--------------------------------------------------------------------

#ifndef __KXCOSUBS_HPP
#define __KXCOSUBS_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcosyn.hpp>
#include <kxcosec.hpp>
#include <kxcomid.hpp>
#include <kxcoils.hpp>
#include <kxcosub.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplSubscription;


//--------------------------------------------------------------------
// The RomaSubscription class.
// 
// This class is used to subscribe to publications.
//
// Objects of this and derived classes are used by 
//  RomaClient::subscribe()
//  RomaClient::unsubscribe()
//  RomaClient::unsubscribeAll()
//  RomaClient::requestUpdates()
//--------------------------------------------------------------------
class KXCODLL RomaSubscription : public RomaTypeBase, 
                                 public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructs a subscription with the default options and the
    // specified service name and subject.
    // 
    // Parameters:
    //   (in) serviceName - a name of a business service.
    //   (in) subject     - a subject of a publication.
    //----------------------------------------------------------------
    RomaSubscription(const char* serviceName,
                     const RomaSubject& subject);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaSubscription();

    //----------------------------------------------------------------
    // Copy and assignment.
    //
    // Note: these methods don't create independent copy of the
    // message, but just share common implementation instance.
    //----------------------------------------------------------------
    RomaSubscription(const RomaSubscription&);
    RomaSubscription& operator = (const RomaSubscription&);

    //----------------------------------------------------------------
    // Set a business service name.
    // 
    // The service name must be defined before the object will be
    // used
    //----------------------------------------------------------------
    void setServiceName(const char* name);

    //----------------------------------------------------------------
    // Set theme options.
    //
    // If a new theme is requested, a new theme will be created for
    // each request. Otherwise the specified theme will be used.
    //  
    // If close theme is requested, the theme is closed after the
    // request is sent. If retain theme is requested, the theme is
    // retained for future use.
    //  
    // These selections may be overridden if an API profile is
    // defined.
    //----------------------------------------------------------------
    void setNewTheme(bool f = true);
    void setUseTheme(bool f = true);
    void setRetainTheme(bool f = true);
    void setCloseTheme(bool f = true);

    //----------------------------------------------------------------
    // Controls the type of message:
    //   true  - DATAGRAM
    //   false - REQUEST (default)
    // 
    // However, if an API profile is used (isApiProfileUsed() == true)
    // then either the default or the profile value is used.
    //----------------------------------------------------------------
    void setDatagram(bool f = true);

    //----------------------------------------------------------------
    // Set or clear the following flags:
    //    Confirmation of delivery (COD)
    //    Confirmation of arrival  (COA)
    //----------------------------------------------------------------
    void setCOD(bool f = true);
    void setCOA(bool f = true);

    //----------------------------------------------------------------
    // Set a security definition to be used with this request.
    //----------------------------------------------------------------
    void setSecurityDef(const RomaSecurityDef& secDef);

    //----------------------------------------------------------------
    // Set/get a Roma Theme ID.
    //----------------------------------------------------------------
    const RomaTheme& theme() const;
    void  setTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Set the message priority.
    //  A numeric value or one of the following constants:
    //  PRIORITY_PROFILE - use priority specified in API profile, or
    //  PRIORITY_DEFAULT - default transport priority
    //----------------------------------------------------------------
    void setPriority(int priority);

    //----------------------------------------------------------------
    // Set the message persistence to one of ...
    //  PERSISTENCE_YES     - persistent
    //  PERSISTENCE_NO      - not persistent
    //  PERSISTENCE_PROFILE - use value in the API profile, default NO
    //----------------------------------------------------------------
    void setPersistence(Roma::Persistence persistence);

    //-----------------------------------------------------------------
    // Set the feedback code.
    //  If FEEDBACK_PROFILE is specified then code is retrieved from
    //  the API profile, defaulting to FEEDBACK_NONE.
    //----------------------------------------------------------------
    void setFeedback(int feedback);

    //----------------------------------------------------------------
    // Set the expiry interval.
    //  If EXPIRY_PROFILE is specified then interval is retrieved from
    //  the API profile, defaulting to EXPIRY_NEVER.
    //----------------------------------------------------------------
    void setExpiry(int expiry);

    //-----------------------------------------------------------------
    // Set a message name.
    // Defines a message name passed on to the Application Management 
    // inline service. 
    //----------------------------------------------------------------
    void setMsgName(const char* name);

    //----------------------------------------------------------------
    // Set an API profile name. Supply null name to suppress use.
    //----------------------------------------------------------------
    void setApiProfileName(const char* name);

    //----------------------------------------------------------------
    // Sets an error profile name.
    //----------------------------------------------------------------
    void setErrorProfileName(const char* name);

    //----------------------------------------------------------------
    // If true specified, Roma does not put the message to an
    // exception message service on an error.
    //----------------------------------------------------------------
    void setNoEms(bool f = true);

    //----------------------------------------------------------------
    // The Application Management inline service can return an
    // indication that the application is no longer needed.  (e.g. if
    // the workload has decreased such that some instances of the
    // application are no longer needed.) Only applies to servers.
    //
    // This method allows a Business Element to register to receive
    // these indications (which can still be ignored).
    //----------------------------------------------------------------
    void setTermSupported(bool f = true);

    //----------------------------------------------------------------
    // This method allows an application to prevent the intervention
    // of any Error Analysis Routines defined in the configuration.
    //----------------------------------------------------------------
    void setSuppressErrorProfile(bool f = true);

    //----------------------------------------------------------------
    // Set the ILS Data definition to be used with this request.
    //----------------------------------------------------------------
    void setILSData(const RomaILSData& data);

    //----------------------------------------------------------------
    // Return any inline service data related to this request.
    //----------------------------------------------------------------
    RomaILSData* getILSData() const;

    //----------------------------------------------------------------
    // Sets a syncpoint definition.
    //----------------------------------------------------------------
    void setSyncpointDef(const RomaSyncpointDef& def);

    //----------------------------------------------------------------
    // Set a subject.
    //----------------------------------------------------------------
    void setSubject(const RomaSubject&);

    //----------------------------------------------------------------
    // If set to true, this means that the subscriber will only
    // receive messages that are published after the broker receives
    // this subscription. It is set to false by default.
    //----------------------------------------------------------------
    void setNewPubsOnly(bool f = true);

    //----------------------------------------------------------------
    // If set to true, the application will not receive any
    // publications until it requests them with the
    // RomaClient::requestUpdates() method. It is set to false by
    // default.
    //----------------------------------------------------------------
    void setOnRequest(bool f = true);


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //---------------------------------------------------------------- 
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaSubscription();

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplSubscription* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOSUBS_HPP

