//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcostq.hpp,v 1.11 2003/02/21 22:48:08 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcostq.hpp,v $
// $Revision: 1.11 $
//--------------------------------------------------------------------
// The RomaStorageQueue public interface.
//--------------------------------------------------------------------

#ifndef __KXCOSTQ_HPP
#define __KXCOSTQ_HPP

#include <kxcocl.hpp>
#include <kxcobe.hpp>
#include <kxcoerr.hpp>
#include <kxcotb.hpp>
#include <kxcosbl.hpp>
#include <kxcoso.hpp>
#include <kxcorvo.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplStorageQueue;


//--------------------------------------------------------------------
// The RomaStorageQueue class.
//
// This class supports store and retrieval of RomaStorable objects.
//--------------------------------------------------------------------
class KXCODLL RomaStorageQueue : public RomaTypeBase, 
                                 public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Construct and open a storage queue supplying a connected
    // Roma client or business element.
    //----------------------------------------------------------------
    RomaStorageQueue(RomaClient& client,
                     const char* name,
                     KXCULONG openOptions = Roma::OPEN_DEFAULT,
                     const char* modelQueueName = NULL);

    RomaStorageQueue(RomaBusinessElement& be,
                     const char* name,
                     KXCULONG openOptions = Roma::OPEN_DEFAULT,
                     const char* modelQueueName = NULL);

    //----------------------------------------------------------------
    // Destructor. Closes the queue.
    //----------------------------------------------------------------
    virtual ~RomaStorageQueue();

    //----------------------------------------------------------------
    // Open a storage queue.
    //
    // Parameters:
    //   (in) context - Client or Business Element
    //        The queue can only be accessed while this context is
    //        valid.
    //   (in) name    - the queue name
    //   (in) options - OPEN_DEFAULT, or a combination of:
    //        OPEN_NON_XACTIONAL   - MSMQ queue created as non 
    //                               transactional
    //        OPEN_EXCLUSIVE       - queue opened for exclusive 
    //                               access
    //        OPEN_DELETE_ON_CLOSE - queue deleted on close
    //        OPEN_MUST_EXIST      - specified queue must already 
    //                               exist
    //        OPEN_OUTPUT          - open for store
    //        OPEN_INPUT           - open for retrieve
    //        OPEN_PEEK            - non-destructive operations 
    //                               allowed
    //
    //        OPEN_DEFAULT is OPEN_INPUT|OPEN_OUTPUT|OPEN_PEEK
    //----------------------------------------------------------------
    bool open(RomaClient& client,
              const char* name,
              KXCULONG openOptions = Roma::OPEN_DEFAULT,
              const char* modelQueueName = NULL);

    bool open(RomaBusinessElement& be,
              const char* name,
              KXCULONG openOptions = Roma::OPEN_DEFAULT,
              const char* modelQueueName = NULL);

    //----------------------------------------------------------------
    // Returns true if the storage queue is open.
    //----------------------------------------------------------------
    bool isOpen() const;

    //----------------------------------------------------------------
    // Close the storage queue.
    //----------------------------------------------------------------
    bool close();

    //----------------------------------------------------------------
    // Store a message on the queue.
    //
    // A RomaStorable object provides options and data to be stored.
    // The virtual RomaStorable::marshalToStorage() method is called
    // before putting the message.
    //
    // Parameters:
    //   (in/out) message
    //
    // Output attributes of the message are:
    //   - RomaMessage::msgId()
    //       set to generated msg id if USE_MSGID not specified
    //   - RomaMessage::correlId()
    //       reset to all 0 if USE_CORRELID is not specified
    //   - RomaMessage::timestamp()
    //
    // Returns:
    //   false - error, use RomaError methods to get error information
    //   true - success
    //----------------------------------------------------------------
    bool store(RomaStorable& message,
          const RomaStoreOptions& storeOptions = RomaStoreOptions());

    //----------------------------------------------------------------
    // Retrieve a message from the queue.
    //
    // Retrieve RomaStorable according to the specified get options.
    // Calls createStorable() to unmarshal retrieved message.
    // Default implementation of the createStorable() creates 
    // instance of RomaStorable. Overriden methods can return objects
    // of classes derived from RomaStorable.
    //
    // To safely cast returned pointer use ROMA_SAFE_PTR_CAST() macro.
    //
    // Parameters:
    //   (in) getOptions
    //
    // Returns:
    //   0      - an error or timeout occurred, else ...
    //          - pointer to a storable object and buffer allocated
    //            by Roma.
    //----------------------------------------------------------------
    RomaStorable* retrieve(
        const RomaRetrieveOptions& opt = RomaRetrieveOptions());
    RomaStorable* retrieve(
        const RomaRetrieveOptions& opt, void* buffer, int& len);


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Unmarshals the retrieved message.
    //
    // This method is intended for overriding in derived classes.
    // Overriden method should assign "newStorable" parameter with 
    // the pointer to the object of the RomaStorable or derived class, 
    // created by new. If overriden method doesn't set "storable" 
    // parameter or set it to 0 then get() method sets 
    // KXORC_CREATE_STORABLE error.
    //
    // Parameters:
    //   (in) message - retrieved message
    //   (out) storable - marshaled storable object
    //
    // Returns:
    //   true  - success
    //   false - fail, get() method will set KXCORC_CREATE_STORABLE
    //           error.
    //
    // The default implementation does nothing.
    //----------------------------------------------------------------
    virtual bool createStorable(const RomaStorable& baseStorable,
                                RomaStorable*& newStorable);

    //----------------------------------------------------------------
    // Default constructor. Creates unconnected object.
    //----------------------------------------------------------------
    RomaStorageQueue();

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplStorageQueue* m_impl;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaStorageQueue(const RomaStorageQueue&);
    RomaStorageQueue& operator = (const RomaStorageQueue&);

    friend class RomaImplProxy;
};


#endif // __KXCOSTQ_HPP

