//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcoutil.hpp
//
// Declares     : Includes ...
//                 a simple string class
//                 a single Linked List implementation
//
// Description  : The list handles a pointer to the user data object
//
//                Iterators are provided to traverse the lists in a
//                forward direction, and a Splice list iterator allows
//                insertion and deletion of entries while iterating.
//
// Notes        : The construction and destruction of the list
//                elements only affect the list objects not the user
//                data objects
//              : Access methods are provided that return the user
//                data pointer directly, in which case it is not
//                possible to distinguish out of bounds or a Null user
//                data entry mid list.
//                Thus an additional set of bool access methods are
//                also provided to allow this distinction
//
// Usage        : Macros are provided to allow "type"ing of the list
//                user data by creating a derived class with
//                appropriate casting,
//                See KLINKED_LIST_OF The new list class will be named
//                <type>LList
// Example      :
//                    class Foo { ..... };
//                    KLINKED_LIST_OF(Foo)
//
//                    FooLList FoosToDo;
//                    FoosToDo.Append(MyFoo);
//
//                Another macro KLINKED_LIST_OF_WITH(type, extra) can
//                be used to setup lists of typed objects based on
//                LList.
//	      	      The "extra" parameters allows the user to add a
//                special method(s) and or data member(s) to the
//                public section of the new class
//
//--------------------------------------------------------------------
// $Id: kxcoutil.hpp,v 1.8 2004/02/18 16:35:08 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoutil.hpp,v $
// $Revision: 1.8 $
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Protect against multiple includes
//--------------------------------------------------------------------
#ifndef __KXCOUTIL_H
#define __KXCOUTIL_H

//--------------------------------------------------------------------
// KXC Includes
//--------------------------------------------------------------------
#include <stdio.h>
#include <kxco.hpp>
#include <kxc.h>

//--------------------------------------------------------------------
// The list manages a pointer to the user supplied object
// by default this managed as a void *
//--------------------------------------------------------------------
#define LGENERICTYPE void

//--------------------------------------------------------------------
// Class predeclarations
//--------------------------------------------------------------------
class kxcSpliceLListIterator;
class kxcLListEnt;


//--------------------------------------------------------------------
// Class		    : kLinkedList
// Description	: Encapsulation of a singly linked list
//--------------------------------------------------------------------
class KXCODLL kLinkedList {
public:
  //------------------------------------------------------------------
  // Construct an empty list
  //------------------------------------------------------------------
  kLinkedList();

  //------------------------------------------------------------------
  // Construct a full new copy of an existing list
  //------------------------------------------------------------------
  kLinkedList(const kLinkedList &);

  //------------------------------------------------------------------
  // Destruct a list - deletion of user data is users responsibilty
  //------------------------------------------------------------------
  virtual ~kLinkedList();

  //------------------------------------------------------------------
  // Prevent compiler bitwise copy as list items must be got with new
  //------------------------------------------------------------------
  const kLinkedList& operator = (const kLinkedList &_CopyList);

  //------------------------------------------------------------------
  // Get the current length of the list
  //------------------------------------------------------------------
  int Length() const;

  //------------------------------------------------------------------
  // Adding elements to the ends of the list
  //------------------------------------------------------------------
  void Append(LGENERICTYPE *);
  void Prepend(LGENERICTYPE *);

  //------------------------------------------------------------------
  // Removing elements from the ends of the list
  // Returns a pointer to the contained user data and deletes the
  // entry. Returns Null if no entry.
  // Note: If entry had a null user data pointer could not tell if
  // an entry was deleted
  //------------------------------------------------------------------
  LGENERICTYPE * ExtractFirst();
  LGENERICTYPE * ExtractLast();

  //------------------------------------------------------------------
  // Removing elements from the ends of the list
  // Returns bool if entry exists and a pointer to the contained
  // user data from a deleted entry.
  //------------------------------------------------------------------
  bool ExtractFirst(LGENERICTYPE **);
  bool ExtractLast(LGENERICTYPE **);

  //------------------------------------------------------------------
  // Accessing elements of the list
  // Get at a specified index, if index out of range returns NULL
  // Note: If user data can be NULL then cannot tell if index out
  //       of range
  //------------------------------------------------------------------
  LGENERICTYPE * GetAtIndex(int) const;

  //------------------------------------------------------------------
  // Get at specified index and use bool return to identify if
  // index was out of range
  //------------------------------------------------------------------
  bool GetAtIndex(int, LGENERICTYPE **) const;

  //------------------------------------------------------------------
  // Clear all list entries, user is responsible for user objects
  // destruction
  //------------------------------------------------------------------
  void Clear(void);

protected:
  int m_len;
  //------------------------------------------------------------------
  // The head of the List
  //------------------------------------------------------------------
  kxcLListEnt *m_pHead;

private:
  friend class kxcSpliceLListIterator;
  friend class kxcLListIterator;
};


//--------------------------------------------------------------------
// Class	     	: kxcLListIterator
// Description	: Iterate down a linked list
//--------------------------------------------------------------------
class KXCODLL kxcLListIterator
{
public:
  kxcLListIterator(const kLinkedList &);

  //------------------------------------------------------------------
  // Iterate to next returning EntryData or Null for end of list
  // Note: If an entry has a null m_pEntryData then user will assume
  //       they have hit end of list
  //------------------------------------------------------------------
  LGENERICTYPE * GetNext(void);

  //------------------------------------------------------------------
  // Iterate to next returning false for end of list
  //------------------------------------------------------------------
  bool GetNext(LGENERICTYPE **);

  //------------------------------------------------------------------
  // Reset iterator
  //------------------------------------------------------------------
  void Reset();

private:
  //------------------------------------------------------------------
  // Store the head of the list we are iterating for resets
  //------------------------------------------------------------------
  kxcLListEnt *m_pListHead;

  //------------------------------------------------------------------
  // Store where we have iterated too, each iteration will
  // return the UserData from here and move the iterator on one
  //------------------------------------------------------------------
  kxcLListEnt *m_pListEntIterator;
};


//--------------------------------------------------------------------
// Class	     	: kxcSpliceLListIterator
//
// Description	: Iterate down a linked list, and provide methods
//			      	  to insert and delete
//--------------------------------------------------------------------
class KXCODLL kxcSpliceLListIterator
{
public:
  kxcSpliceLListIterator(kLinkedList &);

  //------------------------------------------------------------------
  // Iterate to next returning EntryData or Null for end of list
  // Note: If an entry has a null m_pEntryData then user will assume
  //       they have hit end of list
  //------------------------------------------------------------------
  LGENERICTYPE * GetNext();

  //------------------------------------------------------------------
  // Iterate to next returning false for end of list
  //------------------------------------------------------------------
  bool GetNext(LGENERICTYPE **);

  //------------------------------------------------------------------
  // Reset iterator
  //------------------------------------------------------------------
  void Reset(void);

  //------------------------------------------------------------------
  // Splice after current
  // Note: Multiple uses of this call without iterating
  // inbetween inserts after current but before previous insert
  //------------------------------------------------------------------
  void InsertAfter(LGENERICTYPE *);

  //------------------------------------------------------------------
  // Splice before
  //------------------------------------------------------------------
  void InsertBefore(LGENERICTYPE *);

  //------------------------------------------------------------------
  // Remove current element
  //------------------------------------------------------------------
  LGENERICTYPE * RemoveCurrent();

private:
  //------------------------------------------------------------------
  // Entry we have iterated too
  //------------------------------------------------------------------
  kxcLListEnt *m_pEntIterator;

  //------------------------------------------------------------------
  // Points to link into
  //------------------------------------------------------------------
  kxcLListEnt **m_ppSpliceBeforeLink;
  kxcLListEnt **m_ppSpliceAfterLink;

  //------------------------------------------------------------------
  // The list we are iterating
  //------------------------------------------------------------------
  kLinkedList *m_pList;
};


//--------------------------------------------------------------------
// Macro		    : ITERATORS_OF(type)
// Description	: Part of folowing KXC_LLIST_CLASS_ macros
//--------------------------------------------------------------------
#define ITERATORS_OF(type) \
class type ## LListIterator : public kxcLListIterator \
{\
public:\
  type ## LListIterator(const type ## LList &l) \
    : kxcLListIterator(l) {};\
  type * GetNext(void) \
    { return (type *)kxcLListIterator::GetNext(); };\
  bool GetNext(type **ppe) \
   { return kxcLListIterator::GetNext((LGENERICTYPE **)ppe); };\
};\
class type ## SpliceLListIterator : public kxcSpliceLListIterator \
{\
public:\
  type ## SpliceLListIterator(type ## LList &l) \
    : kxcSpliceLListIterator(l) {};\
  type * GetNext(void) \
    { return (type *)kxcSpliceLListIterator::GetNext(); };\
  bool GetNext(type **ppe) \
    { return kxcSpliceLListIterator::GetNext((LGENERICTYPE **)ppe); };\
  void InsertAfter(type *ppe) \
    { kxcSpliceLListIterator::InsertAfter((LGENERICTYPE *)ppe); };\
  void InsertBefore(type *ppe) \
    { kxcSpliceLListIterator::InsertBefore((LGENERICTYPE *)ppe); };\
  type * RemoveCurrent() \
    { return (type *)kxcSpliceLListIterator::RemoveCurrent(); };\
};

//--------------------------------------------------------------------
// Macro		    : KLINKED_LIST_OF(type)
// Description	: Use this macro to setup lists of typed objects
//                based on kLinkedList
//                The new list class will be named <type>LList
// e.g
// class Foo { ..... };
// KLINKED_LIST_OF(Foo)
// .....
// FooLList FoosToDo;
// FoosToDo.Append(MyFoo);
//--------------------------------------------------------------------
#define KLINKED_LIST_OF(type)	\
class KXCODLL type ## LList : public kLinkedList \
{\
public:\
  type ## LList() : kLinkedList() {};\
  type * ExtractFirst(void) \
    { return (type *)kLinkedList::ExtractFirst(); };\
  type * ExtractLast(void) \
    { return (type *)kLinkedList::ExtractLast(); };\
  bool ExtractFirst(type **ppe) \
    { return kLinkedList::ExtractFirst((LGENERICTYPE **)ppe); };\
  bool ExtractLast(type **ppe) \
    { return kLinkedList::ExtractLast((LGENERICTYPE **)ppe); };\
  type * GetAtIndex(int i) const \
    { return (type *)kLinkedList::GetAtIndex(i); };\
  bool GetAtIndex(int i, type **ppe) const \
    { return kLinkedList::GetAtIndex(i, (LGENERICTYPE **)ppe); };\
};\
ITERATORS_OF(type)


//--------------------------------------------------------------------
// Macro	      : KLINKED_LIST_OF_WITH(type, extra)
// Description	: Use this macro to setup lists of typed objects based
//                on kLinkedList
//	      	  The "extra" parameters allows the user to add a
//                special method(s) and or data member(s) to the
//                public section of the new class
// e.g
// class Foo { ..... };
// KLINKED_LIST_OF_WITH(Foo, void DumpAll(); )
// .....
// FooLList::DumpAll() { ... };
// .....
// SomeFunc()
// {
// FooLList FoosToDo;
// .....
// Foostodo.DumpAll();
// }
//--------------------------------------------------------------------
#define KLINKED_LIST_OF_WITH(type, extrastuff) \
class type ## LList : public kLinkedList \
{\
public:\
  type ## LList() : kLinkedList() {};\
  type * ExtractFirst(void) \
    { return (type *)kLinkedList::ExtractFirst(); };\
  type * ExtractLast(void) \
    { return (type *)kLinkedList::ExtractLast(); };\
  bool ExtractFirst(type **ppe) \
    { return kLinkedList::ExtractFirst((LGENERICTYPE **)ppe); };\
  bool ExtractLast(type **ppe) \
    { return kLinkedList::ExtractLast((LGENERICTYPE **)ppe); };\
  type * GetAtIndex(int i) const \
    { return (type *)kLinkedList::GetAtIndex(i); };\
  bool GetAtIndex(int i, type **ppe) const \
    { return kLinkedList::GetAtIndex(i, (LGENERICTYPE **)ppe); };\
extrastuff\
};\
ITERATORS_OF(type)


//--------------------------------------------------------------------
// Class        : kxcLListEnt
// Description	: An entry of the Linked List
// Note         : Internal to the kLinkedList class
//--------------------------------------------------------------------
class kxcLListEnt
{
protected:
  //------------------------------------------------------------------
  // Constructor
  //------------------------------------------------------------------
  kxcLListEnt(LGENERICTYPE *pd, kxcLListEnt *pn = 0)
    : m_pEntryData(pd), m_pNext(pn) {};

  //------------------------------------------------------------------
  // Copy Constructor
  //------------------------------------------------------------------
  kxcLListEnt(const kxcLListEnt &e)	
    : m_pEntryData(e.m_pEntryData), m_pNext(e.m_pNext) {};

  //------------------------------------------------------------------
  // Destructor - user data deletion is the users responsibility
  //------------------------------------------------------------------
  ~kxcLListEnt() {};

  //------------------------------------------------------------------
  // The users data pointer
  //------------------------------------------------------------------
  LGENERICTYPE *m_pEntryData;

  //------------------------------------------------------------------
  // The Link to the next entry
  //------------------------------------------------------------------
  kxcLListEnt *m_pNext;
  friend class kLinkedList;
  friend class kxcSpliceLListIterator;
  friend class kxcLListIterator;
};


//--------------------------------------------------------------------
//
// Class        : kString
// Description	: Simple string class.
// Notes        : Implements some of the core MFC CString functionalty
//                Uses new,delete to allocate string space.
//
//--------------------------------------------------------------------
class KXCODLL kString
{
public :
  // Default constructor - create empty string
  kString ();

  // Fill with copy of another string
  kString (const kString &str);

  // Fill with copy of character string
  kString (const char *init);

  // Fill with leftmost 'len' characters
  kString (const char *init, int len);

  // Fill with same character
  kString (char ch, int nRepeat);

  virtual ~kString ();

  // Get at contents as a constant character string
  operator const char*() const;

  // Fairly commonly used to assign a quoted string into it
  // this is why it is an inline
  const kString& operator = (const char *Str);
  // assign contents from another kString
  const kString& operator = (const kString &_CopyStr);

  // Compare
  bool operator ==( const char * _pCompareStr) const;
  bool operator !=( const char * _pComapreStr) const;

  // Concatenate
  const kString& operator +=( const char * _pStr );

  // Concatenate with other string types
  friend kString operator +( const kString &_Str1, const kString &_Str2);
  friend kString operator +( const kString &_Str1, const char * _pStr2);
  friend kString operator +( const char * _pStr1, const kString& _Str2);

  // Remove string memory
  void Empty ();

  // Access an individual character
  char GetAt (int _Ind) const {
    return (m_pStr != NULL && _Ind >=0 && _Ind < m_Len) ? m_pStr [_Ind] : '\0';
  }

  // Check if empty - same as no memory allocated
  bool IsEmpty () const { return m_Len == 0; }

  // Find current length - N.B. does not use strlen !
  int GetLength () const {return m_Len;}

  // Extra one to allow formatting from a printf specifier
  // NOTE Assumes maximum formatted string of 1024 characters
  void Printf (const char *Fmt, ...);

protected :
  // Shared utility to concaternate strings
  void Concatenate(const char *_pStr1, int Len1, const char *_pStr2, int _Len2);

  // Pointer to string data, NULL if string empty
  char *m_pStr;
  // The length of the string
  int m_Len;
};


//--------------------------------------------------------------------
// Utility routines.
//--------------------------------------------------------------------
class RomaUtil
{
public:
    //----------------------------------------------------------------
    // Sets a variable-size string to a fixed-size string.
    //----------------------------------------------------------------
    static void setString(KXCCHAR dst[], const char* src, size_t sz)
    {
        if (src == NULL) 
        {
            memset(dst, 0, sz);
            return;
        }

        strncpy((char*)dst, src, sz);
        dst[sz - 1] = 0;
    }
};


#endif
